@@ -112,7 +112,7 @@ PIXAC   *pixac1, *pixac2, *pixac3;
     pixac3 = pixacompInterleave(pixac1, pixac2);
     pixa2 = pixaCreateFromPixacomp(pixac3, L_CLONE);
     pixa3 = pixaConvertToNUpPixa(pixa2, NULL, 1, 2, 1000, 6, 2, 0);
-    fprintf(stderr, "Time with pixac interleaving = %7.3f sec\n", stopTimer());
+    lept_stderr("Time with pixac interleaving = %7.3f sec\n", stopTimer());
     L_INFO("Writing to /tmp/lept/adapt/cleaning2.pdf\n", procName);
     pixaConvertToPdf(pixa3, 100, 1.0, L_JPEG_ENCODE,
                      75, "Adaptive cleaning", "/tmp/lept/adapt/cleaning2.pdf");
@@ -135,7 +135,7 @@ PIXAC   *pixac1, *pixac2, *pixac3;
     pixa2 = pixaCopy(pixa1, L_COPY_CLONE);
     pixa3 = pixaInterleave(pixa1, pixa2, L_CLONE);
     pixa4 = pixaConvertToNUpPixa(pixa3, NULL, 1, 2, 1000, 6, 2, 0);
-    fprintf(stderr, "Time with pixa interleaving = %7.3f sec\n", stopTimer());
+    lept_stderr("Time with pixa interleaving = %7.3f sec\n", stopTimer());
     L_INFO("Writing to /tmp/lept/adapt/cleaning3.pdf\n", procName);
     pixaConvertToPdf(pixa4, 100, 1.0, L_JPEG_ENCODE,
                      75, "Adaptive cleaning", "/tmp/lept/adapt/cleaning3.pdf");
@@ -166,7 +166,7 @@ PIX     *pix1, *pix2, *pix3, *pix4, *pix5;
 
     pix2 = pixBackgroundNorm(pix1, NULL, NULL, 10, 15, thresh, 25, 200, 2, 1);
     snprintf(buf, sizeof(buf), "Norm color: fg thresh = %d", thresh);
-    fprintf(stderr, "%s\n", buf);
+    lept_stderr("%s\n", buf);
     pix3 = pixAddTextlines(pix2, bmf, buf, 0x00ff0000, L_ADD_BELOW);
     snprintf(buf, sizeof(buf), "/tmp/lept/adapt/adapt_%03d.jpg", index++);
     pixWrite(buf, pix3, IFF_JFIF_JPEG);
