@@ -62,7 +62,7 @@ l_int32  ret;
 char    *dirin, *substr, *fileout;
 
     if (argc != 3 && argc != 4) {
-        fprintf(stderr,
+        lept_stderr(
             " Syntax: converttopdf dir [substr] fileout\n"
             "         substr:  Leave this out to bundle all files\n"
             "         fileout:  Output pdf file\n");
