@@ -50,20 +50,23 @@
 #include <config_auto.h>
 #endif /* HAVE_CONFIG_H */
 
-#define   FILE_1BPP          "rabi.png"
-#define   FILE_2BPP          "speckle2.png"
-#define   FILE_2BPP_C        "weasel2.4g.png"
-#define   FILE_4BPP          "speckle4.png"
-#define   FILE_4BPP_C        "weasel4.16c.png"
-#define   FILE_8BPP          "dreyfus8.png"
-#define   FILE_8BPP_C        "weasel8.240c.png"
-#define   FILE_16BPP         "test16.png"
-#define   FILE_32BPP         "weasel32.png"
-#define   FILE_32BPP_ALPHA   "test32-alpha.png"
-#define   FILE_CMAP_ALPHA    "test-cmap-alpha.png"
-#define   FILE_CMAP_ALPHA2   "test-cmap-alpha2.png"
-#define   FILE_TRANS_ALPHA   "test-fulltrans-alpha.png"
-#define   FILE_GRAY_ALPHA    "test-gray-alpha.png"
+#define   FILE_1BPP             "rabi.png"
+#define   FILE_2BPP             "speckle2.png"
+#define   FILE_2BPP_C           "weasel2.4g.png"
+#define   FILE_4BPP             "speckle4.png"
+#define   FILE_4BPP_C           "weasel4.16c.png"
+#define   FILE_8BPP             "dreyfus8.png"
+#define   FILE_8BPP_C           "weasel8.240c.png"
+#define   FILE_16BPP            "test16.png"
+#define   FILE_32BPP            "weasel32.png"
+#define   FILE_32BPP_ALPHA      "test32-alpha.png"
+#define   FILE_CMAP_ALPHA       "test-cmap-alpha.png"
+#define   FILE_CMAP_ALPHA2      "test-cmap-alpha2.png"
+#define   FILE_TRANS_ALPHA      "test-fulltrans-alpha.png"
+#define   FILE_GRAY_ALPHA       "test-gray-alpha.png"
+#define   FILE_TRANS_CMAP_2BPP  "trans-2bpp-cmap.png"
+#define   FILE_TRANS_CMAP_4BPP  "trans-4bpp-cmap.png"
+#define   FILE_TRANS_CMAP_8BPP  "trans-8bpp-cmap.png"
 
 static l_int32 test_mem_png(const char *fname);
 static l_int32 get_header_data(const char *filename);
@@ -72,6 +75,9 @@ static l_int32 test_1bpp_color(L_REGPARAMS *rp);
 static l_int32 test_1bpp_gray(L_REGPARAMS *rp);
 static l_int32 test_1bpp_bw1(L_REGPARAMS *rp);
 static l_int32 test_1bpp_bw2(L_REGPARAMS *rp);
+static l_int32 test_2bpp_cmap_trans(L_REGPARAMS  *rp);
+static l_int32 test_4bpp_cmap_trans(L_REGPARAMS  *rp);
+static l_int32 test_8bpp_cmap_trans(L_REGPARAMS  *rp);
 static l_int32 test_8bpp_trans(L_REGPARAMS  *rp);
 
 LEPT_DLL extern const char *ImageFileFormatExtensions[];
@@ -94,39 +100,45 @@ L_REGPARAMS  *rp;
 
     failure = FALSE;
     success = TRUE;
-    fprintf(stderr, "Test bmp 1 bpp file:\n");
+    lept_stderr("Test bmp 1 bpp file:\n");
     if (ioFormatTest(FILE_1BPP)) success = FALSE;
-    fprintf(stderr, "\nTest 2 bpp file:\n");
+    lept_stderr("\nTest 2 bpp file:\n");
     if (ioFormatTest(FILE_2BPP)) success = FALSE;
-    fprintf(stderr, "\nTest 2 bpp file with cmap:\n");
+    lept_stderr("\nTest 2 bpp file with cmap:\n");
     if (ioFormatTest(FILE_2BPP_C)) success = FALSE;
-    fprintf(stderr, "\nTest 4 bpp file:\n");
+    lept_stderr("\nTest 4 bpp file:\n");
     if (ioFormatTest(FILE_4BPP)) success = FALSE;
-    fprintf(stderr, "\nTest 4 bpp file with cmap:\n");
+    lept_stderr("\nTest 4 bpp file with cmap:\n");
     if (ioFormatTest(FILE_4BPP_C)) success = FALSE;
-    fprintf(stderr, "\nTest 8 bpp grayscale file with cmap:\n");
+    lept_stderr("\nTest 8 bpp grayscale file with cmap:\n");
     if (ioFormatTest(FILE_8BPP)) success = FALSE;
-    fprintf(stderr, "\nTest 8 bpp color file with cmap:\n");
+    lept_stderr("\nTest 8 bpp color file with cmap:\n");
     if (ioFormatTest(FILE_8BPP_C)) success = FALSE;
-    fprintf(stderr, "\nTest 16 bpp file:\n");
+    lept_stderr("\nTest 16 bpp file:\n");
     if (ioFormatTest(FILE_16BPP)) success = FALSE;
-    fprintf(stderr, "\nTest 32 bpp RGB file:\n");
+    lept_stderr("\nTest 32 bpp RGB file:\n");
     if (ioFormatTest(FILE_32BPP)) success = FALSE;
-    fprintf(stderr, "\nTest 32 bpp RGBA file:\n");
+    lept_stderr("\nTest 32 bpp RGBA file:\n");
     if (ioFormatTest(FILE_32BPP_ALPHA)) success = FALSE;
-    fprintf(stderr, "\nTest spp = 1, cmap with alpha file:\n");
+    lept_stderr("\nTest spp = 1, cmap with alpha file:\n");
     if (ioFormatTest(FILE_CMAP_ALPHA)) success = FALSE;
-    fprintf(stderr, "\nTest spp = 1, cmap with alpha (small alpha array):\n");
+    lept_stderr("\nTest spp = 1, cmap with alpha (small alpha array):\n");
     if (ioFormatTest(FILE_CMAP_ALPHA2)) success = FALSE;
-    fprintf(stderr, "\nTest spp = 1, fully transparent with alpha file:\n");
+    lept_stderr("\nTest spp = 1, fully transparent with alpha file:\n");
     if (ioFormatTest(FILE_TRANS_ALPHA)) success = FALSE;
-    fprintf(stderr, "\nTest spp = 2, gray with alpha file:\n");
+    lept_stderr("\nTest spp = 2, gray with alpha file:\n");
     if (ioFormatTest(FILE_GRAY_ALPHA)) success = FALSE;
+    lept_stderr("\nTest spp = 2, cmap with alpha file:\n");
+    if (ioFormatTest(FILE_TRANS_CMAP_2BPP)) success = FALSE;
+    lept_stderr("\nTest spp = 4, cmap with alpha file:\n");
+    if (ioFormatTest(FILE_TRANS_CMAP_4BPP)) success = FALSE;
+    lept_stderr("\nTest spp = 8, cmap with alpha file:\n");
+    if (ioFormatTest(FILE_TRANS_CMAP_8BPP)) success = FALSE;
     if (success) {
-        fprintf(stderr,
+        lept_stderr(
             "\n  ********** Success on lossless r/w to file *********\n\n");
     } else {
-        fprintf(stderr,
+        lept_stderr(
             "\n  ******* Failure on at least one r/w to file ******\n\n");
     }
     if (!success) failure = TRUE;
@@ -147,30 +159,36 @@ L_REGPARAMS  *rp;
     if (test_mem_png(FILE_CMAP_ALPHA2)) success = FALSE;
     if (test_mem_png(FILE_TRANS_ALPHA)) success = FALSE;
     if (test_mem_png(FILE_GRAY_ALPHA)) success = FALSE;
+    if (test_mem_png(FILE_TRANS_CMAP_2BPP)) success = FALSE;
+    if (test_mem_png(FILE_TRANS_CMAP_4BPP)) success = FALSE;
+    if (test_mem_png(FILE_TRANS_CMAP_8BPP)) success = FALSE;
     if (success) {
-        fprintf(stderr,
+        lept_stderr(
             "\n  ****** Success on lossless r/w to memory *****\n");
     } else {
-        fprintf(stderr,
+        lept_stderr(
             "\n  ******* Failure on at least one r/w to memory ******\n");
     }
     if (!success) failure = TRUE;
 
-    /* ------------ Part 3: Test lossless 1 and 8 bpp r/w ------------ */
-    fprintf(stderr, "\nTest lossless 1 and 8 bpp r/w\n");
+    /* ------------ Part 3: Test lossless 1, 2, 4 and 8 bpp r/w ------------ */
+    lept_stderr("\nTest lossless 1 and 8 bpp r/w\n");
     success = TRUE;
     if (test_1bpp_trans(rp) == 0) success = FALSE;
     if (test_1bpp_color(rp) == 0) success = FALSE;
     if (test_1bpp_gray(rp) == 0) success = FALSE;
     if (test_1bpp_bw1(rp) == 0) success = FALSE;
     if (test_1bpp_bw2(rp) == 0) success = FALSE;
+    if (test_2bpp_cmap_trans(rp) == 0) success = FALSE;
+    if (test_4bpp_cmap_trans(rp) == 0) success = FALSE;
+    if (test_8bpp_cmap_trans(rp) == 0) success = FALSE;
     if (test_8bpp_trans(rp) == 0) success = FALSE;
 
     if (success) {
-        fprintf(stderr,
+        lept_stderr(
             "\n  ******* Success on 1 and 8 bpp lossless *******\n\n");
     } else {
-        fprintf(stderr,
+        lept_stderr(
             "\n  ******* Failure on 1 and 8 bpp lossless *******\n\n");
     }
     if (!success) failure = TRUE;
@@ -191,22 +209,20 @@ L_REGPARAMS  *rp;
     if (get_header_data(FILE_CMAP_ALPHA2)) success = FALSE;
     if (get_header_data(FILE_TRANS_ALPHA)) success = FALSE;
     if (get_header_data(FILE_GRAY_ALPHA)) success = FALSE;
-
+    if (get_header_data(FILE_TRANS_CMAP_2BPP)) success = FALSE;
+    if (get_header_data(FILE_TRANS_CMAP_4BPP)) success = FALSE;
+    if (get_header_data(FILE_TRANS_CMAP_8BPP)) success = FALSE;
     if (success) {
-        fprintf(stderr,
-            "\n  ******* Success on reading headers *******\n\n");
+        lept_stderr("\n  ******* Success on reading headers *******\n\n");
     } else {
-        fprintf(stderr,
-            "\n  ******* Failure on reading headers *******\n\n");
+        lept_stderr("\n  ******* Failure on reading headers *******\n\n");
     }
     if (!success) failure = TRUE;
 
     if (!failure) {
-        fprintf(stderr,
-            "  ******* Success on all tests *******\n\n");
+        lept_stderr("  ******* Success on all tests *******\n\n");
     } else {
-        fprintf(stderr,
-            "  ******* Failure on at least one test *******\n\n");
+        lept_stderr("  ******* Failure on at least one test *******\n\n");
     }
 
     if (failure) rp->success = FALSE;
@@ -225,22 +241,22 @@ PIX      *pixs;
 PIX      *pixd = NULL;
 
     if ((pixs = pixRead(fname)) == NULL) {
-        fprintf(stderr, "Failure to read %s\n", fname);
+        lept_stderr("Failure to read %s\n", fname);
         return 1;
     }
     if (pixWriteMem(&data, &size, pixs, IFF_PNG)) {
-        fprintf(stderr, "Mem write fail for png\n");
+        lept_stderr("Mem write fail for png\n");
         return 1;
     }
     if ((pixd = pixReadMem(data, size)) == NULL) {
-        fprintf(stderr, "Mem read fail for png\n");
+        lept_stderr("Mem read fail for png\n");
         lept_free(data);
         return 1;
     }
 
     pixEqual(pixs, pixd, &same);
     if (!same)
-        fprintf(stderr, "Mem write/read fail for file %s\n", fname);
+        lept_stderr("Mem write/read fail for file %s\n", fname);
     pixDestroy(&pixs);
     pixDestroy(&pixd);
     lept_free(data);
@@ -262,17 +278,17 @@ size_t    nbytes1, nbytes2;
     d1 = bps1 * spp1;
     if (d1 == 24) d1 = 32;
     if (ret1) {
-        fprintf(stderr, "Error: couldn't read header data from file: %s\n",
-                filename);
+        lept_stderr("Error: couldn't read header data from file: %s\n",
+                    filename);
     } else {
-        fprintf(stderr, "Format data for image %s with format %s:\n"
-            "  nbytes = %lu, size (w, h, d) = (%d, %d, %d)\n"
-            "  bps = %d, spp = %d, iscmap = %d\n",
-            filename, ImageFileFormatExtensions[format1],
-            (unsigned long)nbytes1, w1, h1, d1, bps1, spp1, iscmap1);
+        lept_stderr("Format data for image %s with format %s:\n"
+                    "  nbytes = %lu, size (w, h, d) = (%d, %d, %d)\n"
+                    "  bps = %d, spp = %d, iscmap = %d\n",
+                    filename, ImageFileFormatExtensions[format1],
+                    (unsigned long)nbytes1, w1, h1, d1, bps1, spp1, iscmap1);
         if (format1 != IFF_PNG) {
-            fprintf(stderr, "Error: format is %d; should be %d\n",
-                    format1, IFF_PNG);
+            lept_stderr("Error: format is %d; should be %d\n",
+                        format1, IFF_PNG);
             ret1 = 1;
         }
     }
@@ -285,13 +301,13 @@ size_t    nbytes1, nbytes2;
     d2 = bps2 * spp2;
     if (d2 == 24) d2 = 32;
     if (ret2) {
-        fprintf(stderr, "Error: couldn't mem-read header data: %s\n", filename);
+        lept_stderr("Error: couldn't mem-read header data: %s\n", filename);
     } else {
         if (nbytes1 != nbytes2 || format1 != format2 || w1 != w2 ||
             h1 != h2 || d1 != d2 || bps1 != bps2 || spp1 != spp2 ||
             iscmap1 != iscmap2) {
-            fprintf(stderr, "Incomsistency reading image %s with format %s\n",
-                    filename, ImageFileFormatExtensions[IFF_PNG]);
+            lept_stderr("Incomsistency reading image %s with format %s\n",
+                        filename, ImageFileFormatExtensions[IFF_PNG]);
             ret2 = 1;
         }
     }
@@ -316,9 +332,9 @@ PIXCMAP  *cmap;
     pix2 = pixRead("/tmp/lept/regout/1bpp-trans.png");
     pixEqual(pix1, pix2, &same);
     if (same)
-        fprintf(stderr, "1bpp_trans: success\n");
+        lept_stderr("1bpp_trans: success\n");
     else
-        fprintf(stderr, "1bpp_trans: bad output\n");
+        lept_stderr("1bpp_trans: bad output\n");
     pixDisplayWithTitle(pix2, 700, 0, NULL, rp->display);
     pixDestroy(&pix1);
     pixDestroy(&pix2);
@@ -326,9 +342,9 @@ PIXCMAP  *cmap;
     fgetPngColormapInfo(fp, &cmap, &transp);
     if (fp) fclose(fp);
     if (transp)
-        fprintf(stderr, "1bpp_trans: correct -- transparency found\n");
+        lept_stderr("1bpp_trans: correct -- transparency found\n");
     else
-        fprintf(stderr, "1bpp_trans: error -- no transparency found!\n");
+        lept_stderr("1bpp_trans: error -- no transparency found!\n");
     if (rp->display) pixcmapWriteStream(stderr, cmap);
     pixcmapDestroy(&cmap);
     return same;
@@ -351,9 +367,9 @@ PIXCMAP  *cmap;
     pix2 = pixRead("/tmp/lept/regout/1bpp-color.png");
     pixEqual(pix1, pix2, &same);
     if (same)
-        fprintf(stderr, "1bpp_color: success\n");
+        lept_stderr("1bpp_color: success\n");
     else
-        fprintf(stderr, "1bpp_color: bad output\n");
+        lept_stderr("1bpp_color: bad output\n");
     pixDisplayWithTitle(pix2, 700, 100, NULL, rp->display);
     pixDestroy(&pix1);
     pixDestroy(&pix2);
@@ -361,9 +377,9 @@ PIXCMAP  *cmap;
     fgetPngColormapInfo(fp, &cmap, &transp);
     if (fp) fclose(fp);
     if (transp)
-        fprintf(stderr, "1bpp_color: error -- transparency found!\n");
+        lept_stderr("1bpp_color: error -- transparency found!\n");
     else
-        fprintf(stderr, "1bpp_color: correct -- no transparency found\n");
+        lept_stderr("1bpp_color: correct -- no transparency found\n");
     if (rp->display) pixcmapWriteStream(stderr, cmap);
     pixcmapDestroy(&cmap);
     return same;
@@ -385,9 +401,9 @@ PIXCMAP  *cmap;
     pix2 = pixRead("/tmp/lept/regout/1bpp-gray.png");
     pixEqual(pix1, pix2, &same);
     if (same)
-        fprintf(stderr, "1bpp_gray: success\n");
+        lept_stderr("1bpp_gray: success\n");
     else
-        fprintf(stderr, "1bpp_gray: bad output\n");
+        lept_stderr("1bpp_gray: bad output\n");
     pixDisplayWithTitle(pix2, 700, 200, NULL, rp->display);
     pixDestroy(&pix1);
     pixDestroy(&pix2);
@@ -410,9 +426,9 @@ PIXCMAP  *cmap;
     pix2 = pixRead("/tmp/lept/regout/1bpp-bw1.png");
     pixEqual(pix1, pix2, &same);
     if (same)
-        fprintf(stderr, "1bpp_bw1: success\n");
+        lept_stderr("1bpp_bw1: success\n");
     else
-        fprintf(stderr, "1bpp_bw1: bad output\n");
+        lept_stderr("1bpp_bw1: bad output\n");
     pixDisplayWithTitle(pix2, 700, 300, NULL, rp->display);
     pixDestroy(&pix1);
     pixDestroy(&pix2);
@@ -435,15 +451,109 @@ PIXCMAP  *cmap;
     pix2 = pixRead("/tmp/lept/regout/1bpp-bw2.png");
     pixEqual(pix1, pix2, &same);
     if (same)
-        fprintf(stderr, "1bpp_bw2: success\n");
+        lept_stderr("1bpp_bw2: success\n");
     else
-        fprintf(stderr, "1bpp_bw2: bad output\n");
+        lept_stderr("1bpp_bw2: bad output\n");
     pixDisplayWithTitle(pix2, 700, 400, NULL, rp->display);
     pixDestroy(&pix1);
     pixDestroy(&pix2);
     return same;
 }
 
+static l_int32
+test_2bpp_cmap_trans(L_REGPARAMS  *rp)
+{
+l_int32   w, h, same;
+PIX      *pix1, *pix2;
+PIXCMAP  *cmap;
+
+    pix1 = pixRead(FILE_TRANS_CMAP_2BPP);
+    pixGetDimensions(pix1, &w, &h, NULL);
+    if (w != 82 || h != 73) {
+        lept_stderr("%s: bad dimensions\n", FILE_TRANS_CMAP_2BPP);
+        return 0;
+    }
+    pixDestroy(&pix1);
+
+    pix1 = pixRead("weasel2.4g.png");
+    cmap = pixGetColormap(pix1);
+    pixcmapSetAlpha(cmap, 2, 100);
+    pixWrite("/tmp/lept/regout/2bpp-cmap-trans.png", pix1, IFF_PNG);
+    pix2 = pixRead("/tmp/lept/regout/2bpp-cmap-trans.png");
+    pixEqual(pix1, pix2, &same);
+    if (same)
+        lept_stderr("2bpp-cmap-trans: success\n");
+    else
+        lept_stderr("2bpp-cmap-trans: bad output\n");
+    pixDisplayWithTitle(pix2, 0, 800, NULL, rp->display);
+    pixDestroy(&pix1);
+    pixDestroy(&pix2);
+    return same;
+}
+
+static l_int32
+test_4bpp_cmap_trans(L_REGPARAMS  *rp)
+{
+l_int32   w, h, same;
+PIX      *pix1, *pix2;
+PIXCMAP  *cmap;
+
+    pix1 = pixRead(FILE_TRANS_CMAP_4BPP);
+    pixGetDimensions(pix1, &w, &h, NULL);
+    if (w != 82 || h != 73) {
+        lept_stderr("%s: bad dimensions\n", FILE_TRANS_CMAP_4BPP);
+        return 0;
+    }
+    pixDestroy(&pix1);
+
+    pix1 = pixRead("weasel4.5g.png");
+    cmap = pixGetColormap(pix1);
+    pixcmapSetAlpha(cmap, 2, 60);
+    pixWrite("/tmp/lept/regout/4bpp-cmap-trans.png", pix1, IFF_PNG);
+    pix2 = pixRead("/tmp/lept/regout/4bpp-cmap-trans.png");
+    pixEqual(pix1, pix2, &same);
+    if (same)
+        lept_stderr("4bpp-cmap-trans: success\n");
+    else
+        lept_stderr("4bpp-cmap-trans: bad output\n");
+    pixDisplayWithTitle(pix2, 200, 800, NULL, rp->display);
+    pixDestroy(&pix1);
+    pixDestroy(&pix2);
+    return same;
+}
+
+static l_int32
+test_8bpp_cmap_trans(L_REGPARAMS  *rp)
+{
+l_int32   w, h, same;
+PIX      *pix1, *pix2;
+PIXCMAP  *cmap;
+
+    pix1 = pixRead(FILE_TRANS_CMAP_8BPP);
+    pixGetDimensions(pix1, &w, &h, NULL);
+    if (w != 82 || h != 73) {
+        lept_stderr("%s: bad dimensions\n", FILE_TRANS_CMAP_8BPP);
+        return 0;
+    }
+    pixDestroy(&pix1);
+
+    pix1 = pixRead("weasel8.5g.png");
+    cmap = pixGetColormap(pix1);
+    pixcmapSetAlpha(cmap, 2, 80);
+    pixcmapSetAlpha(cmap, 3, 80);
+    pixWrite("/tmp/lept/regout/8bpp-cmap-trans.png", pix1, IFF_PNG);
+    pix2 = pixRead("/tmp/lept/regout/8bpp-cmap-trans.png");
+    pixEqual(pix1, pix2, &same);
+    if (same)
+        lept_stderr("8bpp-cmap-trans: success\n");
+    else
+        lept_stderr("8bpp-cmap-trans: bad output\n");
+    pixDisplayWithTitle(pix2, 200, 800, NULL, rp->display);
+    pixDestroy(&pix1);
+    pixDestroy(&pix2);
+    return same;
+}
+
 static l_int32
 test_8bpp_trans(L_REGPARAMS  *rp)
 {
@@ -460,9 +570,9 @@ PIXCMAP  *cmap;
     pix3 = pixRead("/tmp/lept/regout/8bpp-trans.png");
     pixEqual(pix2, pix3, &same);
     if (same)
-        fprintf(stderr, "8bpp_trans: success\n");
+        lept_stderr("8bpp_trans: success\n");
     else
-        fprintf(stderr, "8bpp_trans: bad output\n");
+        lept_stderr("8bpp_trans: bad output\n");
     pixDisplayWithTitle(pix3, 700, 0, NULL, rp->display);
     pixDestroy(&pix1);
     pixDestroy(&pix2);
@@ -471,9 +581,9 @@ PIXCMAP  *cmap;
     fgetPngColormapInfo(fp, &cmap, &transp);
     if (fp) fclose(fp);
     if (transp)
-        fprintf(stderr, "8bpp_trans: correct -- transparency found\n");
+        lept_stderr("8bpp_trans: correct -- transparency found\n");
     else
-        fprintf(stderr, "8bpp_trans: error -- no transparency found!\n");
+        lept_stderr("8bpp_trans: error -- no transparency found!\n");
     if (rp->display) pixcmapWriteStream(stderr, cmap);
     pixcmapDestroy(&cmap);
     return same;
