@@ -59,7 +59,7 @@ PIXA     *pixa1, *pixa2;
 L_RECOG  *recog;
 
     if (argc != 1) {
-        fprintf(stderr, " Syntax: recogtest6\n");
+        lept_stderr(" Syntax: recogtest6\n");
         return 1;
     }
 
@@ -91,14 +91,14 @@ L_RECOG  *recog;
             rchaExtract(recog->rcha, NULL, &nascore, NULL, NULL,
                         NULL, NULL, NULL);
             pixDisplay(pixdb, 300, 500);
-            boxaWriteStream(stderr, boxa);
-            numaWriteStream(stderr, nascore);
+            boxaWriteStderr(boxa);
+            numaWriteStderr(nascore);
             numaDestroy(&nascore);
             pixDestroy(&pixdb);
         } else {  /* just get the timing */
             startTimer();
             recogIdentifyMultiple(recog, pix2, 0, 0, &boxa, NULL, NULL, 0);
-            fprintf(stderr, "Time: %5.3f\n", stopTimer());
+            lept_stderr("Time: %5.3f\n", stopTimer());
         }
         pixDestroy(&pix1);
         pixDestroy(&pix2);
