@@ -73,6 +73,10 @@
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <math.h>
 #include "allheaders.h"
 #include "bilateral.h"
@@ -88,19 +92,19 @@ static void bilateralDestroy(L_BILATERAL **pbil);
 #define  DEBUG_BILATERAL    0
 #endif  /* ~NO_CONSOLE_IO */
 
-
 /*--------------------------------------------------------------------------*
  *  Top level approximate separable grayscale or color bilateral filtering  *
  *--------------------------------------------------------------------------*/
 /*!
  * \brief   pixBilateral()
  *
- * \param[in]    pixs 8 bpp gray or 32 bpp rgb, no colormap
- * \param[in]    spatial_stdev  of gaussian kernel; in pixels, > 0.5
- * \param[in]    range_stdev  of gaussian range kernel; > 5.0; typ. 50.0
- * \param[in]    ncomps number of intermediate sums J(k,x); in [4 ... 30]
- * \param[in]    reduction  1, 2 or 4
- * \return  pixd bilateral filtered image, or NULL on error
+ * \param[in]    pixs            8 bpp gray or 32 bpp rgb, no colormap
+ * \param[in]    spatial_stdev   of gaussian kernel; in pixels, > 0.5
+ * \param[in]    range_stdev     of gaussian range kernel; > 5.0; typ. 50.0
+ * \param[in]    ncomps          number of intermediate sums J(k,x);
+ *                               in [4 ... 30]
+ * \param[in]    reduction       1, 2 or 4
+ * \return  pixd   bilateral filtered image, or NULL on error
  *
  * <pre>
  * Notes:
@@ -111,19 +115,19 @@ static void bilateralDestroy(L_BILATERAL **pbil);
  *      (2) We impose minimum values for range_stdev and ncomps to
  *          avoid nasty artifacts when either are too small.  We also
  *          impose a constraint on their product:
- *               ncomps * range_stdev \>= 100.
- *          So for values of range_stdev \>= 25, ncomps can be as small as 4.
+ *               ncomps * range_stdev >= 100.
+ *          So for values of range_stdev >= 25, ncomps can be as small as 4.
  *          Here is a qualitative, intuitive explanation for this constraint.
  *          Call the difference in k values between the J(k) == 'delta', where
  *              'delta' ~ 200 / ncomps
  *          Then this constraint is roughly equivalent to the condition:
- *              'delta' \< 2 * range_stdev
+ *              'delta' < 2 * range_stdev
  *          Note that at an intensity difference of (2 * range_stdev), the
  *          range part of the kernel reduces the effect by the factor 0.14.
  *          This constraint requires that we have a sufficient number of
  *          PCBs (i.e, a small enough 'delta'), so that for any value of
  *          image intensity I, there exists a k (and a PCB, J(k), such that
- *              |I - k| \< range_stdev
+ *              |I - k| < range_stdev
  *          Any fewer PCBs and we don't have enough to support this condition.
  *      (3) The upper limit of 30 on ncomps is imposed because the
  *          gain in accuracy is not worth the extra computation.
@@ -144,6 +148,9 @@ static void bilateralDestroy(L_BILATERAL **pbil);
  *          range_stdev = 60, ncomps = 6, and spatial_dev = {10, 30, 50}.
  *          As spatial_dev gets larger, we get the counter-intuitive
  *          result that the body of the red fish becomes less blurry.
+ *      (8) The image must be sufficiently big to get reasonable results.
+ *          This requires the dimensions to be at least twice the filter size.
+ *          Otherwise, return a copy of the input with warning.
  * </pre>
  */
 PIX *
@@ -153,7 +160,7 @@ pixBilateral(PIX       *pixs,
              l_int32    ncomps,
              l_int32    reduction)
 {
-l_int32       d;
+l_int32       w, h, d, filtersize;
 l_float32     sstdev;  /* scaled spatial stdev */
 PIX          *pixt, *pixr, *pixg, *pixb, *pixd;
 
@@ -161,11 +168,17 @@ PIX          *pixt, *pixr, *pixg, *pixb, *pixd;
 
     if (!pixs || pixGetColormap(pixs))
         return (PIX *)ERROR_PTR("pixs not defined or cmapped", procName, NULL);
-    d = pixGetDepth(pixs);
+    pixGetDimensions(pixs, &w, &h, &d);
     if (d != 8 && d != 32)
         return (PIX *)ERROR_PTR("pixs not 8 or 32 bpp", procName, NULL);
     if (reduction != 1 && reduction != 2 && reduction != 4)
         return (PIX *)ERROR_PTR("reduction invalid", procName, NULL);
+    filtersize = (l_int32)(2.0 * spatial_stdev + 1.0 + 0.5);
+    if (w < 2 * filtersize || h < 2 * filtersize) {
+        L_WARNING("w = %d, h = %d; w or h < 2 * filtersize = %d; "
+                  "returning copy\n", procName, w, h, 2 * filtersize);
+        return pixCopy(NULL, pixs);
+    }
     sstdev = spatial_stdev / (l_float32)reduction;  /* reduced spat. stdev */
     if (sstdev < 0.5)
         return (PIX *)ERROR_PTR("sstdev < 0.5", procName, NULL);
@@ -203,12 +216,13 @@ PIX          *pixt, *pixr, *pixg, *pixb, *pixd;
 /*!
  * \brief   pixBilateralGray()
  *
- * \param[in]    pixs 8 bpp gray
- * \param[in]    spatial_stdev  of gaussian kernel; in pixels, > 0.5
- * \param[in]    range_stdev  of gaussian range kernel; > 5.0; typ. 50.0
- * \param[in]    ncomps number of intermediate sums J(k,x); in [4 ... 30]
- * \param[in]    reduction  1, 2 or 4
- * \return  pixd 8 bpp bilateral filtered image, or NULL on error
+ * \param[in]    pixs             8 bpp gray
+ * \param[in]    spatial_stdev    of gaussian kernel; in pixels, > 0.5
+ * \param[in]    range_stdev      of gaussian range kernel; > 5.0; typ. 50.0
+ * \param[in]    ncomps           number of intermediate sums J(k,x);
+ *                                in [4 ... 30]
+ * \param[in]    reduction        1, 2 or 4
+ * \return  pixd   8 bpp bilateral filtered image, or NULL on error
  *
  * <pre>
  * Notes:
@@ -259,11 +273,12 @@ L_BILATERAL  *bil;
 /*!
  * \brief   bilateralCreate()
  *
- * \param[in]    pixs 8 bpp gray, no colormap
- * \param[in]    spatial_stdev  of gaussian kernel; in pixels, > 0.5
- * \param[in]    range_stdev  of gaussian range kernel; > 5.0; typ. 50.0
- * \param[in]    ncomps number of intermediate sums J(k,x); in [4 ... 30]
- * \param[in]    reduction  1, 2 or 4
+ * \param[in]    pixs            8 bpp gray, no colormap
+ * \param[in]    spatial_stdev   of gaussian kernel; in pixels, > 0.5
+ * \param[in]    range_stdev     of gaussian range kernel; > 5.0; typ. 50.0
+ * \param[in]    ncomps          number of intermediate sums J(k,x);
+ *                               in [4 ... 30]
+ * \param[in]    reduction       1, 2 or 4
  * \return  bil, or NULL on error
  *
  * <pre>
@@ -291,41 +306,42 @@ l_int32      *nc, *kindex;
 l_float32    *kfract, *range, *spatial;
 l_uint32     *datas, *datat, *datad, *lines, *linet, *lined;
 L_BILATERAL  *bil;
-PIX          *pixt, *pixt2, *pixsc, *pixd;
+PIX          *pix1, *pix2, *pixt, *pixsc, *pixd;
 PIXA         *pixac;
 
     PROCNAME("bilateralCreate");
 
+    if (reduction == 1) {
+        pix1 = pixClone(pixs);
+    } else if (reduction == 2) {
+        pix1 = pixScaleAreaMap2(pixs);
+    } else {  /* reduction == 4) */
+        pix2 = pixScaleAreaMap2(pixs);
+        pix1 = pixScaleAreaMap2(pix2);
+        pixDestroy(&pix2);
+    }
+    if (!pix1)
+        return (L_BILATERAL *)ERROR_PTR("pix1 not made", procName, NULL);
+
     sstdev = spatial_stdev / (l_float32)reduction;  /* reduced spat. stdev */
-    if ((bil = (L_BILATERAL *)LEPT_CALLOC(1, sizeof(L_BILATERAL))) == NULL)
-        return (L_BILATERAL *)ERROR_PTR("bil not made", procName, NULL);
+    border = (l_int32)(2 * sstdev + 1);
+    pixsc = pixAddMirroredBorder(pix1, border, border, border, border);
+    pixGetExtremeValue(pix1, 1, L_SELECT_MIN, NULL, NULL, NULL, &minval);
+    pixGetExtremeValue(pix1, 1, L_SELECT_MAX, NULL, NULL, NULL, &maxval);
+    pixDestroy(&pix1);
+    if (!pixsc)
+        return (L_BILATERAL *)ERROR_PTR("pixsc not made", procName, NULL);
+
+    bil = (L_BILATERAL *)LEPT_CALLOC(1, sizeof(L_BILATERAL));
     bil->spatial_stdev = sstdev;
     bil->range_stdev = range_stdev;
     bil->reduction = reduction;
     bil->ncomps = ncomps;
-
-    if (reduction == 1) {
-        pixt = pixClone(pixs);
-    } else if (reduction == 2) {
-        pixt = pixScaleAreaMap2(pixs);
-    } else {  /* reduction == 4) */
-        pixt2 = pixScaleAreaMap2(pixs);
-        pixt = pixScaleAreaMap2(pixt2);
-        pixDestroy(&pixt2);
-    }
-
-    pixGetExtremeValue(pixt, 1, L_SELECT_MIN, NULL, NULL, NULL, &minval);
-    pixGetExtremeValue(pixt, 1, L_SELECT_MAX, NULL, NULL, NULL, &maxval);
     bil->minval = minval;
     bil->maxval = maxval;
-
-    border = (l_int32)(2 * sstdev + 1);
-    pixsc = pixAddMirroredBorder(pixt, border, border, border, border);
     bil->pixsc = pixsc;
-    pixDestroy(&pixt);
     bil->pixs = pixClone(pixs);
 
-
     /* -------------------------------------------------------------------- *
      * Generate arrays for interpolation of J(k,x):
      *  (1.0 - kfract[.]) * J(kindex[.], x) + kfract[.] * J(kindex[.] + 1, x),
@@ -365,17 +381,16 @@ PIXA         *pixac;
 
 #if  DEBUG_BILATERAL
     for (i = minval; i <= maxval; i++)
-      fprintf(stderr, "kindex[%d] = %d; kfract[%d] = %5.3f\n",
-              i, kindex[i], i, kfract[i]);
+      lept_stderr("kindex[%d] = %d; kfract[%d] = %5.3f\n",
+                  i, kindex[i], i, kfract[i]);
     for (i = 0; i < ncomps; i++)
-      fprintf(stderr, "nc[%d] = %d\n", i, nc[i]);
+      lept_stderr("nc[%d] = %d\n", i, nc[i]);
 #endif  /* DEBUG_BILATERAL */
 
-
     /* -------------------------------------------------------------------- *
      *             Generate 1-D kernel arrays (spatial and range)           *
      * -------------------------------------------------------------------- */
-    spatial_size = 2 * sstdev + 1;
+    spatial_size = 2 * sstdev + 1;  /* same as the added border */
     spatial = (l_float32 *)LEPT_CALLOC(spatial_size, sizeof(l_float32));
     denom = 2. * sstdev * sstdev;
     for (i = 0; i < spatial_size; i++)
@@ -388,7 +403,6 @@ PIXA         *pixac;
         range[i] = expf(-(l_float32)(i * i) / denom);
     bil->range = range;
 
-
     /* -------------------------------------------------------------------- *
      *            Generate principal bilateral component images             *
      * -------------------------------------------------------------------- */
@@ -418,8 +432,10 @@ PIXA         *pixac;
                     sum += kern * nval;
                     norm += kern;
                 }
-                dval = (l_int32)((sum / norm) + 0.5);
-                SET_DATA_BYTE(linet, border + j, dval);
+                if (norm > 0.0) {
+                    dval = (l_int32)((sum / norm) + 0.5);
+                    SET_DATA_BYTE(linet, border + j, dval);
+                }
             }
         }
             /* Vertical convolution */
@@ -438,7 +454,10 @@ PIXA         *pixac;
                     sum += kern * nval;
                     norm += kern;
                 }
-                dval = (l_int32)((sum / norm) + 0.5);
+                if (norm > 0.0)
+                    dval = (l_int32)((sum / norm) + 0.5);
+                else
+                    dval = GET_DATA_BYTE(linet, border + j);
                 SET_DATA_BYTE(lined, j, dval);
             }
         }
@@ -447,7 +466,6 @@ PIXA         *pixac;
     }
     bil->pixac = pixac;
     bil->lineset = (l_uint32 ***)pixaGetLinePtrs(pixac, NULL);
-
     return bil;
 }
 
@@ -515,7 +533,7 @@ PIXA        *pixac;
 /*!
  * \brief   bilateralDestroy()
  *
- * \param[in,out]   pbil will be nulled
+ * \param[in,out]   pbil    will be set to null before returning
  */
 static void
 bilateralDestroy(L_BILATERAL  **pbil)
@@ -546,7 +564,6 @@ L_BILATERAL  *bil;
     LEPT_FREE(bil->lineset);
     LEPT_FREE(bil);
     *pbil = NULL;
-    return;
 }
 
 
@@ -556,10 +573,10 @@ L_BILATERAL  *bil;
 /*!
  * \brief   pixBilateralExact()
  *
- * \param[in]    pixs 8 bpp gray or 32 bpp rgb
- * \param[in]    spatial_kel  gaussian kernel
- * \param[in]    range_kel [optional] 256 x 1, monotonically decreasing
- * \return  pixd 8 bpp bilateral filtered image
+ * \param[in]    pixs          8 bpp gray or 32 bpp rgb
+ * \param[in]    spatial_kel   gaussian kernel
+ * \param[in]    range_kel     [optional] 256 x 1, monotonically decreasing
+ * \return  pixd   8 bpp bilateral filtered image
  *
  * <pre>
  * Notes:
@@ -620,10 +637,10 @@ PIX     *pixt, *pixr, *pixg, *pixb, *pixd;
 /*!
  * \brief   pixBilateralGrayExact()
  *
- * \param[in]    pixs 8 bpp gray
- * \param[in]    spatial_kel  gaussian kernel
- * \param[in]    range_kel [optional] 256 x 1, monotonically decreasing
- * \return  pixd 8 bpp bilateral filtered image
+ * \param[in]    pixs          8 bpp gray
+ * \param[in]    spatial_kel   gaussian kernel
+ * \param[in]    range_kel     [optional] 256 x 1, monotonically decreasing
+ * \return  pixd   8 bpp bilateral filtered image
  *
  * <pre>
  * Notes:
@@ -651,9 +668,14 @@ PIX       *pixt, *pixd;
     pixGetDimensions(pixs, &w, &h, &d);
     if (!spatial_kel)
         return (PIX *)ERROR_PTR("spatial kel not defined", procName, NULL);
-
+    kernelGetParameters(spatial_kel, &sy, &sx, NULL, NULL);
+    if (w < 2 * sx + 1 || h < 2 * sy + 1) {
+        L_WARNING("w = %d < 2 * sx + 1 = %d, or h = %d < 2 * sy + 1 = %d; "
+                  "returning copy\n", procName, w, 2 * sx + 1, h, 2 * sy + 1);
+        return pixCopy(NULL, pixs);
+    }
     if (!range_kel)
-      return pixConvolve(pixs, spatial_kel, 8, 1);
+        return pixConvolve(pixs, spatial_kel, 8, 1);
     if (range_kel->sx != 256 || range_kel->sy != 1)
         return (PIX *)ERROR_PTR("range kel not {256 x 1", procName, NULL);
 
@@ -698,10 +720,10 @@ PIX       *pixt, *pixd;
 /*!
  * \brief   pixBlockBilateralExact()
  *
- * \param[in]    pixs 8 bpp gray or 32 bpp rgb
- * \param[in]    spatial_stdev > 0.0
- * \param[in]    range_stdev > 0.0
- * \return  pixd 8 bpp or 32 bpp bilateral filtered image
+ * \param[in]    pixs             8 bpp gray or 32 bpp rgb
+ * \param[in]    spatial_stdev    must be > 0.0
+ * \param[in]    range_stdev      must be > 0.0
+ * \return  pixd   8 bpp or 32 bpp bilateral filtered image
  *
  * <pre>
  * Notes:
@@ -770,7 +792,7 @@ PIX       *pixd;
 /*!
  * \brief   makeRangeKernel()
  *
- * \param[in]    range_stdev > 0
+ * \param[in]    range_stdev   must be > 0.0
  * \return  kel, or NULL on error
  *
  * <pre>
