@@ -35,6 +35,7 @@
  *           BOX             *boxTransformOrdered()
  *           BOXA            *boxaRotateOrth()
  *           BOX             *boxRotateOrth()
+ *           BOXA            *boxaShiftWithPta()
  *
  *      Boxa sort
  *           BOXA            *boxaSort()
@@ -51,6 +52,7 @@
  *      Boxa array extraction
  *           l_int32          boxaExtractAsNuma()
  *           l_int32          boxaExtractAsPta()
+ *           PTA             *boxaExtractCorners()
  *
  *      Other Boxaa functions
  *           l_int32          boxaaGetExtent()
@@ -62,14 +64,17 @@
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <math.h>
 #include "allheaders.h"
 
     /* For more than this number of c.c. in a binarized image of
      * semi-perimeter (w + h) about 5000 or less, the O(n) binsort
      * is faster than the O(nlogn) shellsort.  */
-static const l_int32   MIN_COMPS_FOR_BIN_SORT = 200;
-
+static const l_int32   MinCompsForBinSort = 200;
 
 /*---------------------------------------------------------------------*
  *      Boxa/Box transform (shift, scale) and orthogonal rotation      *
@@ -78,13 +83,19 @@ static const l_int32   MIN_COMPS_FOR_BIN_SORT = 200;
  * \brief   boxaTransform()
  *
  * \param[in]    boxas
- * \param[in]    shiftx, shifty
- * \param[in]    scalex, scaley
+ * \param[in]    shiftx
+ * \param[in]    shifty
+ * \param[in]    scalex
+ * \param[in]    scaley
  * \return  boxad, or NULL on error
  *
  * <pre>
  * Notes:
  *      (1) This is a very simple function that first shifts, then scales.
+ *      (2) The UL corner coordinates of all boxes in the output %boxad
+ *      (3) For the boxes in the output %boxad, the UL corner coordinates
+ *          must be non-negative, and the width and height of valid
+ *          boxes must be at least 1.
  * </pre>
  */
 BOXA *
@@ -123,14 +134,18 @@ BOXA    *boxad;
  * \brief   boxTransform()
  *
  * \param[in]    box
- * \param[in]    shiftx, shifty
- * \param[in]    scalex, scaley
+ * \param[in]    shiftx
+ * \param[in]    shifty
+ * \param[in]    scalex
+ * \param[in]    scaley
  * \return  boxd, or NULL on error
  *
  * <pre>
  * Notes:
  *      (1) This is a very simple function that first shifts, then scales.
  *      (2) If the box is invalid, a new invalid box is returned.
+ *      (3) The UL corner coordinates must be non-negative, and the
+ *          width and height of valid boxes must be at least 1.
  * </pre>
  */
 BOX *
@@ -147,8 +162,8 @@ boxTransform(BOX       *box,
     if (box->w <= 0 || box->h <= 0)
         return boxCreate(0, 0, 0, 0);
     else
-        return boxCreate((l_int32)(scalex * (box->x + shiftx) + 0.5),
-                         (l_int32)(scaley * (box->y + shifty) + 0.5),
+        return boxCreate((l_int32)(L_MAX(0, scalex * (box->x + shiftx) + 0.5)),
+                         (l_int32)(L_MAX(0, scaley * (box->y + shifty) + 0.5)),
                          (l_int32)(L_MAX(1.0, scalex * box->w + 0.5)),
                          (l_int32)(L_MAX(1.0, scaley * box->h + 0.5)));
 }
@@ -158,17 +173,16 @@ boxTransform(BOX       *box,
  * \brief   boxaTransformOrdered()
  *
  * \param[in]    boxas
- * \param[in]    shiftx, shifty
- * \param[in]    scalex, scaley
- * \param[in]    xcen, ycen center of rotation
- * \param[in]    angle in radians; clockwise is positive
- * \param[in]    order one of 6 combinations: L_TR_SC_RO, ...
+ * \param[in]    shiftx
+ * \param[in]    shifty
+ * \param[in]    scalex
+ * \param[in]    scaley
+ * \param[in]    xcen, ycen    center of rotation
+ * \param[in]    angle         in radians; clockwise is positive
+ * \param[in]    order         one of 6 combinations: L_TR_SC_RO, ...
  * \return  boxd, or NULL on error
  *
  * <pre>
- * Notes:
- *      (1) This allows a sequence of linear transforms on each box.
- *          the transforms are from the affine set, composed of
  *          shift, scaling and rotation, and the order of the
  *          transforms is specified.
  *      (2) Although these operations appear to be on an infinite
@@ -229,11 +243,13 @@ BOXA    *boxad;
  * \brief   boxTransformOrdered()
  *
  * \param[in]    boxs
- * \param[in]    shiftx, shifty
- * \param[in]    scalex, scaley
- * \param[in]    xcen, ycen center of rotation
- * \param[in]    angle in radians; clockwise is positive
- * \param[in]    order one of 6 combinations: L_TR_SC_RO, ...
+ * \param[in]    shiftx
+ * \param[in]    shifty
+ * \param[in]    scalex
+ * \param[in]    scaley
+ * \param[in]    xcen, ycen   center of rotation
+ * \param[in]    angle        in radians; clockwise is positive
+ * \param[in]    order        one of 6 combinations: L_TR_SC_RO, ...
  * \return  boxd, or NULL on error
  *
  * <pre>
@@ -440,9 +456,9 @@ BOX       *boxd;
  * \brief   boxaRotateOrth()
  *
  * \param[in]    boxas
- * \param[in]    w, h of image in which the boxa is embedded
- * \param[in]    rotation 0 = noop, 1 = 90 deg, 2 = 180 deg, 3 = 270 deg;
- *                        all rotations are clockwise
+ * \param[in]    w, h       of image in which the boxa is embedded
+ * \param[in]    rotation   0 = noop, 1 = 90 deg, 2 = 180 deg, 3 = 270 deg;
+ *                          all rotations are clockwise
  * \return  boxad, or NULL on error
  *
  * <pre>
@@ -490,9 +506,9 @@ BOXA    *boxad;
  * \brief   boxRotateOrth()
  *
  * \param[in]    box
- * \param[in]    w, h of image in which the box is embedded
- * \param[in]    rotation 0 = noop, 1 = 90 deg, 2 = 180 deg, 3 = 270 deg;
- *                        all rotations are clockwise
+ * \param[in]    w, h       of image in which the box is embedded
+ * \param[in]    rotation   0 = noop, 1 = 90 deg, 2 = 180 deg, 3 = 270 deg;
+ *                          all rotations are clockwise
  * \return  boxd, or NULL on error
  *
  * <pre>
@@ -533,6 +549,67 @@ l_int32  bx, by, bw, bh, xdist, ydist;
 }
 
 
+/*!
+ * \brief   boxaShiftWithPta()
+ *
+ * \param[in]    boxas
+ * \param[in]    pta       aligned with the boxes; determines shift amount
+ * \param[in]    dir       +1 to shift by the values in pta; -1 to shift
+ *                         by the negative of the values in the pta.
+ * \return  boxad, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) In use, %pta may come from the UL corners of of a boxa, each
+ *          of whose boxes contains the corresponding box of %boxas
+ *          within it.  The output %boxad is then a boxa in the (global)
+ *          coordinates of the containing boxa.  So the input %pta
+ *          could come from boxaExtractCorners().
+ *      (2) The operations with %dir == 1 and %dir == -1 are inverses if
+ *          called in order (1, -1).  Starting with an input boxa and
+ *          calling twice with these values of %dir results in a boxa
+ *          identical to the input.  However, because box parameters can
+ *          never be negative, calling in the order (-1, 1) may result
+ *          in clipping at the left side and the top.
+ * </pre>
+ */
+BOXA *
+boxaShiftWithPta(BOXA    *boxas,
+                 PTA     *pta,
+                 l_int32  dir)
+{
+l_int32  i, n, x, y, full;
+BOX     *box1, *box2;
+BOXA    *boxad;
+
+    PROCNAME("boxaShiftWithPta");
+
+    if (!boxas)
+        return (BOXA *)ERROR_PTR("boxas not defined", procName, NULL);
+    boxaIsFull(boxas, &full);
+    if (!full)
+        return (BOXA *)ERROR_PTR("boxas not full", procName, NULL);
+    if (!pta)
+        return (BOXA *)ERROR_PTR("pta not defined", procName, NULL);
+    if (dir != 1 && dir != -1)
+        return (BOXA *)ERROR_PTR("invalid dir", procName, NULL);
+    n = boxaGetCount(boxas);
+    if (n != ptaGetCount(pta))
+        return (BOXA *)ERROR_PTR("boxas and pta not same size", procName, NULL);
+
+    if ((boxad = boxaCreate(n)) == NULL)
+        return (BOXA *)ERROR_PTR("boxad not made", procName, NULL);
+    for (i = 0; i < n; i++) {
+        box1 = boxaGetBox(boxas, i, L_COPY);
+        ptaGetIPt(pta, i, &x, &y);
+        box2 = boxTransform(box1, dir * x, dir * y, 1.0, 1.0);
+        boxaAddBox(boxad, box2, L_INSERT);
+        boxDestroy(&box1);
+    }
+    return boxad;
+}
+
+
 /*---------------------------------------------------------------------*
  *                              Boxa sort                              *
  *---------------------------------------------------------------------*/
@@ -540,15 +617,15 @@ l_int32  bx, by, bw, bh, xdist, ydist;
  * \brief   boxaSort()
  *
  * \param[in]    boxas
- * \param[in]    sorttype L_SORT_BY_X, L_SORT_BY_Y,
- *                        L_SORT_BY_RIGHT, L_SORT_BY_BOT,
- *                        L_SORT_BY_WIDTH, L_SORT_BY_HEIGHT,
- *                        L_SORT_BY_MIN_DIMENSION, L_SORT_BY_MAX_DIMENSION,
- *                        L_SORT_BY_PERIMETER, L_SORT_BY_AREA,
- *                        L_SORT_BY_ASPECT_RATIO
+ * \param[in]    sorttype   L_SORT_BY_X, L_SORT_BY_Y,
+ *                          L_SORT_BY_RIGHT, L_SORT_BY_BOT,
+ *                          L_SORT_BY_WIDTH, L_SORT_BY_HEIGHT,
+ *                          L_SORT_BY_MIN_DIMENSION, L_SORT_BY_MAX_DIMENSION,
+ *                          L_SORT_BY_PERIMETER, L_SORT_BY_AREA,
+ *                          L_SORT_BY_ASPECT_RATIO
  * \param[in]    sortorder  L_SORT_INCREASING, L_SORT_DECREASING
- * \param[out]   pnaindex [optional] index of sorted order into
- *                        original array
+ * \param[out]   pnaindex   [optional] index of sorted order into
+ *                          original array
  * \return  boxad sorted version of boxas, or NULL on error
  *
  * <pre>
@@ -588,7 +665,7 @@ NUMA      *na, *naindex;
         return (BOXA *)ERROR_PTR("invalid sort order", procName, NULL);
 
         /* Use O(n) binsort if possible */
-    if (n > MIN_COMPS_FOR_BIN_SORT &&
+    if (n > MinCompsForBinSort &&
         ((sorttype == L_SORT_BY_X) || (sorttype == L_SORT_BY_Y) ||
          (sorttype == L_SORT_BY_WIDTH) || (sorttype == L_SORT_BY_HEIGHT) ||
          (sorttype == L_SORT_BY_PERIMETER)))
@@ -664,11 +741,11 @@ NUMA      *na, *naindex;
  * \brief   boxaBinSort()
  *
  * \param[in]    boxas
- * \param[in]    sorttype L_SORT_BY_X, L_SORT_BY_Y, L_SORT_BY_WIDTH,
- *                        L_SORT_BY_HEIGHT, L_SORT_BY_PERIMETER
- * \param[in]    sortorder  L_SORT_INCREASING, L_SORT_DECREASING
- * \param[out]   pnaindex [optional] index of sorted order into
- *                        original array
+ * \param[in]    sorttype    L_SORT_BY_X, L_SORT_BY_Y, L_SORT_BY_WIDTH,
+ *                           L_SORT_BY_HEIGHT, L_SORT_BY_PERIMETER
+ * \param[in]    sortorder   L_SORT_INCREASING, L_SORT_DECREASING
+ * \param[out]   pnaindex    [optional] index of sorted order into
+ *                           original array
  * \return  boxad sorted version of boxas, or NULL on error
  *
  * <pre>
@@ -754,7 +831,7 @@ NUMA    *na, *naindex;
  * \brief   boxaSortByIndex()
  *
  * \param[in]    boxas
- * \param[in]    naindex na that maps from the new boxa to the input boxa
+ * \param[in]    naindex    na that maps from the new boxa to the input boxa
  * \return  boxad sorted, or NULL on error
  */
 BOXA *
@@ -791,14 +868,14 @@ BOXA    *boxad;
  * \brief   boxaSort2d()
  *
  * \param[in]    boxas
- * \param[out]   pnaad [optional] numaa with sorted indices
- *                    whose values are the indices of the input array
- * \param[in]    delta1 min overlap that permits aggregation of a box
- *                      onto a boxa of horizontally-aligned boxes; pass 1
- * \param[in]    delta2 min overlap that permits aggregation of a box
- *                      onto a boxa of horizontally-aligned boxes; pass 2
- * \param[in]    minh1 components less than this height either join an
- *                     existing boxa or are set aside for pass 2
+ * \param[out]   pnaad    [optional] numaa with sorted indices
+ *                        whose values are the indices of the input array
+ * \param[in]    delta1   min separation that permits aggregation of a box
+ *                        onto a boxa of horizontally-aligned boxes; pass 1
+ * \param[in]    delta2   min separation that permits aggregation of a box
+ *                        onto a boxa of horizontally-aligned boxes; pass 2
+ * \param[in]    minh1    components less than this height either join an
+ *                        existing boxa or are set aside for pass 2
  * \return  baa 2d sorted version of boxa, or NULL on error
  *
  * <pre>
@@ -816,11 +893,12 @@ BOXA    *boxad;
  *          is overlapping are joined.  After that, the boxes in each
  *          boxa are sorted horizontally, and finally the boxa are
  *          sorted vertically.
- *      (3) If delta1 < 0, the first pass allows aggregation when
- *          boxes in the same boxa do not overlap vertically.
- *          The distance by which they can miss and still be aggregated
- *          is the absolute value |delta1|.   Similar for delta2 on
- *          the second pass.
+ *      (3) If %delta1 > 0, the first pass allows aggregation when
+ *          boxes in the same boxa do not overlap vertically.  In fact,
+ *          %delta1 is the max distance by which they can miss and still
+ *          be aggregated.  If %delta1 < 0, the box must have vertical
+ *          overlap of at least abs(%delta1) with the boxa before it
+ *          can be merged.  Similar for delta2 on the second pass.
  *      (4) On the first pass, any component of height less than minh1
  *          cannot start a new boxa; it's put aside for later insertion.
  *      (5) On the second pass, any small component that doesn't align
@@ -996,7 +1074,7 @@ NUMAA   *naa, *naa1, *naad;
     }
 
 
-/*    fprintf(stderr, "box count = %d, numaa count = %d\n", nt,
+/*    lept_stderr("box count = %d, numaa count = %d\n", nt,
             numaaGetNumberCount(naad)); */
 
     boxaaDestroy(&baa);
@@ -1017,7 +1095,7 @@ NUMAA   *naa, *naa1, *naad;
  * \brief   boxaSort2dByIndex()
  *
  * \param[in]    boxas
- * \param[in]    naa numaa that maps from the new baa to the input boxa
+ * \param[in]    naa     numaa that maps from the new baa to the input boxa
  * \return  baa sorted boxaa, or NULL on error
  */
 BOXAA *
@@ -1070,13 +1148,13 @@ NUMA    *na;
  * \brief   boxaExtractAsNuma()
  *
  * \param[in]    boxa
- * \param[out]   pnal [optional] array of left locations
- * \param[out]   pnat [optional] array of top locations
- * \param[out]   pnar [optional] array of right locations
- * \param[out]   pnab [optional] array of bottom locations
- * \param[out]   pnaw [optional] array of widths
- * \param[out]   pnah [optional] array of heights
- * \param[in]    keepinvalid 1 to keep invalid boxes; 0 to remove them
+ * \param[out]   pnal          [optional] array of left locations
+ * \param[out]   pnat          [optional] array of top locations
+ * \param[out]   pnar          [optional] array of right locations
+ * \param[out]   pnab          [optional] array of bottom locations
+ * \param[out]   pnaw          [optional] array of widths
+ * \param[out]   pnah          [optional] array of heights
+ * \param[in]    keepinvalid   1 to keep invalid boxes; 0 to remove them
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1089,7 +1167,7 @@ NUMA    *na;
  *          doing the extraction. This is easily done with boxaFillSequence().
  * </pre>
  */
-l_int32
+l_ok
 boxaExtractAsNuma(BOXA    *boxa,
                   NUMA   **pnal,
                   NUMA   **pnat,
@@ -1145,13 +1223,13 @@ l_int32  i, n, left, top, right, bot, w, h;
  * \brief   boxaExtractAsPta()
  *
  * \param[in]    boxa
- * \param[out]   pptal [optional] array of left locations vs. index
- * \param[out]   pptat [optional] array of top locations vs. index
- * \param[out]   pptar [optional] array of right locations vs. index
- * \param[out]   pptab [optional] array of bottom locations vs. index
- * \param[out]   pptaw [optional] array of widths vs. index
- * \param[out]   pptah [optional] array of heights vs. index
- * \param[in]    keepinvalid 1 to keep invalid boxes; 0 to remove them
+ * \param[out]   pptal         [optional] array of left locations vs. index
+ * \param[out]   pptat         [optional] array of top locations vs. index
+ * \param[out]   pptar         [optional] array of right locations vs. index
+ * \param[out]   pptab         [optional] array of bottom locations vs. index
+ * \param[out]   pptaw         [optional] array of widths vs. index
+ * \param[out]   pptah         [optional] array of heights vs. index
+ * \param[in]    keepinvalid   1 to keep invalid boxes; 0 to remove them
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1164,9 +1242,13 @@ l_int32  i, n, left, top, right, bot, w, h;
  *          boxaExtractAsNuma(), is not necessary.
  *      (2) If invalid boxes are retained, each one will result in
  *          entries (typically 0) in all selected output pta.
+ *      (3) Other boxa --> pta functions are:
+ *          * boxaExtractCorners(): extracts any of the four corners as a pta.
+ *          * boxaConvertToPta(): extracts sufficient number of corners
+ *            to allow reconstruction of the original boxa from the pta.
  * </pre>
  */
-l_int32
+l_ok
 boxaExtractAsPta(BOXA    *boxa,
                  PTA    **pptal,
                  PTA    **pptat,
@@ -1218,6 +1300,69 @@ l_int32  i, n, left, top, right, bot, w, h;
 }
 
 
+/*!
+ * \brief   boxaExtractCorners()
+ *
+ * \param[in]    boxa
+ * \param[in]    loc       L_UPPER_LEFT, L_UPPER_RIGHT, L_LOWER_LEFT,
+ *                         L_LOWER_RIGHT, L_BOX_CENTER
+ * \return  pta of requested coordinates, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) Extracts (0,0) for invalid boxes.
+ *      (2) Other boxa --> pta functions are:
+ *          * boxaExtractAsPta(): allows extraction of any dimension
+ *            and/or side location, with each in a separate pta.
+ *          * boxaConvertToPta(): extracts sufficient number of corners
+ *            to allow reconstruction of the original boxa from the pta.
+ * </pre>
+ */
+PTA *
+boxaExtractCorners(BOXA    *boxa,
+                   l_int32  loc)
+{
+l_int32  i, n, left, top, right, bot, w, h;
+PTA     *pta;
+
+    PROCNAME("boxaExtractCorners");
+
+    if (!boxa)
+        return (PTA *)ERROR_PTR("boxa not defined", procName, NULL);
+    if (loc != L_UPPER_LEFT && loc != L_UPPER_RIGHT && loc != L_LOWER_LEFT &&
+        loc != L_LOWER_RIGHT && loc != L_BOX_CENTER)
+        return (PTA *)ERROR_PTR("invalid location", procName, NULL);
+
+    n = boxaGetCount(boxa);
+    if ((pta = ptaCreate(n)) == NULL)
+        return (PTA *)ERROR_PTR("pta not made", procName, NULL);
+
+    for (i = 0; i < n; i++) {
+        boxaGetBoxGeometry(boxa, i, &left, &top, &w, &h);
+        right = left + w - 1;
+        bot = top + h - 1;
+        if (w == 0 || h == 0) {  /* invalid */
+            left = 0;
+            top = 0;
+            right = 0;
+            bot = 0;
+        }
+        if (loc == L_UPPER_LEFT)
+            ptaAddPt(pta, left, top);
+        else if (loc == L_UPPER_RIGHT)
+            ptaAddPt(pta, right, top);
+        else if (loc == L_LOWER_LEFT)
+            ptaAddPt(pta, left, bot);
+        else if (loc == L_LOWER_RIGHT)
+            ptaAddPt(pta, right, bot);
+        else if (loc == L_BOX_CENTER)
+            ptaAddPt(pta, (left + right) / 2, (top + bot) / 2);
+    }
+
+    return pta;
+}
+
+
 /*---------------------------------------------------------------------*
  *                            Boxa statistics                          *
  *---------------------------------------------------------------------*/
@@ -1225,43 +1370,52 @@ l_int32  i, n, left, top, right, bot, w, h;
  * \brief   boxaGetRankVals()
  *
  * \param[in]    boxa
- * \param[in]    fract use 0.0 for smallest, 1.0 for largest width and height
- * \param[out]   px  [optional] rank value of x
- * \param[out]   py  [optional] rank value of y
- * \param[out]   pw  [optional] rank value of width
- * \param[out]   ph  [optional] rank value of height
+ * \param[in]    fract   use 0.0 for smallest, 1.0 for largest width and height
+ * \param[out]   px      [optional] rank value of x (left side)
+ * \param[out]   py      [optional] rank value of y (top side)
+ * \param[out]   pr      [optional] rank value of right side
+ * \param[out]   pb      [optional] rank value of bottom side
+ * \param[out]   pw      [optional] rank value of width
+ * \param[out]   ph      [optional] rank value of height
  * \return  0 if OK, 1 on error or if the boxa is empty or has no valid boxes
  *
  * <pre>
  * Notes:
  *      (1) This function does not assume that all boxes in the boxa are valid
- *      (2) The four box parameters are sorted independently.
+ *      (2) The six box parameters are sorted independently.
  *          For rank order, the width and height are sorted in increasing
  *          order.  But what does it mean to sort x and y in "rank order"?
  *          If the boxes are of comparable size and somewhat
  *          aligned (e.g., from multiple images), it makes some sense
  *          to give a "rank order" for x and y by sorting them in
- *          decreasing order.  But in general, the interpretation of a rank
- *          order on x and y is highly application dependent.  In summary:
+ *          decreasing order.  (By the same argument, we choose to sort
+ *          the r and b sides in increasing order.)  In general, the
+ *          interpretation of a rank order on x and y (or on r and b)
+ *          is highly application dependent.  In summary:
  *             ~ x and y are sorted in decreasing order
+ *             ~ r and b are sorted in increasing order
  *             ~ w and h are sorted in increasing order
  * </pre>
  */
-l_int32
+l_ok
 boxaGetRankVals(BOXA      *boxa,
                 l_float32  fract,
                 l_int32   *px,
                 l_int32   *py,
+                l_int32   *pr,
+                l_int32   *pb,
                 l_int32   *pw,
                 l_int32   *ph)
 {
-l_float32  xval, yval, wval, hval;
-NUMA      *nax, *nay, *naw, *nah;
+l_float32  xval, yval, rval, bval, wval, hval;
+NUMA      *nax, *nay, *nar, *nab, *naw, *nah;
 
     PROCNAME("boxaGetRankVals");
 
     if (px) *px = 0;
     if (py) *py = 0;
+    if (pr) *pr = 0;
+    if (pb) *pb = 0;
     if (pw) *pw = 0;
     if (ph) *ph = 0;
     if (!boxa)
@@ -1272,7 +1426,7 @@ NUMA      *nax, *nay, *naw, *nah;
         return ERROR_INT("no valid boxes in boxa", procName, 1);
 
         /* Use only the valid boxes */
-    boxaExtractAsNuma(boxa, &nax, &nay, NULL, NULL, &naw, &nah, 0);
+    boxaExtractAsNuma(boxa, &nax, &nay, &nar, &nab, &naw, &nah, 0);
 
     if (px) {
         numaGetRankValue(nax, 1.0 - fract, NULL, 1, &xval);
@@ -1282,6 +1436,14 @@ NUMA      *nax, *nay, *naw, *nah;
         numaGetRankValue(nay, 1.0 - fract, NULL, 1, &yval);
         *py = (l_int32)yval;
     }
+    if (pr) {
+        numaGetRankValue(nar, fract, NULL, 1, &rval);
+        *pr = (l_int32)rval;
+    }
+    if (pb) {
+        numaGetRankValue(nab, fract, NULL, 1, &bval);
+        *pb = (l_int32)bval;
+    }
     if (pw) {
         numaGetRankValue(naw, fract, NULL, 1, &wval);
         *pw = (l_int32)wval;
@@ -1292,6 +1454,8 @@ NUMA      *nax, *nay, *naw, *nah;
     }
     numaDestroy(&nax);
     numaDestroy(&nay);
+    numaDestroy(&nar);
+    numaDestroy(&nab);
     numaDestroy(&naw);
     numaDestroy(&nah);
     return 0;
@@ -1302,10 +1466,12 @@ NUMA      *nax, *nay, *naw, *nah;
  * \brief   boxaGetMedianVals()
  *
  * \param[in]    boxa
- * \param[out]   px  [optional] median value of x
- * \param[out]   py  [optional] median value of y
- * \param[out]   pw  [optional] median value of width
- * \param[out]   ph  [optional] median value of height
+ * \param[out]   px     [optional] median value of x (left side)
+ * \param[out]   py     [optional] median value of y (top side)
+ * \param[out]   pr     [optional] median value of right side
+ * \param[out]   pb     [optional] median value of bottom side
+ * \param[out]   pw     [optional] median value of width
+ * \param[out]   ph     [optional] median value of height
  * \return  0 if OK, 1 on error or if the boxa is empty or has no valid boxes
  *
  * <pre>
@@ -1313,10 +1479,12 @@ NUMA      *nax, *nay, *naw, *nah;
  *      (1) See boxaGetRankVals()
  * </pre>
  */
-l_int32
+l_ok
 boxaGetMedianVals(BOXA     *boxa,
                   l_int32  *px,
                   l_int32  *py,
+                  l_int32  *pr,
+                  l_int32  *pb,
                   l_int32  *pw,
                   l_int32  *ph)
 {
@@ -1327,7 +1495,7 @@ boxaGetMedianVals(BOXA     *boxa,
     if (boxaGetValidCount(boxa) == 0)
         return ERROR_INT("no valid boxes in boxa", procName, 1);
 
-    return boxaGetRankVals(boxa, 0.5, px, py, pw, ph);
+    return boxaGetRankVals(boxa, 0.5, px, py, pr, pb, pw, ph);
 }
 
 
@@ -1335,11 +1503,11 @@ boxaGetMedianVals(BOXA     *boxa,
  * \brief   boxaGetAverageSize()
  *
  * \param[in]    boxa
- * \param[out]   pw  [optional] average width
- * \param[out]   ph  [optional] average height
+ * \param[out]   pw     [optional] average width
+ * \param[out]   ph     [optional] average height
  * \return  0 if OK, 1 on error or if the boxa is empty
  */
-l_int32
+l_ok
 boxaGetAverageSize(BOXA       *boxa,
                    l_float32  *pw,
                    l_float32  *ph)
@@ -1376,12 +1544,12 @@ l_float32  sumw, sumh;
  * \brief   boxaaGetExtent()
  *
  * \param[in]    baa
- * \param[out]   pw  [optional] width
- * \param[out]   ph  [optional] height
- * \param[out]   pbox [optional]  minimum box containing all boxa
- *                    in boxaa
- * \param[out]   pboxa [optional]  boxa containing all boxes in each
- *                     boxa in the boxaa
+ * \param[out]   pw      [optional] width
+ * \param[out]   ph      [optional] height
+ * \param[out]   pbox    [optional]  minimum box containing all boxa
+ *                       in boxaa
+ * \param[out]   pboxa   [optional]  boxa containing all boxes in each
+ *                       boxa in the boxaa
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1394,7 +1562,7 @@ l_float32  sumw, sumh;
  *          to its extent has all fields set to 0 (an invalid box).
  * </pre>
  */
-l_int32
+l_ok
 boxaaGetExtent(BOXAA    *baa,
                l_int32  *pw,
                l_int32  *ph,
@@ -1457,8 +1625,8 @@ BOXA    *boxa, *boxa1;
  * \brief   boxaaFlattenToBoxa()
  *
  * \param[in]    baa
- * \param[out]   pnaindex  [optional] the boxa index in the baa
- * \param[in]    copyflag  L_COPY or L_CLONE
+ * \param[out]   pnaindex   [optional] the boxa index in the baa
+ * \param[in]    copyflag   L_COPY or L_CLONE
  * \return  boxa, or NULL on error
  *
  * <pre>
@@ -1525,9 +1693,9 @@ NUMA    *naindex;
  * \brief   boxaaFlattenAligned()
  *
  * \param[in]    baa
- * \param[in]    num number extracted from each
- * \param[in]    fillerbox [optional] that fills if necessary
- * \param[in]    copyflag  L_COPY or L_CLONE
+ * \param[in]    num         number extracted from each
+ * \param[in]    fillerbox   [optional] that fills if necessary
+ * \param[in]    copyflag    L_COPY or L_CLONE
  * \return  boxa, or NULL on error
  *
  * <pre>
@@ -1587,8 +1755,8 @@ BOX     *box;
  * \brief   boxaEncapsulateAligned()
  *
  * \param[in]    boxa
- * \param[in]    num number put into each boxa in the baa
- * \param[in]    copyflag  L_COPY or L_CLONE
+ * \param[in]    num        number put into each boxa in the baa
+ * \param[in]    copyflag   L_COPY or L_CLONE
  * \return  baa, or NULL on error
  *
  * <pre>
@@ -1698,11 +1866,11 @@ BOXAA    *baad;
  * \brief   boxaaAlignBox()
  *
  * \param[in]    baa
- * \param[in]    box to be aligned with the bext boxa in the baa, if possible
- * \param[in]    delta amount by which consecutive components can miss
- *                     in overlap and still be included in the array
- * \param[out]   pindex index of boxa with best overlap, or if none match,
- *                      this is the index of the next boxa to be generated
+ * \param[in]    box      to be aligned with bext boxa in the baa, if possible
+ * \param[in]    delta    amount by which consecutive components can miss
+ *                        in overlap and still be included in the array
+ * \param[out]   pindex   index of boxa with best overlap, or if none match,
+ *                        this is the index of the next boxa to be generated
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1711,7 +1879,7 @@ BOXAA    *baad;
  *          extent has the closest overlap with the input box.
  * </pre>
  */
-l_int32
+l_ok
 boxaaAlignBox(BOXAA    *baa,
               BOX      *box,
               l_int32   delta,
