@@ -29,23 +29,18 @@
  * <pre>
  *
  *      High-level interface for detection and correction
- *          l_int32      pixOrientCorrect()
+ *          PIX         *pixOrientCorrect()
  *
  *      Page orientation detection (pure rotation by 90 degree increments):
  *          l_int32      pixOrientDetect()
  *          l_int32      makeOrientDecision()
  *          l_int32      pixUpDownDetect()
- *          l_int32      pixUpDownDetectGeneral()
- *          l_int32      pixOrientDetectDwa()
- *          l_int32      pixUpDownDetectDwa()
- *          l_int32      pixUpDownDetectGeneralDwa()
  *
  *      Page mirror detection (flip 180 degrees about line in plane of image):
  *          l_int32      pixMirrorDetect()
- *          l_int32      pixMirrorDetectDwa()
  *
  *      Static debug helper
- *          void         pixDebugFlipDetect()
+ *          static void  pixDebugFlipDetect()
  *
  *  ===================================================================
  *
@@ -95,10 +90,10 @@
  *  rotation R, the set of 8 transformations decomposes into
  *  two subgroups linking {0, 3, 4, 7} and {1, 2, 5, 6} independently.
  *
- *  pixOrientDetect*() tests for a pure rotation (0, 90, 180, 270 degrees).
+ *  pixOrientDetect() tests for a pure rotation (0, 90, 180, 270 degrees).
  *  It doesn't change parity.
  *
- *  pixMirrorDetect*() tests for a horizontal flip about the vertical axis.
+ *  pixMirrorDetect() tests for a horizontal flip about the vertical axis.
  *  It changes parity.
  *
  *  The landscape/portrait rotation can be detected in two ways:
@@ -133,34 +128,28 @@
  *  of the signal comes from open regions of common lower-case
  *  letters such as 'e', 'c' and 'f'.
  *
- *  All operations are given in two implementations whose results are
- *  identical: rasterop morphology and dwa morphology.  The dwa
- *  implementations are between 2x and 3x faster.
- *
  *  The set of operations you actually use depends on your prior knowledge:
  *
  *  (1) If the page is known to be either rightside-up or upside-down, use
- *      either pixOrientDetect*() with pleftconf = NULL, or
- *      pixUpDownDetect*().   [The '*' refers to either the rasterop
- *      or dwa versions.]
+ *      either pixOrientDetect() with pleftconf = NULL, or
+ *      pixUpDownDetect().
  *
- *  (2) If any of the four orientations are possible, use pixOrientDetect*().
+ *  (2) If any of the four orientations are possible, use pixOrientDetect().
  *
  *  (3) If the text is horizontal and rightside-up, the only remaining
- *      degree of freedom is a left-right mirror flip: use
- *      pixMirrorDetect*().
+ *      degree of freedom is a left-right mirror flip: use pixMirrorDetect().
  *
  *  (4) If you have a relatively large amount of numbers on the page,
- *      us the slower pixUpDownDetectGeneral().
+ *      use the slower pixUpDownDetect().
  *
  *  We summarize the full orientation and mirror flip detection process:
  *
  *  (1) First determine which of the four 90 degree rotations
  *      causes the text to be rightside-up.  This can be done
- *      with either skew confidence or the pixOrientDetect*()
+ *      with either skew confidence or the pixOrientDetect()
  *      signals.  For the latter, see the table for pixOrientDetect().
  *
- *  (2) Then, with ascenders pointing up, apply pixMirrorDetect*().
+ *  (2) Then, with ascenders pointing up, apply pixMirrorDetect().
  *      In the normal situation the confidence confidence will be
  *      large and positive.  However, if mirror flipped, the
  *      confidence will be large and negative.
@@ -168,6 +157,12 @@
  *  A high-level interface, pixOrientCorrect() combines the detection
  *  of the orientation with the rotation decision and the rotation itself.
  *
+ *  For pedagogical reasons, we have included a dwa implementation of
+ *  this functionality, in flipdetectdwa.c.notused.  It shows by example
+ *  how to make a dwa implementation of an application that uses binary
+ *  morphological operations.  It is faster than the rasterop implementation,
+ *  but not by a large amount.
+ *
  *  Finally, use can be made of programs such as exiftool and convert to
  *  read exif camera orientation data in jpeg files and conditionally rotate.
  *  Here is an example shell script, made by Dan9er:
@@ -294,9 +289,9 @@ PIX       *pix1;
         return (PIX *)ERROR_PTR("pixs undefined or not 1 bpp", procName, NULL);
 
         /* Get confidences for orientation */
-    pixUpDownDetectDwa(pixs, &upconf, 0, debug);
+    pixUpDownDetect(pixs, &upconf, 0, 0, debug);
     pix1 = pixRotate90(pixs, 1);
-    pixUpDownDetectDwa(pix1, &leftconf, 0, debug);
+    pixUpDownDetect(pix1, &leftconf, 0, 0, debug);
     pixDestroy(&pix1);
     if (pupconf) *pupconf = upconf;
     if (pleftconf) *pleftconf = leftconf;
@@ -341,7 +336,6 @@ PIX       *pix1;
 
 /*----------------------------------------------------------------*
  *         Orientation detection (four 90 degree angles)          *
- *                      Rasterop implementation                   *
  *----------------------------------------------------------------*/
 /*!
  * \brief   pixOrientDetect()
@@ -426,10 +420,10 @@ PIX  *pix1;
         mincount = DefaultMinUpDownCount;
 
     if (pupconf)
-        pixUpDownDetect(pixs, pupconf, mincount, debug);
+        pixUpDownDetect(pixs, pupconf, mincount, 0, debug);
     if (pleftconf) {
         pix1 = pixRotate90(pixs, 1);
-        pixUpDownDetect(pix1, pleftconf, mincount, debug);
+        pixUpDownDetect(pix1, pleftconf, mincount, 0, debug);
         pixDestroy(&pix1);
     }
 
@@ -528,37 +522,6 @@ l_float32  absupconf, absleftconf;
  * \param[in]    pixs       1 bpp, deskewed, English text, 150 - 300 ppi
  * \param[out]   pconf      confidence that text is rightside-up
  * \param[in]    mincount   min number of up + down; use 0 for default
- * \param[in]    debug      1 for debug output; 0 otherwise
- * \return  0 if OK, 1 on error
- *
- * <pre>
- * Notes:
- *      (1) Special (typical, slightly faster) case, where the pixels
- *          identified through the HMT (hit-miss transform) are not
- *          clipped by a truncated word mask pixm.  See pixOrientDetect()
- *          and pixUpDownDetectGeneral() for details.
- *      (2) The returned confidence is the normalized difference
- *          between the number of detected up and down ascenders,
- *          assuming that the text is either rightside-up or upside-down
- *          and not rotated at a 90 degree angle.
- * </pre>
- */
-l_ok
-pixUpDownDetect(PIX        *pixs,
-                l_float32  *pconf,
-                l_int32     mincount,
-                l_int32     debug)
-{
-    return pixUpDownDetectGeneral(pixs, pconf, mincount, 0, debug);
-}
-
-
-/*!
- * \brief   pixUpDownDetectGeneral()
- *
- * \param[in]    pixs       1 bpp, deskewed, English text, 150 - 300 ppi
- * \param[out]   pconf      confidence that text is rightside-up
- * \param[in]    mincount   min number of up + down; use 0 for default
  * \param[in]    npixels    number of pixels removed from each side of word box
  * \param[in]    debug      1 for debug output; 0 otherwise
  * \return  0 if OK, 1 on error
@@ -566,10 +529,10 @@ pixUpDownDetect(PIX        *pixs,
  * <pre>
  * Notes:
  *      (1) See pixOrientDetect() for other details.
- *      (2) %conf is the normalized difference between the number of
- *          detected up and down ascenders, assuming that the text
- *          is either rightside-up or upside-down and not rotated
- *          at a 90 degree angle.
+ *      (2) The detected confidence %conf is the normalized difference
+ *          between the number of detected up and down ascenders,
+ *          assuming that the text is either rightside-up or upside-down
+ *          and not rotated at a 90 degree angle.
  *      (3) The typical mode of operation is %npixels == 0.
  *          If %npixels > 0, this removes HMT matches at the
  *          beginning and ending of "words."  This is useful for
@@ -592,18 +555,18 @@ pixUpDownDetect(PIX        *pixs,
  * </pre>
  */
 l_ok
-pixUpDownDetectGeneral(PIX        *pixs,
-                       l_float32  *pconf,
-                       l_int32     mincount,
-                       l_int32     npixels,
-                       l_int32     debug)
+pixUpDownDetect(PIX        *pixs,
+                l_float32  *pconf,
+                l_int32     mincount,
+                l_int32     npixels,
+                l_int32     debug)
 {
 l_int32    countup, countdown, nmax;
 l_float32  nup, ndown;
 PIX       *pix0, *pix1, *pix2, *pix3, *pixm;
 SEL       *sel1, *sel2, *sel3, *sel4;
 
-    PROCNAME("pixUpDownDetectGeneral");
+    PROCNAME("pixUpDownDetect");
 
     if (!pconf)
         return ERROR_INT("&conf not defined", procName, 1);
@@ -709,227 +672,8 @@ SEL       *sel1, *sel2, *sel3, *sel4;
 }
 
 
-/*----------------------------------------------------------------*
- *         Orientation detection (four 90 degree angles)          *
- *                         DWA implementation                     *
- *----------------------------------------------------------------*/
-/*!
- * \brief   pixOrientDetectDwa()
- *
- * \param[in]    pixs        1 bpp, deskewed, English text
- * \param[out]   pupconf     [optional] ; may be NULL
- * \param[out]   pleftconf   [optional] ; may be NULL
- * \param[in]    mincount    min number of up + down; use 0 for default
- * \param[in]    debug       1 for debug output; 0 otherwise
- * \return  0 if OK, 1 on error
- *
- * <pre>
- * Notes:
- *      (1) Same interface as for pixOrientDetect().  See notes
- *          there for usage.
- *      (2) Uses auto-gen'd code for the Sels defined at the
- *          top of this file, with some renaming of functions.
- *          The auto-gen'd code is in fliphmtgen.c, and can
- *          be generated by a simple executable; see prog/flipselgen.c.
- *      (3) This runs about 2.5 times faster than the pixOrientDetect().
- * </pre>
- */
-l_ok
-pixOrientDetectDwa(PIX        *pixs,
-                   l_float32  *pupconf,
-                   l_float32  *pleftconf,
-                   l_int32     mincount,
-                   l_int32     debug)
-{
-PIX  *pix1;
-
-    PROCNAME("pixOrientDetectDwa");
-
-    if (!pixs || pixGetDepth(pixs) != 1)
-        return ERROR_INT("pixs not defined or not 1 bpp", procName, 1);
-    if (!pupconf && !pleftconf)
-        return ERROR_INT("nothing to do", procName, 1);
-    if (mincount == 0)
-        mincount = DefaultMinUpDownCount;
-
-    if (pupconf)
-        pixUpDownDetectDwa(pixs, pupconf, mincount, debug);
-    if (pleftconf) {
-        pix1 = pixRotate90(pixs, 1);
-        pixUpDownDetectDwa(pix1, pleftconf, mincount, debug);
-        pixDestroy(&pix1);
-    }
-
-    return 0;
-}
-
-
-/*!
- * \brief   pixUpDownDetectDwa()
- *
- * \param[in]    pixs       1 bpp, deskewed, English text, 150 - 300 ppi
- * \param[out]   pconf      confidence that text is rightside-up
- * \param[in]    mincount   min number of up + down; use 0 for default
- * \param[in]    debug      1 for debug output; 0 otherwise
- * \return  0 if OK, 1 on error
- *
- * <pre>
- * Notes:
- *      (1) Faster (DWA) version of pixUpDownDetect().
- *      (2) This is a special case (but typical and slightly faster) of
- *          pixUpDownDetectGeneralDwa(), where the pixels identified
- *          through the HMT (hit-miss transform) are not clipped by
- *          a truncated word mask pixm.  See pixUpDownDetectGeneral()
- *          for usage and other details.
- *      (3) The returned confidence is the normalized difference
- *          between the number of detected up and down ascenders,
- *          assuming that the text is either rightside-up or upside-down
- *          and not rotated at a 90 degree angle.
- * </pre>
- */
-l_ok
-pixUpDownDetectDwa(PIX       *pixs,
-                  l_float32  *pconf,
-                  l_int32     mincount,
-                  l_int32     debug)
-{
-    return pixUpDownDetectGeneralDwa(pixs, pconf, mincount, 0, debug);
-}
-
-
-/*!
- * \brief   pixUpDownDetectGeneralDwa()
- *
- * \param[in]    pixs       1 bpp, deskewed, English text
- * \param[out]   pconf      confidence that text is rightside-up
- * \param[in]    mincount   min number of up + down; use 0 for default
- * \param[in]    npixels    number of pixels removed from each side of word box
- * \param[in]    debug      1 for debug output; 0 otherwise
- * \return  0 if OK, 1 on error
- *
- * <pre>
- * Notes:
- *      (1) See the notes in pixUpDownDetectGeneral() for usage.
- * </pre>
- */
-l_ok
-pixUpDownDetectGeneralDwa(PIX        *pixs,
-                          l_float32  *pconf,
-                          l_int32     mincount,
-                          l_int32     npixels,
-                          l_int32     debug)
-{
-char       flipsel1[] = "flipsel1";
-char       flipsel2[] = "flipsel2";
-char       flipsel3[] = "flipsel3";
-char       flipsel4[] = "flipsel4";
-l_int32    countup, countdown, nmax;
-l_float32  nup, ndown;
-PIX       *pixt, *pix0, *pix1, *pix2, *pix3, *pixm;
-
-    PROCNAME("pixUpDownDetectGeneralDwa");
-
-    if (!pconf)
-        return ERROR_INT("&conf not defined", procName, 1);
-    *pconf = 0.0;
-    if (!pixs || pixGetDepth(pixs) != 1)
-        return ERROR_INT("pixs not defined or not 1 bpp", procName, 1);
-    if (mincount == 0)
-        mincount = DefaultMinUpDownCount;
-    if (npixels < 0)
-        npixels = 0;
-
-        /* One of many reasonable pre-filtering sequences: (1, 8) and (30, 1).
-         * This closes holes in x-height characters and joins them at
-         * the x-height.  There is more noise in the descender detection
-         * from this, but it works fairly well. */
-    pixt = pixMorphSequenceDwa(pixs, "c1.8 + c30.1", 0);
-
-        /* Be sure to add the border before the flip DWA operations! */
-    pix0 = pixAddBorderGeneral(pixt, ADDED_BORDER, ADDED_BORDER,
-                                ADDED_BORDER, ADDED_BORDER, 0);
-    pixDestroy(&pixt);
-
-        /* Optionally, make a mask of the word bounding boxes, shortening
-         * each of them by a fixed amount at each end. */
-    pixm = NULL;
-    if (npixels > 0) {
-        l_int32  i, nbox, x, y, w, h;
-        BOX   *box;
-        BOXA  *boxa;
-        pix1 = pixMorphSequenceDwa(pix0, "o10.1", 0);
-        boxa = pixConnComp(pix1, NULL, 8);
-        pixm = pixCreateTemplate(pix1);
-        pixDestroy(&pix1);
-        nbox = boxaGetCount(boxa);
-        for (i = 0; i < nbox; i++) {
-            box = boxaGetBox(boxa, i, L_CLONE);
-            boxGetGeometry(box, &x, &y, &w, &h);
-            if (w > 2 * npixels)
-                pixRasterop(pixm, x + npixels, y - 6, w - 2 * npixels, h + 13,
-                            PIX_SET, NULL, 0, 0);
-            boxDestroy(&box);
-        }
-        boxaDestroy(&boxa);
-    }
-
-        /* Find the ascenders and optionally filter with pixm.
-         * For an explanation of the procedure used for counting the result
-         * of the HMT, see comments in pixUpDownDetectGeneral().  */
-    pix1 = pixFlipFHMTGen(NULL, pix0, flipsel1);
-    pix2 = pixFlipFHMTGen(NULL, pix0, flipsel2);
-    pixOr(pix1, pix1, pix2);
-    if (pixm)
-        pixAnd(pix1, pix1, pixm);
-    pix3 = pixReduceRankBinaryCascade(pix1, 1, 1, 0, 0);
-    pixCountPixels(pix3, &countup, NULL);
-    pixDestroy(&pix1);
-    pixDestroy(&pix2);
-    pixDestroy(&pix3);
-
-        /* Find the ascenders and optionally filter with pixm. */
-    pix1 = pixFlipFHMTGen(NULL, pix0, flipsel3);
-    pix2 = pixFlipFHMTGen(NULL, pix0, flipsel4);
-    pixOr(pix1, pix1, pix2);
-    if (pixm)
-        pixAnd(pix1, pix1, pixm);
-    pix3 = pixReduceRankBinaryCascade(pix1, 1, 1, 0, 0);
-    pixCountPixels(pix3, &countdown, NULL);
-    pixDestroy(&pix1);
-    pixDestroy(&pix2);
-    pixDestroy(&pix3);
-
-        /* Evaluate statistically, generating a confidence that is
-         * related to the probability with a gaussian distribution. */
-    nup = (l_float32)(countup);
-    ndown = (l_float32)(countdown);
-    nmax = L_MAX(countup, countdown);
-    if (nmax > mincount)
-        *pconf = 2. * ((nup - ndown) / sqrt(nup + ndown));
-
-    if (debug) {
-        if (pixm) {
-            lept_mkdir("lept/orient");
-            pixWriteDebug("/tmp/lept/orient/pixm2.png", pixm, IFF_PNG);
-        }
-        lept_stderr("nup = %7.3f, ndown = %7.3f, conf = %7.3f\n",
-                nup, ndown, *pconf);
-        if (*pconf > DefaultMinUpDownConf)
-            lept_stderr("Text is rightside-up\n");
-        if (*pconf < -DefaultMinUpDownConf)
-            lept_stderr("Text is upside-down\n");
-    }
-
-    pixDestroy(&pix0);
-    pixDestroy(&pixm);
-    return 0;
-}
-
-
-
 /*----------------------------------------------------------------*
  *                     Left-right mirror detection                *
- *                       Rasterop implementation                  *
  *----------------------------------------------------------------*/
 /*!
  * \brief   pixMirrorDetect()
@@ -1045,94 +789,6 @@ SEL       *sel1, *sel2;
 }
 
 
-/*----------------------------------------------------------------*
- *                     Left-right mirror detection                *
- *                          DWA implementation                    *
- *----------------------------------------------------------------*/
-/*!
- * \brief   pixMirrorDetectDwa()
- *
- * \param[in]    pixs       1 bpp, deskewed, English text
- * \param[out]   pconf      confidence that text is not LR mirror reversed
- * \param[in]    mincount   min number of left + right; use 0 for default
- * \param[in]    debug      1 for debug output; 0 otherwise
- * \return  0 if OK, 1 on error
- *
- * <pre>
- * Notes:
- *      (1) We assume the text is horizontally oriented, with
- *          ascenders going up.
- *      (2) See notes in pixMirrorDetect().
- * </pre>
- */
-l_ok
-pixMirrorDetectDwa(PIX        *pixs,
-                   l_float32  *pconf,
-                   l_int32     mincount,
-                   l_int32     debug)
-{
-char       flipsel1[] = "flipsel1";
-char       flipsel2[] = "flipsel2";
-l_int32    count1, count2, nmax;
-l_float32  nleft, nright;
-PIX       *pix0, *pix1, *pix2, *pix3;
-
-    PROCNAME("pixMirrorDetectDwa");
-
-    if (!pconf)
-        return ERROR_INT("&conf not defined", procName, 1);
-    *pconf = 0.0;
-    if (!pixs || pixGetDepth(pixs) != 1)
-        return ERROR_INT("pixs not defined or not 1 bpp", procName, 1);
-    if (mincount == 0)
-        mincount = DefaultMinMirrorFlipCount;
-
-        /* Fill x-height characters but not space between them, sort of. */
-    pix3 = pixMorphSequenceDwa(pixs, "d1.30", 0);
-    pixXor(pix3, pix3, pixs);
-    pix0 = pixMorphSequenceDwa(pixs, "c15.1", 0);
-    pixXor(pix0, pix0, pixs);
-    pixAnd(pix0, pix0, pix3);
-    pixOr(pix3, pix0, pixs);
-    pixDestroy(&pix0);
-    pix0 = pixAddBorderGeneral(pix3, ADDED_BORDER, ADDED_BORDER,
-                                ADDED_BORDER, ADDED_BORDER, 0);
-    pixDestroy(&pix3);
-
-        /* Filter the right-facing characters. */
-    pix1 = pixFlipFHMTGen(NULL, pix0, flipsel1);
-    pix3 = pixReduceRankBinaryCascade(pix1, 1, 1, 0, 0);
-    pixCountPixels(pix3, &count1, NULL);
-    pixDestroy(&pix1);
-    pixDestroy(&pix3);
-
-        /* Filter the left-facing characters. */
-    pix2 = pixFlipFHMTGen(NULL, pix0, flipsel2);
-    pix3 = pixReduceRankBinaryCascade(pix2, 1, 1, 0, 0);
-    pixCountPixels(pix3, &count2, NULL);
-    pixDestroy(&pix2);
-    pixDestroy(&pix3);
-
-    pixDestroy(&pix0);
-    nright = (l_float32)count1;
-    nleft = (l_float32)count2;
-    nmax = L_MAX(count1, count2);
-
-    if (nmax > mincount)
-        *pconf = 2. * ((nright - nleft) / sqrt(nright + nleft));
-
-    if (debug) {
-        lept_stderr("nright = %f, nleft = %f\n", nright, nleft);
-        if (*pconf > DefaultMinMirrorFlipConf)
-            lept_stderr("Text is not mirror reversed\n");
-        if (*pconf < -DefaultMinMirrorFlipConf)
-            lept_stderr("Text is mirror reversed\n");
-    }
-
-    return 0;
-}
-
-
 /*----------------------------------------------------------------*
  *                        Static debug helper                     *
  *----------------------------------------------------------------*/
