@@ -40,7 +40,6 @@
  *          FPIX          *fpixCreateTemplate()
  *          FPIX          *fpixClone()
  *          FPIX          *fpixCopy()
- *          l_int32        fpixResizeImageData()
  *          void           fpixDestroy()
  *
  *    FPix accessors
@@ -82,7 +81,6 @@
  *          DPIX          *dpixCreateTemplate()
  *          DPIX          *dpixClone()
  *          DPIX          *dpixCopy()
- *          l_int32        dpixResizeImageData()
  *          void           dpixDestroy()
  *
  *    DPix accessors
@@ -130,9 +128,9 @@
 #include <string.h>
 #include "allheaders.h"
 
-    /* Bounds on initial array size */
-static const l_uint32  MaxPtrArraySize = 100000;
-static const l_int32 InitialPtrArraySize = 20;      /*!< n'importe quoi */
+    /* Bounds on array sizes */
+static const size_t  MaxPtrArraySize = 100000;
+static const size_t  InitialPtrArraySize = 20;      /*!< n'importe quoi */
 
     /* Static functions */
 static l_int32 fpixaExtendArray(FPIXA *fpixa);
@@ -250,66 +248,27 @@ fpixClone(FPIX  *fpix)
 /*!
  * \brief   fpixCopy()
  *
- * \param[in]    fpixd    [optional] can be null, or equal to fpixs,
- *                        or different from fpixs
  * \param[in]    fpixs
  * \return  fpixd, or NULL on error
- *
- * <pre>
- * Notes:
- *      (1) There are three cases:
- *            (a) fpixd == null  (makes a new fpix; refcount = 1)
- *            (b) fpixd == fpixs  (no-op)
- *            (c) fpixd != fpixs  (data copy; no change in refcount)
- *          If the refcount of fpixd > 1, case (c) will side-effect
- *          these handles.
- *      (2) The general pattern of use is:
- *             fpixd = fpixCopy(fpixd, fpixs);
- *          This will work for all three cases.
- *          For clarity when the case is known, you can use:
- *            (a) fpixd = fpixCopy(NULL, fpixs);
- *            (c) fpixCopy(fpixd, fpixs);
- *      (3) For case (c), we check if fpixs and fpixd are the same size.
- *          If so, the data is copied directly.
- *          Otherwise, the data is reallocated to the correct size
- *          and the copy proceeds.  The refcount of fpixd is unchanged.
- *      (4) This operation, like all others that may involve a pre-existing
- *          fpixd, will side-effect any existing clones of fpixd.
- * </pre>
  */
 FPIX *
-fpixCopy(FPIX  *fpixd,   /* can be null */
-         FPIX  *fpixs)
+fpixCopy(FPIX  *fpixs)
 {
 l_int32     w, h, bytes;
 l_float32  *datas, *datad;
+FPIX       *fpixd;
 
     PROCNAME("fpixCopy");
 
     if (!fpixs)
         return (FPIX *)ERROR_PTR("fpixs not defined", procName, NULL);
-    if (fpixs == fpixd)
-        return fpixd;
 
         /* Total bytes in image data */
     fpixGetDimensions(fpixs, &w, &h);
     bytes = 4 * w * h;
 
-        /* If we're making a new fpix ... */
-    if (!fpixd) {
-        if ((fpixd = fpixCreateTemplate(fpixs)) == NULL)
-            return (FPIX *)ERROR_PTR("fpixd not made", procName, NULL);
-        datas = fpixGetData(fpixs);
-        datad = fpixGetData(fpixd);
-        memcpy(datad, datas, bytes);
-        return fpixd;
-    }
-
-        /* Reallocate image data if sizes are different */
-    fpixResizeImageData(fpixd, fpixs);
-
-        /* Copy data */
-    fpixCopyResolution(fpixd, fpixs);
+    if ((fpixd = fpixCreateTemplate(fpixs)) == NULL)
+        return (FPIX *)ERROR_PTR("fpixd not made", procName, NULL);
     datas = fpixGetData(fpixs);
     datad = fpixGetData(fpixd);
     memcpy(datad, datas, bytes);
@@ -317,51 +276,6 @@ l_float32  *datas, *datad;
 }
 
 
-/*!
- * \brief   fpixResizeImageData()
- *
- * \param[in]    fpixd, fpixs
- * \return  0 if OK, 1 on error
- *
- * <pre>
- * Notes:
- *      (1) If the data sizes differ, this destroys the existing
- *          data in fpixd and allocates a new, uninitialized, data array
- *          of the same size as the data in fpixs.  Otherwise, this
- *          doesn't do anything.
- * </pre>
- */
-l_ok
-fpixResizeImageData(FPIX  *fpixd,
-                    FPIX  *fpixs)
-{
-l_int32     ws, hs, wd, hd, bytes;
-l_float32  *data;
-
-    PROCNAME("fpixResizeImageData");
-
-    if (!fpixs)
-        return ERROR_INT("fpixs not defined", procName, 1);
-    if (!fpixd)
-        return ERROR_INT("fpixd not defined", procName, 1);
-
-    fpixGetDimensions(fpixs, &ws, &hs);
-    fpixGetDimensions(fpixd, &wd, &hd);
-    if (ws == wd && hs == hd)  /* nothing to do */
-        return 0;
-
-    fpixSetDimensions(fpixd, ws, hs);
-    fpixSetWpl(fpixd, ws);
-    bytes = 4 * ws * hs;
-    data = fpixGetData(fpixd);
-    if (data) LEPT_FREE(data);
-    if ((data = (l_float32 *)LEPT_MALLOC(bytes)) == NULL)
-        return ERROR_INT("LEPT_MALLOC fail for data", procName, 1);
-    fpixSetData(fpixd, data);
-    return 0;
-}
-
-
 /*!
  * \brief   fpixDestroy()
  *
@@ -397,9 +311,7 @@ FPIX       *fpix;
             LEPT_FREE(data);
         LEPT_FREE(fpix);
     }
-
     *pfpix = NULL;
-    return;
 }
 
 
@@ -458,7 +370,7 @@ fpixSetDimensions(FPIX     *fpix,
  * \brief   fpixGetWpl()
  *
  * \param[in]    fpix
- * \return  wpl, or UNDEF on error
+ * \return  wpl, or 0 on error
  */
 l_int32
 fpixGetWpl(FPIX  *fpix)
@@ -466,7 +378,7 @@ fpixGetWpl(FPIX  *fpix)
     PROCNAME("fpixGetWpl");
 
     if (!fpix)
-        return ERROR_INT("fpix not defined", procName, UNDEF);
+        return ERROR_INT("fpix not defined", procName, 0);
     return fpix->wpl;
 }
 
@@ -729,11 +641,7 @@ FPIXA  *fpixa;
     fpixa->n = 0;
     fpixa->nalloc = n;
     fpixa->refcount = 1;
-    if ((fpixa->fpix = (FPIX **)LEPT_CALLOC(n, sizeof(FPIX *))) == NULL) {
-        fpixaDestroy(&fpixa);
-        return (FPIXA *)ERROR_PTR("fpixa ptrs not made", procName, NULL);
-    }
-
+    fpixa->fpix = (FPIX **)LEPT_CALLOC(n, sizeof(FPIX *));
     return fpixa;
 }
 
@@ -824,9 +732,7 @@ FPIXA   *fpixa;
         LEPT_FREE(fpixa->fpix);
         LEPT_FREE(fpixa);
     }
-
     *pfpixa = NULL;
-    return;
 }
 
 
@@ -859,7 +765,7 @@ FPIX    *fpixc;
     if (copyflag == L_INSERT)
         fpixc = fpix;
     else if (copyflag == L_COPY)
-        fpixc = fpixCopy(NULL, fpix);
+        fpixc = fpixCopy(fpix);
     else if (copyflag == L_CLONE)
         fpixc = fpixClone(fpix);
     else
@@ -868,11 +774,15 @@ FPIX    *fpixc;
         return ERROR_INT("fpixc not made", procName, 1);
 
     n = fpixaGetCount(fpixa);
-    if (n >= fpixa->nalloc)
-        fpixaExtendArray(fpixa);
+    if (n >= fpixa->nalloc) {
+        if (fpixaExtendArray(fpixa)) {
+            if (copyflag != L_INSERT)
+                fpixDestroy(&fpixc);
+            return ERROR_INT("extension failed", procName, 1);
+        }
+    }
     fpixa->fpix[n] = fpixc;
     fpixa->n++;
-
     return 0;
 }
 
@@ -886,6 +796,7 @@ FPIX    *fpixc;
  * <pre>
  * Notes:
  *      (1) Doubles the size of the fpixa ptr array.
+ *      (2) The max number of fpix ptrs is 100000.
  * </pre>
  */
 static l_int32
@@ -909,25 +820,35 @@ fpixaExtendArray(FPIXA  *fpixa)
  *
  * <pre>
  * Notes:
- *      (1) If necessary, reallocs new fpixa ptrs array to %size.
+ *      (1) If necessary, reallocs new fpix ptr array to %size.
+ *      (2) The max number of fpix ptrs is 100K.
  * </pre>
  */
 static l_int32
 fpixaExtendArrayToSize(FPIXA   *fpixa,
                        l_int32  size)
 {
+size_t  oldsize, newsize;
+
     PROCNAME("fpixaExtendArrayToSize");
 
     if (!fpixa)
         return ERROR_INT("fpixa not defined", procName, 1);
-
-    if (size > fpixa->nalloc) {
-        if ((fpixa->fpix = (FPIX **)reallocNew((void **)&fpixa->fpix,
-                                 sizeof(FPIX *) * fpixa->nalloc,
-                                 size * sizeof(FPIX *))) == NULL)
-            return ERROR_INT("new ptr array not returned", procName, 1);
-        fpixa->nalloc = size;
+    if (fpixa->nalloc > MaxPtrArraySize)  /* belt & suspenders */
+        return ERROR_INT("fpixa has too many ptrs", procName, 1);
+    if (size > MaxPtrArraySize)
+        return ERROR_INT("size > 100K ptrs; too large", procName, 1);
+    if (size <= fpixa->nalloc) {
+        L_INFO("size too small; no extension\n", procName);
+        return 0;
     }
+
+    oldsize = fpixa->nalloc * sizeof(FPIX *);
+    newsize = size * sizeof(FPIX *);
+    if ((fpixa->fpix = (FPIX **)reallocNew((void **)&fpixa->fpix,
+                                           oldsize, newsize)) == NULL)
+        return ERROR_INT("new ptr array not returned", procName, 1);
+    fpixa->nalloc = size;
     return 0;
 }
 
@@ -995,7 +916,7 @@ fpixaGetFPix(FPIXA   *fpixa,
         return (FPIX *)ERROR_PTR("index not valid", procName, NULL);
 
     if (accesstype == L_COPY)
-        return fpixCopy(NULL, fpixa->fpix[index]);
+        return fpixCopy(fpixa->fpix[index]);
     else if (accesstype == L_CLONE)
         return fpixClone(fpixa->fpix[index]);
     else
@@ -1242,7 +1163,6 @@ dpixClone(DPIX  *dpix)
     if (!dpix)
         return (DPIX *)ERROR_PTR("dpix not defined", procName, NULL);
     dpixChangeRefcount(dpix, 1);
-
     return dpix;
 }
 
@@ -1250,66 +1170,27 @@ dpixClone(DPIX  *dpix)
 /*!
  * \brief   dpixCopy()
  *
- * \param[in]    dpixd    [optional] can be null, or equal to dpixs,
- *                        or different from dpixs
  * \param[in]    dpixs
  * \return  dpixd, or NULL on error
- *
- * <pre>
- * Notes:
- *      (1) There are three cases:
- *            (a) dpixd == null  (makes a new dpix; refcount = 1)
- *            (b) dpixd == dpixs  (no-op)
- *            (c) dpixd != dpixs  (data copy; no change in refcount)
- *          If the refcount of dpixd > 1, case (c) will side-effect
- *          these handles.
- *      (2) The general pattern of use is:
- *             dpixd = dpixCopy(dpixd, dpixs);
- *          This will work for all three cases.
- *          For clarity when the case is known, you can use:
- *            (a) dpixd = dpixCopy(NULL, dpixs);
- *            (c) dpixCopy(dpixd, dpixs);
- *      (3) For case (c), we check if dpixs and dpixd are the same size.
- *          If so, the data is copied directly.
- *          Otherwise, the data is reallocated to the correct size
- *          and the copy proceeds.  The refcount of dpixd is unchanged.
- *      (4) This operation, like all others that may involve a pre-existing
- *          dpixd, will side-effect any existing clones of dpixd.
- * </pre>
  */
 DPIX *
-dpixCopy(DPIX  *dpixd,   /* can be null */
-         DPIX  *dpixs)
+dpixCopy(DPIX  *dpixs)
 {
 l_int32     w, h, bytes;
 l_float64  *datas, *datad;
+DPIX       *dpixd;
 
     PROCNAME("dpixCopy");
 
     if (!dpixs)
         return (DPIX *)ERROR_PTR("dpixs not defined", procName, NULL);
-    if (dpixs == dpixd)
-        return dpixd;
 
         /* Total bytes in image data */
     dpixGetDimensions(dpixs, &w, &h);
     bytes = 8 * w * h;
 
-        /* If we're making a new dpix ... */
-    if (!dpixd) {
-        if ((dpixd = dpixCreateTemplate(dpixs)) == NULL)
-            return (DPIX *)ERROR_PTR("dpixd not made", procName, NULL);
-        datas = dpixGetData(dpixs);
-        datad = dpixGetData(dpixd);
-        memcpy(datad, datas, bytes);
-        return dpixd;
-    }
-
-        /* Reallocate image data if sizes are different */
-    dpixResizeImageData(dpixd, dpixs);
-
-        /* Copy data */
-    dpixCopyResolution(dpixd, dpixs);
+    if ((dpixd = dpixCreateTemplate(dpixs)) == NULL)
+        return (DPIX *)ERROR_PTR("dpixd not made", procName, NULL);
     datas = dpixGetData(dpixs);
     datad = dpixGetData(dpixd);
     memcpy(datad, datas, bytes);
@@ -1317,43 +1198,6 @@ l_float64  *datas, *datad;
 }
 
 
-/*!
- * \brief   dpixResizeImageData()
- *
- * \param[in]    dpixd, dpixs
- * \return  0 if OK, 1 on error
- */
-l_ok
-dpixResizeImageData(DPIX  *dpixd,
-                    DPIX  *dpixs)
-{
-l_int32     ws, hs, wd, hd, bytes;
-l_float64  *data;
-
-    PROCNAME("dpixResizeImageData");
-
-    if (!dpixs)
-        return ERROR_INT("dpixs not defined", procName, 1);
-    if (!dpixd)
-        return ERROR_INT("dpixd not defined", procName, 1);
-
-    dpixGetDimensions(dpixs, &ws, &hs);
-    dpixGetDimensions(dpixd, &wd, &hd);
-    if (ws == wd && hs == hd)  /* nothing to do */
-        return 0;
-
-    dpixSetDimensions(dpixd, ws, hs);
-    dpixSetWpl(dpixd, ws);  /* 8 byte words */
-    bytes = 8 * ws * hs;
-    data = dpixGetData(dpixd);
-    if (data) LEPT_FREE(data);
-    if ((data = (l_float64 *)LEPT_MALLOC(bytes)) == NULL)
-        return ERROR_INT("LEPT_MALLOC fail for data", procName, 1);
-    dpixSetData(dpixd, data);
-    return 0;
-}
-
-
 /*!
  * \brief   dpixDestroy()
  *
@@ -1389,9 +1233,7 @@ DPIX       *dpix;
             LEPT_FREE(data);
         LEPT_FREE(dpix);
     }
-
     *pdpix = NULL;
-    return;
 }
 
 
@@ -1450,7 +1292,7 @@ dpixSetDimensions(DPIX     *dpix,
  * \brief   dpixGetWpl()
  *
  * \param[in]    dpix
- * \return  wpl, or UNDEF on error
+ * \return  wpl, or 0 on error
  */
 l_int32
 dpixGetWpl(DPIX  *dpix)
@@ -1458,7 +1300,7 @@ dpixGetWpl(DPIX  *dpix)
     PROCNAME("dpixGetWpl");
 
     if (!dpix)
-        return ERROR_INT("dpix not defined", procName, 1);
+        return ERROR_INT("dpix not defined", procName, 0);
     return dpix->wpl;
 }
 
@@ -1916,6 +1758,9 @@ FILE    *fp;
     if ((fp = open_memstream((char **)pdata, psize)) == NULL)
         return ERROR_INT("stream not opened", procName, 1);
     ret = fpixWriteStream(fp, fpix);
+    fputc('\0', fp);
+    fclose(fp);
+    *psize = *psize - 1;
 #else
     L_INFO("work-around: writing to a temp file\n", procName);
   #ifdef _WIN32
@@ -1928,8 +1773,8 @@ FILE    *fp;
     ret = fpixWriteStream(fp, fpix);
     rewind(fp);
     *pdata = l_binaryReadStream(fp, psize);
-#endif  /* HAVE_FMEMOPEN */
     fclose(fp);
+#endif  /* HAVE_FMEMOPEN */
     return ret;
 }
 
@@ -1937,7 +1782,7 @@ FILE    *fp;
 /*!
  * \brief   fpixEndianByteSwap()
  *
- * \param[in]    fpixd     can be equal to fpixs or NULL
+ * \param[in]    fpixd     [optional] can be either NULL, or equal to fpixs
  * \param[in]    fpixs
  * \return  fpixd always
  *
@@ -1971,7 +1816,8 @@ fpixEndianByteSwap(FPIX  *fpixd,
     l_uint32   word;
 
         fpixGetDimensions(fpixs, &w, &h);
-        fpixd = fpixCopy(fpixd, fpixs);  /* no copy if fpixd == fpixs */
+        if (!fpixd)
+            fpixd = fpixCopy(fpixs);
 
         data = (l_uint32 *)fpixGetData(fpixd);
         for (i = 0; i < h; i++) {
@@ -2214,6 +2060,9 @@ FILE    *fp;
     if ((fp = open_memstream((char **)pdata, psize)) == NULL)
         return ERROR_INT("stream not opened", procName, 1);
     ret = dpixWriteStream(fp, dpix);
+    fputc('\0', fp);
+    fclose(fp);
+    *psize = *psize - 1;
 #else
     L_INFO("work-around: writing to a temp file\n", procName);
   #ifdef _WIN32
@@ -2226,8 +2075,8 @@ FILE    *fp;
     ret = dpixWriteStream(fp, dpix);
     rewind(fp);
     *pdata = l_binaryReadStream(fp, psize);
-#endif  /* HAVE_FMEMOPEN */
     fclose(fp);
+#endif  /* HAVE_FMEMOPEN */
     return ret;
 }
 
@@ -2235,7 +2084,7 @@ FILE    *fp;
 /*!
  * \brief   dpixEndianByteSwap()
  *
- * \param[in]    dpixd     can be equal to dpixs or NULL
+ * \param[in]    dpixd     [optional] can be either NULL, or equal to dpixs
  * \param[in]    dpixs
  * \return  dpixd always
  *
@@ -2269,7 +2118,8 @@ dpixEndianByteSwap(DPIX  *dpixd,
     l_uint32   word;
 
         dpixGetDimensions(dpixs, &w, &h);
-        dpixd = dpixCopy(dpixd, dpixs);  /* no copy if dpixd == dpixs */
+        if (!dpixd)
+            dpixd = dpixCopy(dpixs);
 
         data = (l_uint32 *)dpixGetData(dpixd);
         for (i = 0; i < h; i++) {
