@@ -119,6 +119,10 @@
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
     /* Special static operations for 3x1, 1x3 and 3x3 structuring elements */
@@ -641,9 +645,9 @@ cleanup:
 /*!
  * \brief   pixErodeGray3()
  *
- * \param[in]    pixs 8 bpp, not cmapped
- * \param[in]    hsize  1 or 3
- * \param[in]    vsize  1 or 3
+ * \param[in]    pixs    8 bpp, not cmapped
+ * \param[in]    hsize   1 or 3
+ * \param[in]    vsize   1 or 3
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -701,7 +705,7 @@ PIX  *pixt, *pixb, *pixbd, *pixd;
 /*!
  * \brief   pixErodeGray3h()
  *
- * \param[in]    pixs 8 bpp, not cmapped
+ * \param[in]    pixs    8 bpp, not cmapped
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -725,8 +729,7 @@ PIX       *pixd;
     if (pixGetDepth(pixs) != 8)
         return (PIX *)ERROR_PTR("pixs not 8 bpp", procName, NULL);
 
-    pixd = pixCreateTemplateNoInit(pixs);
-    pixSetBorderVal(pixd, 4, 8, 2, 8, 0);  /* only to silence valgrind */
+    pixd = pixCreateTemplate(pixs);
     pixGetDimensions(pixs, &w, &h, NULL);
     datas = pixGetData(pixs);
     datad = pixGetData(pixd);
@@ -766,7 +769,7 @@ PIX       *pixd;
 /*!
  * \brief   pixErodeGray3v()
  *
- * \param[in]    pixs 8 bpp, not cmapped
+ * \param[in]    pixs    8 bpp, not cmapped
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -793,7 +796,7 @@ PIX       *pixd;
     if (pixGetDepth(pixs) != 8)
         return (PIX *)ERROR_PTR("pixs not 8 bpp", procName, NULL);
 
-    pixd = pixCreateTemplateNoInit(pixs);
+    pixd = pixCreateTemplate(pixs);
     pixGetDimensions(pixs, &w, &h, NULL);
     datas = pixGetData(pixs);
     datad = pixGetData(pixd);
@@ -833,9 +836,9 @@ PIX       *pixd;
 /*!
  * \brief   pixDilateGray3()
  *
- * \param[in]    pixs 8 bpp, not cmapped
- * \param[in]    hsize  1 or 3
- * \param[in]    vsize  1 or 3
+ * \param[in]    pixs    8 bpp, not cmapped
+ * \param[in]    hsize   1 or 3
+ * \param[in]    vsize   1 or 3
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -888,7 +891,7 @@ PIX  *pixt, *pixb, *pixbd, *pixd;
 /*!
  * \brief   pixDilateGray3h()
  *
- * \param[in]    pixs 8 bpp, not cmapped
+ * \param[in]    pixs    8 bpp, not cmapped
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -912,8 +915,7 @@ PIX       *pixd;
     if (pixGetDepth(pixs) != 8)
         return (PIX *)ERROR_PTR("pixs not 8 bpp", procName, NULL);
 
-    pixd = pixCreateTemplateNoInit(pixs);
-    pixSetBorderVal(pixd, 4, 8, 2, 8, 0);  /* only to silence valgrind */
+    pixd = pixCreateTemplate(pixs);
     pixGetDimensions(pixs, &w, &h, NULL);
     datas = pixGetData(pixs);
     datad = pixGetData(pixd);
@@ -953,7 +955,7 @@ PIX       *pixd;
 /*!
  * \brief   pixDilateGray3v()
  *
- * \param[in]    pixs 8 bpp, not cmapped
+ * \param[in]    pixs    8 bpp, not cmapped
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -977,7 +979,7 @@ PIX       *pixd;
     if (pixGetDepth(pixs) != 8)
         return (PIX *)ERROR_PTR("pixs not 8 bpp", procName, NULL);
 
-    pixd = pixCreateTemplateNoInit(pixs);
+    pixd = pixCreateTemplate(pixs);
     pixGetDimensions(pixs, &w, &h, NULL);
     datas = pixGetData(pixs);
     datad = pixGetData(pixd);
@@ -1017,9 +1019,9 @@ PIX       *pixd;
 /*!
  * \brief   pixOpenGray3()
  *
- * \param[in]    pixs 8 bpp, not cmapped
- * \param[in]    hsize  1 or 3
- * \param[in]    vsize  1 or 3
+ * \param[in]    pixs    8 bpp, not cmapped
+ * \param[in]    hsize   1 or 3
+ * \param[in]    vsize   1 or 3
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -1085,9 +1087,9 @@ PIX  *pixt, *pixb, *pixbd, *pixd;
 /*!
  * \brief   pixCloseGray3()
  *
- * \param[in]    pixs 8 bpp, not cmapped
- * \param[in]    hsize  1 or 3
- * \param[in]    vsize  1 or 3
+ * \param[in]    pixs    8 bpp, not cmapped
+ * \param[in]    hsize   1 or 3
+ * \param[in]    vsize   1 or 3
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -1154,12 +1156,15 @@ PIX  *pixt, *pixb, *pixbd, *pixd;
 /*!
  * \brief   dilateGrayLow()
  *
- * \param[in]    datad, w, h, wpld 8 bpp image
- * \param[in]    datas, wpls  8 bpp image, of same dimensions
- * \param[in]    size  full length of SEL; restricted to odd numbers
- * \param[in]    direction  L_HORIZ or L_VERT
- * \param[in]    buffer  holds full line or column of src image pixels
- * \param[in]    maxarray  array of dimension 2*size+1
+ * \param[in]    datad       8 bpp dsst image
+ * \param[in]    w, h        dimensions of src and dest
+ * \param[in]    wpld        words/line of dest
+ * \param[in]    datas       8 bpp src image
+ * \param[in]    wpls        words/line of src
+ * \param[in]    size        full length of SEL; restricted to odd numbers
+ * \param[in]    direction   L_HORIZ or L_VERT
+ * \param[in]    buffer      holds full line or column of src image pixels
+ * \param[in]    maxarray    array of dimension 2*size+1
  * \return  void
  *
  * <pre>
@@ -1267,12 +1272,15 @@ l_uint32  *lines, *lined;
 /*!
  * \brief   erodeGrayLow()
  *
- * \param[in]    datad, w, h, wpld 8 bpp image
- * \param[in]    datas, wpls  8 bpp image, of same dimensions
- * \param[in]    size  full length of SEL; restricted to odd numbers
- * \param[in]    direction  L_HORIZ or L_VERT
- * \param[in]    buffer  holds full line or column of src image pixels
- * \param[in]    minarray  array of dimension 2*size+1
+ * \param[in]    datad       8 bpp dsst image
+ * \param[in]    w, h        dimensions of src and dest
+ * \param[in]    wpld        words/line of dest
+ * \param[in]    datas       8 bpp src image
+ * \param[in]    wpls        words/line of src
+ * \param[in]    size        full length of SEL; restricted to odd numbers
+ * \param[in]    direction   L_HORIZ or L_VERT
+ * \param[in]    buffer      holds full line or column of src image pixels
+ * \param[in]    minarray    array of dimension 2*size+1
  * \return  void
  *
  * <pre>
