@@ -165,9 +165,7 @@ L_KERNEL  *kel;
         LEPT_FREE(kel->data[i]);
     LEPT_FREE(kel->data);
     LEPT_FREE(kel);
-
     *pkel = NULL;
-    return;
 }
 
 
@@ -515,8 +513,7 @@ l_float32  **array;
     if (sy <= 0 || sy > MaxArraySize)
         return (l_float32 **)ERROR_PTR("sy out of bounds", procName, NULL);
 
-    if ((array = (l_float32 **)LEPT_CALLOC(sy, sizeof(l_float32 *))) == NULL)
-        return (l_float32 **)ERROR_PTR("ptr array not made", procName, NULL);
+    array = (l_float32 **)LEPT_CALLOC(sy, sizeof(l_float32 *));
     for (i = 0; i < sy; i++)
         array[i] = (l_float32 *)LEPT_CALLOC(sx, sizeof(l_float32));
     return array;
@@ -1200,7 +1197,7 @@ L_KERNEL  *kel;
  *          normalized and un-normalized convolution will be the same
  *          as when convolving with pixConvolve() using the full kernel.
  *      (3) The trick for the un-normalized convolution is to have the
- *          product of the two kernel elemets at (cx,cy) be equal to %max,
+ *          product of the two kernel elements at (cx,cy) be equal to %max,
  *          not max**2.  That's why %max for kely is 1.0.  If instead
  *          we use sqrt(%max) for both, the results are slightly less
  *          accurate, when compared to using the full kernel in
