@@ -2191,9 +2191,9 @@ PIX       *pixw, *pixh;  /* keeps the width and height for the largest */
  *      (1) Computation is similar to pixFindLargestRectangle(), but allows
  *          a different set of results to choose from.
  *      (2) Select the fast scan direction.  Then, scanning in the slow
- *          direction, finds the longest run of ON pixels in the fast
- *          scan direction and look for the first first run that is longer
- *          than %fract of the dimension.  Continues until a shorter run
+ *          direction, find the longest run of ON pixels in the fast
+ *          scan direction and look for the first run that is longer
+ *          than %fract of the dimension.  Continue until a shorter run
  *          is found.  This generates a box of ON pixels fitting into the c.c.
  *      (3) Do this from both slow scan directions and use %select to get
  *          a resulting box from these two.
@@ -2208,7 +2208,7 @@ pixFindRectangleInCC(PIX       *pixs,
                      l_int32    select,
                      l_int32    debug)
 {
-l_int32  x, y, i, j, w, h, w1, h1, w2, h2, found, res;
+l_int32  x, y, i, w, h, w1, h1, w2, h2, found, res;
 l_int32  xfirst, xlast, xstart, yfirst, ylast, length;
 BOX     *box1, *box2, *box3, *box4, *box5;
 PIX     *pix1, *pix2, *pixdb1, *pixdb2;
@@ -2274,7 +2274,8 @@ PIXA    *pixadb;
 
          /* Continue down until the condition fails */
     w1 = xlast - xfirst + 1;
-    h1 = h - yfirst;  /* initialize */
+    h1 = h - yfirst;  /* init */
+    ylast = h - 1;  /* init */
     for (i = yfirst + 1; i < h; i++) {
         pixFindMaxHorizontalRunOnLine(pix2, i, &xstart, &length);
         if (xstart > xfirst || (xstart + length - 1 < xlast) ||
@@ -2399,7 +2400,6 @@ l_float32  fgfract;
 BOX       *box1;
 BOXA      *boxa1;
 PIX       *pix1, *pix2, *pix3, *pix4, *pix5;
-PIXA      *pixa1;
 
     PROCNAME("pixAutoPhotoinvert");
 
@@ -2412,55 +2412,62 @@ PIXA      *pixa1;
         return (PIX *)ERROR_PTR("pix1 not made", procName, NULL);
     if (pixadb) pixaAddPix(pixadb, pix1, L_COPY);
 
-        /* Make the halftone mask to identify region for photo-inversion */
+        /* Identify regions for photo-inversion:
+         * (1) Start with the halftone mask.
+         * (2) Eliminate ordinary text and halftones in the mask.
+         * (3) Some regions of inverted text may have been removed in
+         *     steps (1) and (2).  Conditionally fill holes in the mask,
+         *     but do not fill out to the bounding rect. */
     pix2 = pixGenerateHalftoneMask(pix1, NULL, NULL, pixadb);
-    pix3 = pixMorphSequence(pix2, "o15.15 + c25.25", 0);  /* clean it up */
+    pix3 = pixMorphSequence(pix2, "o15.15 + c25.25", 0);  /* remove noise */
+    pix4 = pixFillHolesToBoundingRect(pix3, 1, 0.5, 1.0);
     if (pixadb) {
         pixaAddPix(pixadb, pix2, L_CLONE);
-        pixaAddPix(pixadb, pix3, L_COPY);
+        pixaAddPix(pixadb, pix3, L_CLONE);
+        pixaAddPix(pixadb, pix4, L_COPY);
     }
     pixDestroy(&pix2);
-    pixZero(pix3, &empty);
+    pixDestroy(&pix3);
+    pixZero(pix4, &empty);
     if (empty) {
-        pixDestroy(&pix3);
+        pixDestroy(&pix4);
         return pix1;
     }
 
         /* Examine each component and validate the inversion.
          * Require at least 60% of pixels under each component to be FG. */
-    boxa1 = pixConnCompBB(pix3, 8);
+    boxa1 = pixConnCompBB(pix4, 8);
     n = boxaGetCount(boxa1);
     for (i = 0; i < n; i++) {
         box1 = boxaGetBox(boxa1, i, L_COPY);
         pix5 = pixClipRectangle(pix1, box1, NULL);
         pixForegroundFraction(pix5, &fgfract);
         if (pixadb) lept_stderr("fg fraction: %5.3f\n", fgfract);
-        if (fgfract < 0.6) {  /* erase from the mask */
-            boxGetGeometry(box1, &x, &y, &w, &h);
-            pixRasterop(pix3, x, y, w, h, PIX_CLR, NULL, 0, 0);
-        }
+        boxGetGeometry(box1, &x, &y, &w, &h);
+        if (fgfract < 0.6)  /* erase from the mask */
+            pixRasterop(pix4, x, y, w, h, PIX_CLR, NULL, 0, 0);
         pixDestroy(&pix5);
         boxDestroy(&box1);
     }
     boxaDestroy(&boxa1);
-    pixZero(pix3, &empty);
+    pixZero(pix4, &empty);
     if (empty) {
-        pixDestroy(&pix3);
+        pixDestroy(&pix4);
         return pix1;
     }
 
         /* Combine pixels of the photo-inverted pix with the binarized input */
-    pix4 = pixInvert(NULL, pix1);
-    pixCombineMasked(pix1, pix4, pix3);
+    pix5 = pixInvert(NULL, pix1);
+    pixCombineMasked(pix1, pix5, pix4);
 
     if (pixadb) {
-        pixaAddPix(pixadb, pix4, L_CLONE);
+        pixaAddPix(pixadb, pix5, L_CLONE);
         pixaAddPix(pixadb, pix1, L_COPY);
     }
-    pixDestroy(&pix4);
+    pixDestroy(&pix5);
     if (ppixm)
-        *ppixm = pix3;
+        *ppixm = pix4;
     else
-        pixDestroy(&pix3);
+        pixDestroy(&pix4);
     return pix1;
 }
