@@ -84,7 +84,7 @@
  *
  *      Pixaa addition
  *           l_int32   pixaaAddPixa()
- *           l_int32   pixaaExtendArray()
+ *           static l_int32   pixaaExtendArray()
  *           l_int32   pixaaAddPix()
  *           l_int32   pixaaAddBox()
  *
@@ -139,12 +139,15 @@
 #include <string.h>
 #include "allheaders.h"
 
-    /* Bounds on initial array size */
-static const l_uint32  MaxPtrArraySize = 100000;
-static const l_int32 InitialPtrArraySize = 20;      /*!< n'importe quoi */
+    /* Bounds on array sizes */
+static const size_t  MaxInitPtrArraySize = 100000;
+static const size_t  MaxPixaPtrArraySize = 5000000;
+static const size_t  MaxPixaaPtrArraySize = 1000000;
+static const size_t  InitialPtrArraySize = 20;      /*!< n'importe quoi */
 
     /* Static functions */
 static l_int32 pixaExtendArray(PIXA  *pixa);
+static l_int32 pixaaExtendArray(PIXAA *paa);
 
 /*---------------------------------------------------------------------*
  *                    Pixa creation, destruction, copy                 *
@@ -167,7 +170,7 @@ PIXA  *pixa;
 
     PROCNAME("pixaCreate");
 
-    if (n <= 0 || n > MaxPtrArraySize)
+    if (n <= 0 || n > MaxInitPtrArraySize)
         n = InitialPtrArraySize;
 
     pixa = (PIXA *)LEPT_CALLOC(1, sizeof(PIXA));
@@ -432,7 +435,6 @@ PIXA    *pixa;
     }
 
     *ppixa = NULL;
-    return;
 }
 
 
@@ -527,11 +529,16 @@ PIX     *pixc;
         return ERROR_INT("pixc not made", procName, 1);
 
     n = pixaGetCount(pixa);
-    if (n >= pixa->nalloc)
-        pixaExtendArray(pixa);
+    if (n >= pixa->nalloc) {
+        if (pixaExtendArray(pixa)) {
+            if (copyflag != L_INSERT)
+                pixDestroy(&pixc);
+            return ERROR_INT("extension failed", procName, 1);
+        }
+    }
+
     pixa->pix[n] = pixc;
     pixa->n++;
-
     return 0;
 }
 
@@ -572,6 +579,7 @@ pixaAddBox(PIXA    *pixa,
  * <pre>
  * Notes:
  *      (1) Doubles the size of the pixa and boxa ptr arrays.
+ *      (2) The max number of pix in the array is 5 million.
  * </pre>
  */
 static l_int32
@@ -590,31 +598,41 @@ pixaExtendArray(PIXA  *pixa)
  * \brief   pixaExtendArrayToSize()
  *
  * \param[in]    pixa
- * \param[in]    size
+ * \param[in]    size     number of pix ptrs in new array
  * \return  0 if OK; 1 on error
  *
  * <pre>
  * Notes:
  *      (1) If necessary, reallocs new pixa and boxa ptrs arrays to %size.
  *          The pixa and boxa ptr arrays must always be equal in size.
+ *      (2) The max number of pix ptrs is 5M.
  * </pre>
  */
 l_ok
-pixaExtendArrayToSize(PIXA    *pixa,
-                      l_int32  size)
+pixaExtendArrayToSize(PIXA   *pixa,
+                      size_t  size)
 {
+size_t  oldsize, newsize;
+
     PROCNAME("pixaExtendArrayToSize");
 
     if (!pixa)
         return ERROR_INT("pixa not defined", procName, 1);
-
-    if (size > pixa->nalloc) {
-        if ((pixa->pix = (PIX **)reallocNew((void **)&pixa->pix,
-                                 sizeof(PIX *) * pixa->nalloc,
-                                 size * sizeof(PIX *))) == NULL)
-            return ERROR_INT("new ptr array not returned", procName, 1);
-        pixa->nalloc = size;
+    if (pixa->nalloc > MaxPixaPtrArraySize)  /* belt & suspenders */
+        return ERROR_INT("pixa has too many ptrs", procName, 1);
+    if (size > MaxPixaPtrArraySize)
+        return ERROR_INT("size > 5M ptrs; too large", procName, 1);
+    if (size <= pixa->nalloc) {
+        L_INFO("size too small; no extension\n", procName);
+        return 0;
     }
+
+    oldsize = pixa->nalloc * sizeof(PIX *);
+    newsize = size * sizeof(PIX *);
+    if ((pixa->pix = (PIX **)reallocNew((void **)&pixa->pix,
+                                         oldsize, newsize)) == NULL)
+        return ERROR_INT("new ptr array not returned", procName, 1);
+    pixa->nalloc = size;
     return boxaExtendArrayToSize(pixa->boxa, size);
 }
 
@@ -1126,9 +1144,11 @@ PIX     *pix;
  * <pre>
  * Notes:
  *      (1) To clear all the text fields, use %sa == NULL and %text == NULL.
- *      (2) To set all the text fields to the same value %text, use %sa = NULL.
- *      (3) If %sa is defined, we ignore %text and use it; %sa must have
- *          the same count as %pixa.
+ *      (2) Otherwise, this replaces all text fields with a copy of a string,
+ *          either the same string or a string from %sa.
+ *      (3) To set all the text fields to the same value %text, use %sa = NULL.
+ *      (4) If %sa is defined, ignore %text and use the strings in %sa.
+ *          %sa must have the same count as %pixa.
  * </pre>
  */
 l_ok
@@ -1359,14 +1379,18 @@ l_int32  i, n;
     if (!pixa)
         return ERROR_INT("pixa not defined", procName, 1);
     n = pixaGetCount(pixa);
-    if (index < 0 || index > n)
-        return ERROR_INT("index not in {0...n}", procName, 1);
+    if (index < 0 || index > n) {
+        L_ERROR("index %d not in [0,...,%d]\n", procName, index, n);
+        return 1;
+    }
     if (!pixs)
         return ERROR_INT("pixs not defined", procName, 1);
 
     if (n >= pixa->nalloc) {  /* extend both ptr arrays */
-        pixaExtendArray(pixa);
-        boxaExtendArray(pixa->boxa);
+        if (pixaExtendArray(pixa))
+            return ERROR_INT("extension failed", procName, 1);
+        if (boxaExtendArray(pixa->boxa))
+            return ERROR_INT("extension failed", procName, 1);
     }
     pixa->n++;
     for (i = n; i > index; i--)
@@ -1376,7 +1400,6 @@ l_int32  i, n;
         /* Optionally, insert the box */
     if (box)
         boxaInsertBox(pixa->boxa, index, box);
-
     return 0;
 }
 
@@ -1409,8 +1432,10 @@ PIX    **array;
     if (!pixa)
         return ERROR_INT("pixa not defined", procName, 1);
     n = pixaGetCount(pixa);
-    if (index < 0 || index >= n)
-        return ERROR_INT("index not in {0...n - 1}", procName, 1);
+    if (index < 0 || index >= n) {
+        L_ERROR("index %d not in [0,...,%d]\n", procName, index, n - 1);
+        return 1;
+    }
 
         /* Remove the pix */
     array = pixa->pix;
@@ -1465,8 +1490,10 @@ PIX    **array;
     if (!pixa)
         return ERROR_INT("pixa not defined", procName, 1);
     n = pixaGetCount(pixa);
-    if (index < 0 || index >= n)
-        return ERROR_INT("index not in {0...n - 1}", procName, 1);
+    if (index < 0 || index >= n) {
+        L_ERROR("index %d not in [0,...,%d]\n", procName, index, n - 1);
+        return 1;
+    }
 
         /* Remove the pix */
     array = pixa->pix;
@@ -1828,7 +1855,7 @@ PIXAA  *paa;
 
     PROCNAME("pixaaCreate");
 
-    if (n <= 0 || n > MaxPtrArraySize)
+    if (n <= 0 || n > MaxInitPtrArraySize)
         n = InitialPtrArraySize;
 
     paa = (PIXAA *)LEPT_CALLOC(1, sizeof(PIXAA));
@@ -1946,11 +1973,8 @@ PIXAA   *paa;
         pixaDestroy(&paa->pixa[i]);
     LEPT_FREE(paa->pixa);
     boxaDestroy(&paa->boxa);
-
     LEPT_FREE(paa);
     *ppaa = NULL;
-
-    return;
 }
 
 
@@ -1996,11 +2020,15 @@ PIXA    *pixac;
     }
 
     n = pixaaGetCount(paa, NULL);
-    if (n >= paa->nalloc)
-        pixaaExtendArray(paa);
+    if (n >= paa->nalloc) {
+        if (pixaaExtendArray(paa)) {
+            if (copyflag != L_INSERT)
+                pixaDestroy(&pixac);
+            return ERROR_INT("extension failed", procName, 1);
+        }
+    }
     paa->pixa[n] = pixac;
     paa->n++;
-
     return 0;
 }
 
@@ -2010,21 +2038,33 @@ PIXA    *pixac;
  *
  * \param[in]    paa
  * \return  0 if OK; 1 on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) The max number of pixa ptrs is 1M.
+ * </pre>
  */
-l_ok
+static l_int32
 pixaaExtendArray(PIXAA  *paa)
 {
+size_t  oldsize, newsize;
+
     PROCNAME("pixaaExtendArray");
 
     if (!paa)
         return ERROR_INT("paa not defined", procName, 1);
+    if (paa->nalloc > MaxPixaaPtrArraySize)  /* belt & suspenders */
+        return ERROR_INT("paa has too many ptrs", procName, 1);
+    oldsize = paa->nalloc * sizeof(PIXA *);
+    newsize = 2 * oldsize;
+    if (newsize > 8 * MaxPixaaPtrArraySize)
+        return ERROR_INT("newsize > 8 MB; too large", procName, 1);
 
     if ((paa->pixa = (PIXA **)reallocNew((void **)&paa->pixa,
-                             sizeof(PIXA *) * paa->nalloc,
-                             2 * sizeof(PIXA *) * paa->nalloc)) == NULL)
+                                         oldsize, newsize)) == NULL)
         return ERROR_INT("new ptr array not returned", procName, 1);
 
-    paa->nalloc = 2 * paa->nalloc;
+    paa->nalloc *= 2;
     return 0;
 }
 
@@ -2601,6 +2641,7 @@ PIXA  *pixa;
  * Notes:
  *      (1) The pix are stored in the file as png.
  *          If the png library is not linked, this will fail.
+ *      (2) It is OK for the pixa to be empty.
  * </pre>
  */
 PIXA *
@@ -2627,6 +2668,11 @@ PIXA    *pixa;
         return (PIXA *)ERROR_PTR("invalid pixa version", procName, NULL);
     if (fscanf(fp, "Number of pix = %d\n", &n) != 1)
         return (PIXA *)ERROR_PTR("not a pixa file", procName, NULL);
+    if (n < 0)
+        return (PIXA *)ERROR_PTR("num pix ptrs < 0", procName, NULL);
+    if (n > MaxPixaPtrArraySize)
+        return (PIXA *)ERROR_PTR("too many pix ptrs", procName, NULL);
+    if (n == 0) L_INFO("the pixa is empty\n", procName);
 
     if ((boxa = boxaReadStream(fp)) == NULL)
         return (PIXA *)ERROR_PTR("boxa not made", procName, NULL);
@@ -2838,6 +2884,9 @@ FILE    *fp;
     if ((fp = open_memstream((char **)pdata, psize)) == NULL)
         return ERROR_INT("stream not opened", procName, 1);
     ret = pixaWriteStream(fp, pixa);
+    fputc('\0', fp);
+    fclose(fp);
+    *psize = *psize - 1;
 #else
     L_INFO("work-around: writing to a temp file\n", procName);
   #ifdef _WIN32
@@ -2850,8 +2899,8 @@ FILE    *fp;
     ret = pixaWriteStream(fp, pixa);
     rewind(fp);
     *pdata = l_binaryReadStream(fp, psize);
-#endif  /* HAVE_FMEMOPEN */
     fclose(fp);
+#endif  /* HAVE_FMEMOPEN */
     return ret;
 }
 
@@ -3010,6 +3059,7 @@ PIXAA  *paa;
  * Notes:
  *      (1) The pix are stored in the file as png.
  *          If the png library is not linked, this will fail.
+ *      (2) It is OK for the pixaa to be empty.
  * </pre>
  */
 PIXAA *
@@ -3036,6 +3086,11 @@ PIXAA   *paa;
         return (PIXAA *)ERROR_PTR("invalid pixaa version", procName, NULL);
     if (fscanf(fp, "Number of pixa = %d\n", &n) != 1)
         return (PIXAA *)ERROR_PTR("not a pixaa file", procName, NULL);
+    if (n < 0)
+        return (PIXAA *)ERROR_PTR("num pixa ptrs < 0", procName, NULL);
+    if (n > MaxPixaaPtrArraySize)
+        return (PIXAA *)ERROR_PTR("too many pixa ptrs", procName, NULL);
+    if (n == 0) L_INFO("the pixaa is empty\n", procName);
 
     if ((paa = pixaaCreate(n)) == NULL)
         return (PIXAA *)ERROR_PTR("paa not made", procName, NULL);
@@ -3214,6 +3269,9 @@ FILE    *fp;
     if ((fp = open_memstream((char **)pdata, psize)) == NULL)
         return ERROR_INT("stream not opened", procName, 1);
     ret = pixaaWriteStream(fp, paa);
+    fputc('\0', fp);
+    fclose(fp);
+    *psize = *psize - 1;
 #else
     L_INFO("work-around: writing to a temp file\n", procName);
   #ifdef _WIN32
@@ -3226,8 +3284,8 @@ FILE    *fp;
     ret = pixaaWriteStream(fp, paa);
     rewind(fp);
     *pdata = l_binaryReadStream(fp, psize);
-#endif  /* HAVE_FMEMOPEN */
     fclose(fp);
+#endif  /* HAVE_FMEMOPEN */
     return ret;
 }
 
