@@ -39,6 +39,8 @@
  *           PIXA     *pixaSelectByPerimSizeRatio()
  *           PIX      *pixSelectByAreaFraction()
  *           PIXA     *pixaSelectByAreaFraction()
+ *           PIX      *pixSelectByArea()
+ *           PIXA     *pixaSelectByArea()
  *           PIX      *pixSelectByWidthHeightRatio()
  *           PIXA     *pixaSelectByWidthHeightRatio()
  *           PIXA     *pixaSelectByNumConnComp()
@@ -84,6 +86,7 @@
  *           l_int32   pixaAnyColormaps()
  *           l_int32   pixaGetDepthInfo()
  *           PIXA     *pixaConvertToSameDepth()
+ *           PIXA     *pixaConvertToGivenDepth()
  *           l_int32   pixaEqual()
  *           l_int32   pixaSetFullSizeBoxa()
  * </pre>
@@ -133,6 +136,7 @@ static const l_float32  MinAngleToRotate = 0.001;  /* radians; ~0.06 deg */
  *        Size
  *        PerimToAreaRatio
  *        PerimSizeRatio
+ *        Area
  *        AreaFraction
  *        WidthHeightRatio
  *
@@ -816,6 +820,140 @@ PIXA  *pixad;
 }
 
 
+/*!
+ * \brief   pixSelectByArea()
+ *
+ * \param[in]    pixs          1 bpp
+ * \param[in]    thresh        threshold number of FG pixels
+ * \param[in]    connectivity  4 or 8
+ * \param[in]    type          L_SELECT_IF_LT, L_SELECT_IF_GT,
+ *                             L_SELECT_IF_LTE, L_SELECT_IF_GTE
+ * \param[out]   pchanged      [optional] 1 if changed; 0 if clone returned
+ * \return  pixd, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) The args specify constraints on the number of foreground
+ *          pixels in the components that are kept.
+ *      (2) If unchanged, returns a copy of pixs.  Otherwise,
+ *          returns a new pix with the filtered components.
+ *      (3) This filters components based on the number of fg pixels
+ *          in each component.
+ *      (4) Use L_SELECT_IF_LT or L_SELECT_IF_LTE to save components
+ *          with less than the threshold number of fg pixels, and
+ *          L_SELECT_IF_GT or L_SELECT_IF_GTE to remove them.
+ * </pre>
+ */
+PIX *
+pixSelectByArea(PIX       *pixs,
+                l_float32  thresh,
+                l_int32    connectivity,
+                l_int32    type,
+                l_int32   *pchanged)
+{
+l_int32  w, h, empty, changed, count;
+BOXA    *boxa;
+PIX     *pixd;
+PIXA    *pixas, *pixad;
+
+    PROCNAME("pixSelectByArea");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    if (connectivity != 4 && connectivity != 8)
+        return (PIX *)ERROR_PTR("connectivity not 4 or 8", procName, NULL);
+    if (type != L_SELECT_IF_LT && type != L_SELECT_IF_GT &&
+        type != L_SELECT_IF_LTE && type != L_SELECT_IF_GTE)
+        return (PIX *)ERROR_PTR("invalid type", procName, NULL);
+    if (pchanged) *pchanged = FALSE;
+
+        /* Check if any components exist */
+    pixZero(pixs, &empty);
+    if (empty)
+        return pixCopy(NULL, pixs);
+
+        /* Filter components */
+    boxa = pixConnComp(pixs, &pixas, connectivity);
+    pixad = pixaSelectByArea(pixas, thresh, type, &changed);
+    boxaDestroy(&boxa);
+    pixaDestroy(&pixas);
+
+    if (!changed) {
+        pixaDestroy(&pixad);
+        return pixCopy(NULL, pixs);
+    }
+
+        /* Render the result */
+    if (pchanged) *pchanged = TRUE;
+    pixGetDimensions(pixs, &w, &h, NULL);
+    count = pixaGetCount(pixad);
+    if (count == 0) {  /* return empty pix */
+        pixd = pixCreateTemplate(pixs);
+    } else {
+        pixd = pixaDisplay(pixad, w, h);
+        pixCopyResolution(pixd, pixs);
+        pixCopyColormap(pixd, pixs);
+        pixCopyText(pixd, pixs);
+        pixCopyInputFormat(pixd, pixs);
+    }
+    pixaDestroy(&pixad);
+    return pixd;
+}
+
+
+/*!
+ * \brief   pixaSelectByArea()
+ *
+ * \param[in]    pixas
+ * \param[in]    thresh      threshold number of fg pixels
+ * \param[in]    type        L_SELECT_IF_LT, L_SELECT_IF_GT,
+ *                           L_SELECT_IF_LTE, L_SELECT_IF_GTE
+ * \param[out]   pchanged    [optional] 1 if changed; 0 if clone returned
+ * \return  pixad, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) Returns a pixa clone if no components are removed.
+ *      (2) Uses pix and box clones in the new pixa.
+ *      (3) This filters components based on the number of fg pixels
+ *          in the component.
+ *      (4) Use L_SELECT_IF_LT or L_SELECT_IF_LTE to save components
+ *          with less than the threshold number of fg pixels, and
+ *          L_SELECT_IF_GT or L_SELECT_IF_GTE to remove them.
+ * </pre>
+ */
+PIXA *
+pixaSelectByArea(PIXA      *pixas,
+                 l_float32  thresh,
+                 l_int32    type,
+                 l_int32   *pchanged)
+{
+NUMA  *na, *nai;
+PIXA  *pixad;
+
+    PROCNAME("pixaSelectByArea");
+
+    if (!pixas)
+        return (PIXA *)ERROR_PTR("pixas not defined", procName, NULL);
+    if (type != L_SELECT_IF_LT && type != L_SELECT_IF_GT &&
+        type != L_SELECT_IF_LTE && type != L_SELECT_IF_GTE)
+        return (PIXA *)ERROR_PTR("invalid type", procName, NULL);
+
+        /* Compute area of each component */
+    na = pixaCountPixels(pixas);
+
+        /* Generate indicator array for elements to be saved. */
+    nai = numaMakeThresholdIndicator(na, thresh, type);
+    numaDestroy(&na);
+
+        /* Filter to get output */
+    pixad = pixaSelectWithIndicator(pixas, nai, pchanged);
+
+    numaDestroy(&nai);
+    return pixad;
+}
+
+
 /*!
  * \brief   pixSelectByWidthHeightRatio()
  *
@@ -2613,6 +2751,17 @@ PIX     *pix1, *pix2;
  * \param[in]    pixa
  * \param[out]   pdepth   depth required to render if all colormaps are removed
  * \return  0 if OK; 1 on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) Any pix with 16 bpp will be considered as having 8 bpp.
+ *          If all pix have bpp = 1, this returns 1.
+ *          If any pix has color (either rgb or a colormap with color),
+ *          this return 32.
+ *          Otherwise, this returns the maximum of 8 and the max depth
+ *          of all the pix.
+ *      (2) This can be used to allow lossless rendering onto a single pix.
+ * </pre>
  */
 l_ok
 pixaGetRenderingDepth(PIXA     *pixa,
@@ -2774,16 +2923,20 @@ l_int32  maxd, same;  /* depth info */
  *
  * <pre>
  * Notes:
- *      (1) If any pix has a colormap, they are all converted to rgb.
- *          Otherwise, they are all converted to the maximum depth of
- *          all the pix.
- *      (2) This can be used to allow lossless rendering onto a single pix.
+ *      (1) Any pix with 16 bpp will be converted to 8 bpp.
+ *          If all pix have bpp = 1, the output depth will be 1.
+ *          If any pix has color (either rgb or a colormap with color),
+ *          the output depth will be 32.
+ *          Otherwise, the output depth is the maximum of 8 and the
+ *          the max depth of all the pix.
+ *      (2) This can be used to allow lossless rendering onto
+ *          a single pix. (Except: 16 bpp gets converted to 8.)
  * </pre>
  */
 PIXA *
 pixaConvertToSameDepth(PIXA  *pixas)
 {
-l_int32  i, n, same, hascmap, maxdepth;
+l_int32  i, n, depth, same, hascmap, maxdepth;
 BOXA    *boxa;
 PIX     *pix1, *pix2;
 PIXA    *pixa1, *pixad;
@@ -2792,16 +2945,21 @@ PIXA    *pixa1, *pixad;
 
     if (!pixas)
         return (PIXA *)ERROR_PTR("pixas not defined", procName, NULL);
-
-        /* Remove colormaps to rgb */
     if ((n = pixaGetCount(pixas)) == 0)
         return (PIXA *)ERROR_PTR("no components", procName, NULL);
+
+
+        /* Remove colormaps if necessary */
+    pixaGetRenderingDepth(pixas, &depth);
     pixaAnyColormaps(pixas, &hascmap);
     if (hascmap) {
         pixa1 = pixaCreate(n);
         for (i = 0; i < n; i++) {
             pix1 = pixaGetPix(pixas, i, L_CLONE);
-            pix2 = pixConvertTo32(pix1);
+            if (depth == 32)
+                pix2 = pixConvertTo32(pix1);
+            else  /* depth = 8 */
+                pix2 = pixConvertTo8(pix1, 0);
             pixaAddPix(pixa1, pix2, L_INSERT);
             pixDestroy(&pix1);
         }
@@ -2814,7 +2972,7 @@ PIXA    *pixa1, *pixad;
         pixad = pixaCreate(n);
         for (i = 0; i < n; i++) {
             pix1 = pixaGetPix(pixa1, i, L_CLONE);
-            if (maxdepth <= 8)
+            if (maxdepth <= 16)
                 pix2 = pixConvertTo8(pix1, 0);
             else
                 pix2 = pixConvertTo32(pix1);
@@ -2832,6 +2990,66 @@ PIXA    *pixa1, *pixad;
 }
 
 
+/*!
+ * \brief   pixaConvertToGivenDepth()
+ *
+ * \param[in]    pixas
+ * \param[in]    depth    specify either 8 or 32 bpp
+ * \return  pixad, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) Use this to remove any colormaps and convert all pix to either
+ *          8 or 32 bpp.
+ *      (2) To convert losslessly, get %depth from pixaGetRenderingDepth().
+ *      (3) Clone pix may be in the returned pixa if conversion is to 32 bpp.
+ * </pre>
+ */
+PIXA *
+pixaConvertToGivenDepth(PIXA    *pixas,
+                        l_int32  depth)
+{
+l_int32  i, n, maxd;
+BOXA    *boxa;
+PIX     *pix1, *pix2;
+PIXA    *pixad;
+
+    PROCNAME("pixaConvertToGivenDepth");
+
+    if (!pixas)
+        return (PIXA *)ERROR_PTR("pixas not defined", procName, NULL);
+    if ((n = pixaGetCount(pixas)) == 0)
+        return (PIXA *)ERROR_PTR("no components", procName, NULL);
+    if (depth != 8 && depth != 32)
+        return (PIXA *)ERROR_PTR("depth not 8 or 32", procName, NULL);
+
+        /* Warn with 1 --> {8,32} or lossy conversions */
+    pixaGetRenderingDepth(pixas, &maxd);
+    if (maxd == 1)
+        L_WARNING("All pix are 1 bpp; converting to %d bpp\n", procName, depth);
+    if (maxd > depth)
+        L_WARNING("Lossy conversion: max rendering depth %d > input %d\n",
+                  procName, maxd, depth);
+
+    pixad = pixaCreate(n);
+    for (i = 0; i < n; i++) {
+        pix1 = pixaGetPix(pixas, i, L_CLONE);
+        if (depth == 32) {
+            pix2 = (pixGetDepth(pix1) == 32) ? pixClone(pix1) :
+                   pixConvertTo32(pix1);
+        } else {  /* depth = 8 */
+            pix2 = pixConvertTo8(pix1, 0);
+        }
+        pixaAddPix(pixad, pix2, L_INSERT);
+        pixDestroy(&pix1);
+    }
+
+    boxa = pixaGetBoxa(pixas, L_COPY);
+    pixaSetBoxa(pixad, boxa, L_INSERT);
+    return pixad;
+}
+
+
 /*!
  * \brief   pixaEqual()
  *
