@@ -1,20 +1,32 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
-/*
- *  pixalloc.c
+/*!
+ * \file pixalloc.c
+ * <pre>
  *
  *      Custom memory storage with allocator and deallocator
  *
@@ -26,12 +38,14 @@
  *          l_int32       pmsGetLevelForAlloc()
  *          l_int32       pmsGetLevelForDealloc()
  *          void          pmsLogInfo()
+ * </pre>
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include "allheaders.h"
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
 
+#include "allheaders.h"
 
 /*-------------------------------------------------------------------------*
  *                          Pix Memory Storage                             *
@@ -79,7 +93,7 @@
  *  Any pix requiring less than 0.5 MB or more than 8 MB of memory will
  *  not come from the memory store.  Instead, it will be dynamically
  *  allocated and freed after use.
- *             
+ *
  *  How is this implemented?
  *
  *  At setup, the full data block size is computed and allocated.
@@ -99,27 +113,27 @@
  *  will free them.
  */
 
-
+/*! Pix memory storage */
 struct PixMemoryStore
 {
-    struct L_Ptraa  *paa;          /* Holds ptrs to allocated memory        */
-    size_t           minsize;      /* Pix smaller than this (in bytes)      */
-                                   /* are allocated dynamically             */
-    size_t           smallest;     /* Smallest mem (in bytes) alloc'd       */
-    size_t           largest;      /* Larest mem (in bytes) alloc'd         */
-    size_t           nbytes;       /* Size of allocated block w/ all chunks */
-    l_int32          nlevels;      /* Num of power-of-2 sizes pre-alloc'd   */
-    size_t          *sizes;        /* Mem sizes at each power-of-2 level    */
-    l_int32         *allocarray;   /* Number of mem alloc'd at each size    */
-    l_uint32        *baseptr;      /* ptr to allocated array                */
-    l_uint32        *maxptr;       /* ptr just beyond allocated memory      */
-    l_uint32       **firstptr;     /* array of ptrs to first chunk in size  */
-    l_int32         *memused;      /* log: total # of pix used (by level)   */
-    l_int32         *meminuse;     /* log: # of pix in use (by level)       */
-    l_int32         *memmax;       /* log: max # of pix in use (by level)   */
-    l_int32         *memempty;     /* log: # of pix alloc'd because         */
-                                   /*      the store was empty (by level)   */
-    char            *logfile;      /* log: set to null if no logging        */
+    struct L_Ptraa  *paa;        /*!< Holds ptrs to allocated memory        */
+    size_t           minsize;    /*!< Pix smaller than this (in bytes)      */
+                                 /*!< are allocated dynamically             */
+    size_t           smallest;   /*!< Smallest mem (in bytes) alloc'd       */
+    size_t           largest;    /*!< Larest mem (in bytes) alloc'd         */
+    size_t           nbytes;     /*!< Size of allocated block w/ all chunks */
+    l_int32          nlevels;    /*!< Num of power-of-2 sizes pre-alloc'd   */
+    size_t          *sizes;      /*!< Mem sizes at each power-of-2 level    */
+    l_int32         *allocarray; /*!< Number of mem alloc'd at each size    */
+    l_uint32        *baseptr;    /*!< ptr to allocated array                */
+    l_uint32        *maxptr;     /*!< ptr just beyond allocated memory      */
+    l_uint32       **firstptr;   /*!< array of ptrs to first chunk in size  */
+    l_int32         *memused;    /*!< log: total # of pix used (by level)   */
+    l_int32         *meminuse;   /*!< log: # of pix in use (by level)       */
+    l_int32         *memmax;     /*!< log: max # of pix in use (by level)   */
+    l_int32         *memempty;   /*!< log: # of pix alloc'd because         */
+                                 /*!<      the store was empty (by level)   */
+    char            *logfile;    /*!< log: set to null if no logging        */
 };
 typedef struct PixMemoryStore   L_PIX_MEM_STORE;
 
@@ -127,21 +141,22 @@ static L_PIX_MEM_STORE  *CustomPMS = NULL;
 
 
 /*!
- *  pmsCreate()
+ * \brief   pmsCreate()
  *
- *      Input:  minsize (of data chunk that can be supplied by pms)
- *              smallest (bytes of the smallest pre-allocated data chunk.
- *              numalloc (array with the number of data chunks for each
- *                        size that are in the memory store)
- *              logfile (use for debugging; null otherwise)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    minsize    of data chunk that can be supplied by pms
+ * \param[in]    smallest   bytes of the smallest pre-allocated data chunk.
+ * \param[in]    numalloc   array with the number of data chunks for each
+ *                          size that are in the memory store
+ * \param[in]    logfile    use for debugging; null otherwise
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This computes the size of the block of memory required
  *          and allocates it.  Each chunk starts on a 32-bit word boundary.
- *          The chunk sizes are in powers of 2, starting at @smallest,
+ *          The chunk sizes are in powers of 2, starting at %smallest,
  *          and the number of levels and chunks at each level is
- *          specified by @numalloc.
+ *          specified by %numalloc.
  *      (2) This is intended to manage the image data for a small number
  *          of relatively large pix.  The system malloc is expected to
  *          handle very large numbers of small chunks efficiently.
@@ -151,8 +166,9 @@ static L_PIX_MEM_STORE  *CustomPMS = NULL;
  *      (4) The pms struct is stored in a static global, so this function
  *          is not thread-safe.  When used, there must be only one thread
  *          per process.
+ * </pre>
  */
-l_int32
+l_ok
 pmsCreate(size_t       minsize,
           size_t       smallest,
           NUMA        *numalloc,
@@ -174,10 +190,9 @@ L_PTRAA          *paa;
         return ERROR_INT("numalloc not defined", procName, 1);
     numaGetSum(numalloc, &nchunks);
     if (nchunks > 1000.0)
-        L_WARNING_FLOAT("There are %.0f chunks", procName, nchunks);
+        L_WARNING("There are %.0f chunks\n", procName, nchunks);
 
-    if ((pms = (L_PIX_MEM_STORE *)CALLOC(1, sizeof(L_PIX_MEM_STORE))) == NULL)
-        return ERROR_INT("pms not made", procName, 1);
+    pms = (L_PIX_MEM_STORE *)LEPT_CALLOC(1, sizeof(L_PIX_MEM_STORE));
     CustomPMS = pms;
 
         /* Make sure that minsize and smallest are multiples of 32 bit words */
@@ -187,7 +202,7 @@ L_PTRAA          *paa;
     nlevels = numaGetCount(numalloc);
     pms->nlevels = nlevels;
 
-    if ((sizes = (size_t *)CALLOC(nlevels, sizeof(size_t))) == NULL)
+    if ((sizes = (size_t *)LEPT_CALLOC(nlevels, sizeof(size_t))) == NULL)
         return ERROR_INT("sizes not made", procName, 1);
     pms->sizes = sizes;
     if (smallest % 4 != 0)
@@ -207,14 +222,16 @@ L_PTRAA          *paa;
         nbytes += alloca[i] * sizes[i];
     pms->nbytes = nbytes;
 
-    if ((baseptr = (l_uint32 *)CALLOC(nbytes / 4, sizeof(l_uint32))) == NULL)
+    if ((baseptr = (l_uint32 *)LEPT_CALLOC(nbytes / 4, sizeof(l_uint32)))
+        == NULL)
         return ERROR_INT("calloc fail for baseptr", procName, 1);
     pms->baseptr = baseptr;
     pms->maxptr = baseptr + nbytes / 4;  /* just beyond the memory store */
-    if ((firstptr = (l_uint32 **)CALLOC(nlevels, sizeof(l_uint32 *))) == NULL)
+    if ((firstptr = (l_uint32 **)LEPT_CALLOC(nlevels, sizeof(l_uint32 *)))
+        == NULL)
         return ERROR_INT("calloc fail for firstptr", procName, 1);
     pms->firstptr = firstptr;
-    
+
     data = baseptr;
     for (i = 0; i < nlevels; i++) {
         if ((pa = ptraCreate(alloca[i])) == NULL)
@@ -226,12 +243,12 @@ L_PTRAA          *paa;
             data += sizes[i] / 4;
         }
     }
-            
+
     if (logfile) {
-        pms->memused = (l_int32 *)CALLOC(nlevels, sizeof(l_int32));
-        pms->meminuse = (l_int32 *)CALLOC(nlevels, sizeof(l_int32));
-        pms->memmax = (l_int32 *)CALLOC(nlevels, sizeof(l_int32));
-        pms->memempty = (l_int32 *)CALLOC(nlevels, sizeof(l_int32));
+        pms->memused = (l_int32 *)LEPT_CALLOC(nlevels, sizeof(l_int32));
+        pms->meminuse = (l_int32 *)LEPT_CALLOC(nlevels, sizeof(l_int32));
+        pms->memmax = (l_int32 *)LEPT_CALLOC(nlevels, sizeof(l_int32));
+        pms->memempty = (l_int32 *)LEPT_CALLOC(nlevels, sizeof(l_int32));
         pms->logfile = stringNew(logfile);
     }
 
@@ -240,17 +257,16 @@ L_PTRAA          *paa;
 
 
 /*!
- *  pmsDestroy()
+ * \brief   pmsDestroy()
  *
- *      Input:  (none)
- *      Return: void
- *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Important: call this function at the end of the program, after
  *          the last pix has been destroyed.
+ * </pre>
  */
 void
-pmsDestroy()
+pmsDestroy(void)
 {
 L_PIX_MEM_STORE  *pms;
 
@@ -258,38 +274,39 @@ L_PIX_MEM_STORE  *pms;
         return;
 
     ptraaDestroy(&pms->paa, FALSE, FALSE);  /* don't touch the ptrs */
-    FREE(pms->baseptr);  /* free the memory */
+    LEPT_FREE(pms->baseptr);  /* free the memory */
 
     if (pms->logfile) {
         pmsLogInfo();
-        FREE(pms->logfile);
-        FREE(pms->memused);
-        FREE(pms->meminuse);
-        FREE(pms->memmax);
-        FREE(pms->memempty);
+        LEPT_FREE(pms->logfile);
+        LEPT_FREE(pms->memused);
+        LEPT_FREE(pms->meminuse);
+        LEPT_FREE(pms->memmax);
+        LEPT_FREE(pms->memempty);
     }
 
-    FREE(pms->sizes);
-    FREE(pms->allocarray);
-    FREE(pms->firstptr);
-    FREE(pms);
+    LEPT_FREE(pms->sizes);
+    LEPT_FREE(pms->allocarray);
+    LEPT_FREE(pms->firstptr);
+    LEPT_FREE(pms);
     CustomPMS = NULL;
-    return;
 }
 
 
 /*!
- *  pmsCustomAlloc()
+ * \brief   pmsCustomAlloc()
  *
- *      Input: nbytes (min number of bytes in the chunk to be retrieved)
- *      Return: data (ptr to chunk)
+ * \param[in]   nbytes    min number of bytes in the chunk to be retrieved
+ * \return  data ptr to chunk
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This attempts to find a suitable pre-allocated chunk.
  *          If not found, it dynamically allocates the chunk.
  *      (2) If logging is turned on, the allocations that are not taken
  *          from the memory store, and are at least as large as the
  *          minimum size the store can handle, are logged to file.
+ * </pre>
  */
 void *
 pmsCustomAlloc(size_t  nbytes)
@@ -309,20 +326,19 @@ L_PTRA           *pa;
     if (level < 0) {  /* size range invalid; must alloc */
         if ((data = pmsGetAlloc(nbytes)) == NULL)
             return (void *)ERROR_PTR("data not made", procName, NULL);
-    }
-    else {  /* get from store */
+    } else {  /* get from store */
         pa = ptraaGetPtra(pms->paa, level, L_HANDLE_ONLY);
         data = ptraRemoveLast(pa);
         if (data && pms->logfile) {
-            pms->memused[level]++; 
-            pms->meminuse[level]++; 
-            if (pms->meminuse[level] > pms->memmax[level]) 
+            pms->memused[level]++;
+            pms->meminuse[level]++;
+            if (pms->meminuse[level] > pms->memmax[level])
                 pms->memmax[level]++;
         }
         if (!data) {  /* none left at this level */
             data = pmsGetAlloc(nbytes);
             if (pms->logfile)
-                pms->memempty[level]++; 
+                pms->memempty[level]++;
         }
     }
 
@@ -331,52 +347,57 @@ L_PTRA           *pa;
 
 
 /*!
- *  pmsCustomDealloc()
+ * \brief   pmsCustomDealloc()
  *
- *      Input: data (to be freed or returned to the storage)
- *      Return: void
+ * \param[in]   data    to be freed or returned to the storage
+ * \return  void
  */
 void
 pmsCustomDealloc(void  *data)
 {
 l_int32           level;
-FILE             *fp;
 L_PIX_MEM_STORE  *pms;
 L_PTRA           *pa;
 
     PROCNAME("pmsCustomDealloc");
 
-    if ((pms = CustomPMS) == NULL)
-        return ERROR_VOID("pms not defined", procName);
+    if ((pms = CustomPMS) == NULL) {
+        L_ERROR("pms not defined\n", procName);
+        return;
+    }
 
-    if (pmsGetLevelForDealloc(data, &level) == 1)
-        return ERROR_VOID("level not found", procName);
+    if (pmsGetLevelForDealloc(data, &level) == 1) {
+        L_ERROR("level not found\n", procName);
+        return;
+    }
 
-    if (level < 0)  /* no logging; just free the data */
-        FREE(data);
-    else {  /* return the data to the store */
+    if (level < 0) {  /* no logging; just free the data */
+        LEPT_FREE(data);
+    } else {  /* return the data to the store */
         pa = ptraaGetPtra(pms->paa, level, L_HANDLE_ONLY);
         ptraAdd(pa, data);
         if (pms->logfile)
-            pms->meminuse[level]--; 
+            pms->meminuse[level]--;
     }
-
-    return;
 }
 
 
 /*!
- *  pmsGetAlloc()
+ * \brief   pmsGetAlloc()
  *
- *      Input:  nbytes
- *      Return: data
+ * \param[in]    nbytes
+ * \return  data
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This is called when a request for pix data cannot be
  *          obtained from the preallocated memory store.  After use it
  *          is freed like normal memory.
  *      (2) If logging is on, only write out allocs that are as large as
  *          the minimum size handled by the memory store.
+ *      (3) The C99 platform-independent format specifier for size_t is %zu.
+ *          Windows since at least VC-2015 is conforming; we can now use %zu.
+ * </pre>
  */
 void *
 pmsGetAlloc(size_t  nbytes)
@@ -390,26 +411,28 @@ L_PIX_MEM_STORE  *pms;
     if ((pms = CustomPMS) == NULL)
         return (void *)ERROR_PTR("pms not defined", procName, NULL);
 
-    if ((data = (void *)CALLOC(nbytes, sizeof(char))) == NULL)
+    if ((data = (void *)LEPT_CALLOC(nbytes, sizeof(char))) == NULL)
         return (void *)ERROR_PTR("data not made", procName, NULL);
     if (pms->logfile && nbytes >= pms->smallest) {
-        fp = fopen(pms->logfile, "a");
-        fprintf(fp, "Alloc %d bytes at %x\n", nbytes, data);
-        fclose(fp);
+        if ((fp = fopenWriteStream(pms->logfile, "a")) != NULL) {
+            fprintf(fp, "Alloc %zu bytes at %p\n", nbytes, data);
+            fclose(fp);
+        } else {
+            L_ERROR("failed to open stream for %s\n", procName, pms->logfile);
+        }
     }
-
     return data;
 }
 
 
 /*!
- *  pmsGetLevelForAlloc()
+ * \brief   pmsGetLevelForAlloc()
  *
- *      Input: nbytes (min number of bytes in the chunk to be retrieved)
- *             &level (<return>; -1 if either too small or too large)
- *      Return: 0 if OK, 1 on error
+ * \param[in]   nbytes min number of bytes in the chunk to be retrieved
+ * \param[out]  plevel  -1 if either too small or too large
+ * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 pmsGetLevelForAlloc(size_t    nbytes,
                     l_int32  *plevel)
 {
@@ -441,14 +464,14 @@ L_PIX_MEM_STORE  *pms;
 
 
 /*!
- *  pmsGetLevelForDealloc()
+ * \brief   pmsGetLevelForDealloc()
  *
- *      Input: data (ptr to memory chunk)
- *             &level (<return> level in memory store; -1 if allocated
- *                     outside the store)
- *      Return: 0 if OK, 1 on error
+ * \param[in]   data ptr to memory chunk
+ * \param[out]  plevel level in memory store; -1 if allocated
+ *                     outside the store
+ * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 pmsGetLevelForDealloc(void     *data,
                       l_int32  *plevel)
 {
@@ -481,13 +504,10 @@ L_PIX_MEM_STORE  *pms;
 
 
 /*!
- *  pmsLogInfo()
- *
- *      Input:  (none)
- *      Return: void
+ * \brief   pmsLogInfo()
  */
 void
-pmsLogInfo()
+pmsLogInfo(void)
 {
 l_int32           i;
 L_PIX_MEM_STORE  *pms;
@@ -495,22 +515,18 @@ L_PIX_MEM_STORE  *pms;
     if ((pms = CustomPMS) == NULL)
         return;
 
-    fprintf(stderr, "Total number of pix used at each level\n");
+    lept_stderr("Total number of pix used at each level\n");
     for (i = 0; i < pms->nlevels; i++)
-         fprintf(stderr, " Level %d (%d bytes): %d\n", i, pms->sizes[i],
-                 pms->memused[i]);
+         lept_stderr(" Level %d (%zu bytes): %d\n", i,
+                     pms->sizes[i], pms->memused[i]);
 
-    fprintf(stderr, "Max number of pix in use at any time in each level\n");
+    lept_stderr("Max number of pix in use at any time in each level\n");
     for (i = 0; i < pms->nlevels; i++)
-         fprintf(stderr, " Level %d (%d bytes): %d\n", i, pms->sizes[i],
-                 pms->memmax[i]);
+         lept_stderr(" Level %d (%zu bytes): %d\n", i,
+                     pms->sizes[i], pms->memmax[i]);
 
-    fprintf(stderr, "Number of pix alloc'd because none were available\n");
+    lept_stderr("Number of pix alloc'd because none were available\n");
     for (i = 0; i < pms->nlevels; i++)
-         fprintf(stderr, " Level %d (%d bytes): %d\n", i, pms->sizes[i],
-                 pms->memempty[i]);
-
-    return;
+         lept_stderr(" Level %d (%zu bytes): %d\n", i,
+                     pms->sizes[i], pms->memempty[i]);
 }
-
-
