@@ -155,12 +155,12 @@
 #include <string.h>
 #include "allheaders.h"
 
-    /* Bounds on initial array size */
+    /* Bounds on pixacomp array size */
 static const l_uint32  MaxPtrArraySize = 1000000;
 static const l_int32  InitialPtrArraySize = 20;      /*!< n'importe quoi */
 
-    /* Bound on data size */
-static const size_t  MaxDataSize = 1000000000;
+    /* Bound on size for a compressed data string */
+static const size_t  MaxDataSize = 1000000000;   /* 1 GB */
 
     /* These two globals are defined in writefile.c */
 extern l_int32  NumImageFileFormatExtensions;
@@ -377,7 +377,6 @@ PIXC  *pixc;
         LEPT_FREE(pixc->text);
     LEPT_FREE(pixc);
     *ppixc = NULL;
-    return;
 }
 
 
@@ -386,6 +385,11 @@ PIXC  *pixc;
  *
  * \param[in]    pixcs
  * \return  pixcd, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) Limit the size of the compressed pix to 500 MB.
+ * </pre>
  */
 PIXC *
 pixcompCopy(PIXC  *pixcs)
@@ -398,6 +402,9 @@ PIXC     *pixcd;
 
     if (!pixcs)
         return (PIXC *)ERROR_PTR("pixcs not defined", procName, NULL);
+    size = pixcs->size;
+    if (size > MaxDataSize)
+        return (PIXC *)ERROR_PTR("size > 1 GB; too big", procName, NULL);
 
     pixcd = (PIXC *)LEPT_CALLOC(1, sizeof(PIXC));
     pixcd->w = pixcs->w;
@@ -411,7 +418,6 @@ PIXC     *pixcd;
     pixcd->cmapflag = pixcs->cmapflag;
 
         /* Copy image data */
-    size = pixcs->size;
     datas = pixcs->data;
     if ((datad = (l_uint8 *)LEPT_CALLOC(size, sizeof(l_int8))) == NULL) {
         pixcompDestroy(&pixcd);
@@ -889,9 +895,7 @@ PIXAC   *pixac;
     LEPT_FREE(pixac->pixc);
     boxaDestroy(&pixac->boxa);
     LEPT_FREE(pixac);
-
     *ppixac = NULL;
-    return;
 }
 
 
@@ -974,8 +978,11 @@ l_int32  n;
         return ERROR_INT("invalid copyflag", procName, 1);
 
     n = pixac->n;
-    if (n >= pixac->nalloc)
-        pixacompExtendArray(pixac);
+    if (n >= pixac->nalloc) {
+        if (pixacompExtendArray(pixac))
+            return ERROR_INT("extension failed", procName, 1);
+    }
+
     if (copyflag == L_INSERT)
         pixac->pixc[n] = pixc;
     else  /* L_COPY */
@@ -998,21 +1005,29 @@ l_int32  n;
  *          necessary in case we are NOT adding boxes simultaneously
  *          with adding pixc.  We always want the sizes of the
  *          pixac and boxa ptr arrays to be equal.
+ *      (2) The max number of pixcomp ptrs is 1M.
  * </pre>
  */
 static l_int32
 pixacompExtendArray(PIXAC  *pixac)
 {
+size_t  oldsize, newsize;
+
     PROCNAME("pixacompExtendArray");
 
     if (!pixac)
         return ERROR_INT("pixac not defined", procName, 1);
+    if (pixac->nalloc > MaxPtrArraySize)  /* belt & suspenders */
+        return ERROR_INT("pixac has too many ptrs", procName, 1);
+    oldsize = pixac->nalloc * sizeof(PIXC *);
+    newsize = 2 * oldsize;
+    if (newsize > 8 * MaxPtrArraySize)  /* ptrs for 1M pixcomp */
+        return ERROR_INT("newsize > 8 MB; too large", procName, 1);
 
     if ((pixac->pixc = (PIXC **)reallocNew((void **)&pixac->pixc,
-                            sizeof(PIXC *) * pixac->nalloc,
-                            2 * sizeof(PIXC *) * pixac->nalloc)) == NULL)
+                                           oldsize, newsize)) == NULL)
         return ERROR_INT("new ptr array not returned", procName, 1);
-    pixac->nalloc = 2 * pixac->nalloc;
+    pixac->nalloc *= 2;
     boxaExtendArray(pixac->boxa);
     return 0;
 }
@@ -1675,6 +1690,11 @@ PIXAC  *pixac;
  *
  * \param[in]    fp   file stream
  * \return  pixac, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) It is OK for the pixacomp to be empty.
+ * </pre>
  */
 PIXAC *
 pixacompReadStream(FILE  *fp)
@@ -1701,6 +1721,11 @@ PIXAC    *pixac;
         return (PIXAC *)ERROR_PTR("not a pixacomp file", procName, NULL);
     if (fscanf(fp, "Offset of index into array = %d", &offset) != 1)
         return (PIXAC *)ERROR_PTR("offset not read", procName, NULL);
+    if (n < 0)
+        return (PIXAC *)ERROR_PTR("num pixcomp ptrs < 0", procName, NULL);
+    if (n > MaxPtrArraySize)
+        return (PIXAC *)ERROR_PTR("too many pixcomp ptrs", procName, NULL);
+    if (n == 0) L_INFO("the pixacomp is empty\n", procName);
 
     if ((pixac = pixacompCreate(n)) == NULL)
         return (PIXAC *)ERROR_PTR("pixac not made", procName, NULL);
@@ -1917,6 +1942,9 @@ FILE    *fp;
     if ((fp = open_memstream((char **)pdata, psize)) == NULL)
         return ERROR_INT("stream not opened", procName, 1);
     ret = pixacompWriteStream(fp, pixac);
+    fputc('\0', fp);
+    fclose(fp);
+    *psize = *psize - 1;
 #else
     L_INFO("work-around: writing to a temp file\n", procName);
   #ifdef _WIN32
@@ -1929,8 +1957,8 @@ FILE    *fp;
     ret = pixacompWriteStream(fp, pixac);
     rewind(fp);
     *pdata = l_binaryReadStream(fp, psize);
-#endif  /* HAVE_FMEMOPEN */
     fclose(fp);
+#endif  /* HAVE_FMEMOPEN */
     return ret;
 }
 
