@@ -162,16 +162,18 @@ static l_int32   var_PNG_STRIP_16_TO_8 = 1;
  *          at the beginning of the file.
  *      (2) To do sequential reads of png format images from a stream,
  *          use pixReadStreamPng()
- *      (3) Any image with alpha is converted to RGBA (spp = 4, with
+ *      (3) All images with alpha is converted to RGBA (spp = 4, with
  *          equal red, green and blue channels) on reading.
- *          There are three important cases with alpha:
- *          (a) grayscale-with-alpha (spp = 2), where bpp = 8, and each
+ *          There are three cases with alpha:
+ *          (a) RGBA: spp = 4.  The alpha value is the fourth byte
+ *              (aka "channel, "component") in each 4-byte pixel.
+ *          (b) grayscale-with-alpha (spp = 2), where bpp = 8, and each
  *              pixel has an associated alpha (transparency) value
  *              in the second component of the image data.
- *          (b) spp = 1, d = 1 with colormap and alpha in the trans array.
- *              Transparency is usually associated with the white background.
- *          (c) spp = 1, d = 8 with colormap and alpha in the trans array.
- *              Each color in the colormap has a separate transparency value.
+ *          (c) colormap (spp = 1) with alpha in the trans palette.
+ *              d = 1, 2, 4, 8.  The trans palette in writing is derived
+ *              from the alpha components in the cmap.  Transparency is
+ *              often associated with a white background.
  *      (4) We use the high level png interface, where the transforms are set
  *          up in advance and the header and image are read with a single
  *          call.  The more complicated interface, where the header is
@@ -185,15 +187,13 @@ PIX *
 pixReadStreamPng(FILE  *fp)
 {
 l_uint8      byte;
-l_int32      rval, gval, bval;
-l_int32      i, j, k, index, ncolors, bitval;
-l_int32      wpl, d, spp, cindex, tRNS;
+l_int32      i, j, k, index, ncolors, rval, gval, bval, valid;
+l_int32      wpl, d, spp, cindex, bitval, bival, quadval, tRNS;
 l_uint32     png_transforms;
 l_uint32    *data, *line, *ppixel;
 int          num_palette, num_text, num_trans;
 png_byte     bit_depth, color_type, channels;
-png_uint_32  w, h, rowbytes;
-png_uint_32  xres, yres;
+png_uint_32  w, h, rowbytes, xres, yres;
 png_bytep    rowptr, trans;
 png_bytep   *row_pointers;
 png_structp  png_ptr;
@@ -296,8 +296,11 @@ PIXCMAP     *cmap;
     pixSetInputFormat(pix, IFF_PNG);
     wpl = pixGetWpl(pix);
     data = pixGetData(pix);
-    pixSetColormap(pix, cmap);
     pixSetSpp(pix, spp);
+    if (pixSetColormap(pix, cmap)) {
+        pixDestroy(&pix);
+        return (PIX *)ERROR_PTR("invalid colormap", procName, NULL);
+    }
 
     if (spp == 1 && !tRNS) {  /* copy straight from buffer to pix */
         for (i = 0; i < h; i++) {
@@ -342,7 +345,9 @@ PIXCMAP     *cmap;
         /* Special spp == 1 cases with transparency:
          *    (1) 8 bpp without colormap; assume full transparency
          *    (2) 1 bpp with colormap + trans array (for alpha)
-         *    (3) 8 bpp with colormap + trans array (for alpha)
+         *    (3) 2 bpp with colormap + trans array (for alpha)
+         *    (4) 4 bpp with colormap + trans array (for alpha)
+         *    (5) 8 bpp with colormap + trans array (for alpha)
          * These all require converting to RGBA */
     if (spp == 1 && tRNS) {
         if (!cmap) {
@@ -413,8 +418,48 @@ PIXCMAP     *cmap;
                         }
                     }
                 }
+            } else if (d == 2) {
+                    /* Case 3: 2 bpp with cmap and associated transparency */
+                L_INFO("converting 2 bpp cmap with alpha ==> RGBA\n", procName);
+                for (i = 0; i < h; i++) {
+                    ppixel = data + i * wpl;
+                    rowptr = row_pointers[i];
+                    for (j = 0, index = 0; j < rowbytes; j++) {
+                        byte = rowptr[j];
+                        for (k = 0; k < 4 && index < w; k++, index++) {
+                            bival = (byte >> 2 * (3 - k)) & 3;
+                            pixcmapGetColor(cmap, bival, &rval, &gval, &bval);
+                            composeRGBPixel(rval, gval, bval, ppixel);
+                                /* Assume missing entries to be 255 (opaque)
+                                 * according to the spec:
+                                 * http://www.w3.org/TR/PNG/#11tRNS */
+                            SET_DATA_BYTE(ppixel, L_ALPHA_CHANNEL,
+                                bival < num_trans ? trans[bival] : 255);
+                            ppixel++;
+                        }
+                    }
+                }
+            } else if (d == 4) {
+                    /* Case 4: 4 bpp with cmap and associated transparency */
+                L_INFO("converting 4 bpp cmap with alpha ==> RGBA\n", procName);
+                for (i = 0; i < h; i++) {
+                    ppixel = data + i * wpl;
+                    rowptr = row_pointers[i];
+                    for (j = 0, index = 0; j < rowbytes; j++) {
+                        byte = rowptr[j];
+                        for (k = 0; k < 2 && index < w; k++, index++) {
+                            quadval = (byte >> 4 * (1 - k)) & 0xf;
+                            pixcmapGetColor(cmap, quadval, &rval, &gval, &bval);
+                            composeRGBPixel(rval, gval, bval, ppixel);
+                                /* Assume missing entries to be 255 (opaque) */
+                            SET_DATA_BYTE(ppixel, L_ALPHA_CHANNEL,
+                                quadval < num_trans ? trans[quadval] : 255);
+                            ppixel++;
+                        }
+                    }
+                }
             } else if (d == 8) {
-                    /* Case 3: 8 bpp with cmap and associated transparency */
+                    /* Case 5: 8 bpp with cmap and associated transparency */
                 L_INFO("converting 8 bpp cmap with alpha ==> RGBA\n", procName);
                 for (i = 0; i < h; i++) {
                     ppixel = data + i * wpl;
@@ -423,9 +468,7 @@ PIXCMAP     *cmap;
                         index = rowptr[j];
                         pixcmapGetColor(cmap, index, &rval, &gval, &bval);
                         composeRGBPixel(rval, gval, bval, ppixel);
-                            /* Assume missing entries to be 255 (opaque)
-                             * according to the spec:
-                             * http://www.w3.org/TR/PNG/#11tRNS */
+                            /* Assume missing entries to be 255 (opaque) */
                         SET_DATA_BYTE(ppixel, L_ALPHA_CHANNEL,
                                       index < num_trans ? trans[index] : 255);
                         ppixel++;
@@ -491,6 +534,17 @@ PIXCMAP     *cmap;
         pixSetText(pix, text_ptr->text);
 
     png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
+
+        /* Final validity check on the colormap */
+    if ((cmap = pixGetColormap(pix)) != NULL) {
+        pixcmapIsValid(cmap, pix, &valid);
+        if (!valid) {
+            pixDestroy(&pix);
+            return (PIX *)ERROR_PTR("colormap is not valid", procName, NULL);
+        }
+    }
+
+    pixSetPadBits(pix, 0);
     return pix;
 }
 
@@ -1020,9 +1074,8 @@ pixWriteStreamPng(FILE      *fp,
                   l_float32  gamma)
 {
 char         commentstring[] = "Comment";
-l_int32      i, j, k;
-l_int32      wpl, d, spp, cmflag, opaque;
-l_int32      ncolors, compval;
+l_int32      i, j, k, wpl, d, spp, compval, valid;
+l_int32      cmflag, opaque, max_trans, ncolors;
 l_int32     *rmap, *gmap, *bmap, *amap;
 l_uint32    *data, *ppixel;
 png_byte     bit_depth, color_type;
@@ -1045,42 +1098,20 @@ char        *text;
     if (!pix)
         return ERROR_INT("pix not defined", procName, 1);
 
-        /* Allocate the 2 data structures */
-    if ((png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
-                   (png_voidp)NULL, NULL, NULL)) == NULL)
-        return ERROR_INT("png_ptr not made", procName, 1);
-
-    if ((info_ptr = png_create_info_struct(png_ptr)) == NULL) {
-        png_destroy_write_struct(&png_ptr, (png_infopp)NULL);
-        return ERROR_INT("info_ptr not made", procName, 1);
-    }
-
-        /* Set up png setjmp error handling */
-    if (setjmp(png_jmpbuf(png_ptr))) {
-        png_destroy_write_struct(&png_ptr, &info_ptr);
-        return ERROR_INT("internal png error", procName, 1);
-    }
-
-    png_init_io(png_ptr, fp);
-
-        /* With best zlib compression (9), get between 1 and 10% improvement
-         * over default (6), but the compression is 3 to 10 times slower.
-         * Use the zlib default (6) as our default compression unless
-         * pix->special falls in the range [10 ... 19]; then subtract 10
-         * to get the compression value.  */
-    compval = Z_DEFAULT_COMPRESSION;
-    if (pix->special >= 10 && pix->special < 20)
-        compval = pix->special - 10;
-    png_set_compression_level(png_ptr, compval);
-
     w = pixGetWidth(pix);
     h = pixGetHeight(pix);
     d = pixGetDepth(pix);
     spp = pixGetSpp(pix);
-    if ((cmap = pixGetColormap(pix)))
+
+        /* A cmap validity check should prevent low-level colormap errors. */
+    if ((cmap = pixGetColormap(pix))) {
         cmflag = 1;
-    else
+        pixcmapIsValid(cmap, pix, &valid);
+        if (!valid)
+            return ERROR_INT("colormap is not valid", procName, 1);
+    } else {
         cmflag = 0;
+    }
     pixSetPadBits(pix, 0);
 
         /* Set the color type and bit depth. */
@@ -1104,6 +1135,37 @@ char        *text;
                 cmflag, bit_depth, color_type);
 #endif  /* DEBUG_WRITE */
 
+        /* Allocate the 2 png data structures */
+    if ((png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
+                   (png_voidp)NULL, NULL, NULL)) == NULL)
+        return ERROR_INT("png_ptr not made", procName, 1);
+    if ((info_ptr = png_create_info_struct(png_ptr)) == NULL) {
+        png_destroy_write_struct(&png_ptr, (png_infopp)NULL);
+        return ERROR_INT("info_ptr not made", procName, 1);
+    }
+
+        /* Set up png setjmp error handling */
+    pix1 = NULL;
+    row_pointers = NULL;
+    if (setjmp(png_jmpbuf(png_ptr))) {
+        png_destroy_write_struct(&png_ptr, &info_ptr);
+        LEPT_FREE(row_pointers);
+        pixDestroy(&pix1);
+        return ERROR_INT("internal png error", procName, 1);
+    }
+
+    png_init_io(png_ptr, fp);
+
+        /* With best zlib compression (9), get between 1 and 10% improvement
+         * over default (6), but the compression is 3 to 10 times slower.
+         * Use the zlib default (6) as our default compression unless
+         * pix->special falls in the range [10 ... 19]; then subtract 10
+         * to get the compression value.  */
+    compval = Z_DEFAULT_COMPRESSION;
+    if (pix->special >= 10 && pix->special < 20)
+        compval = pix->special - 10;
+    png_set_compression_level(png_ptr, compval);
+
     png_set_IHDR(png_ptr, info_ptr, w, h, bit_depth, color_type,
                  PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_BASE,
                  PNG_FILTER_TYPE_BASE);
@@ -1117,27 +1179,34 @@ char        *text;
         png_set_pHYs(png_ptr, info_ptr, xres, yres, PNG_RESOLUTION_METER);
 
     if (cmflag) {
-        pixcmapToArrays(cmap, &rmap, &gmap, &bmap, &amap);
-        ncolors = pixcmapGetCount(cmap);
-        pixcmapIsOpaque(cmap, &opaque);
-
             /* Make and save the palette */
+        ncolors = pixcmapGetCount(cmap);
         palette = (png_colorp)LEPT_CALLOC(ncolors, sizeof(png_color));
+        pixcmapToArrays(cmap, &rmap, &gmap, &bmap, &amap);
         for (i = 0; i < ncolors; i++) {
             palette[i].red = (png_byte)rmap[i];
             palette[i].green = (png_byte)gmap[i];
             palette[i].blue = (png_byte)bmap[i];
             alpha[i] = (png_byte)amap[i];
         }
-
-        png_set_PLTE(png_ptr, info_ptr, palette, (int)ncolors);
-        if (!opaque)  /* alpha channel has some transparency; assume valid */
-            png_set_tRNS(png_ptr, info_ptr, (png_bytep)alpha,
-                         (int)ncolors, NULL);
         LEPT_FREE(rmap);
         LEPT_FREE(gmap);
         LEPT_FREE(bmap);
         LEPT_FREE(amap);
+        png_set_PLTE(png_ptr, info_ptr, palette, (int)ncolors);
+        LEPT_FREE(palette);
+
+            /* Add the tRNS chunk.  If the non-opaque colors are listed
+             * first in the colormap, as in the spec, we can use that in
+             * the 4th arg of png_set_tRNS.  Otherwise, transparency will
+             * be lost for some colors.  To prevent that, see the comments
+             * in pixcmapNonOpaqueColorsInfo(). */
+        pixcmapIsOpaque(cmap, &opaque);
+        if (!opaque) {  /* alpha channel has some transparency; assume valid */
+            pixcmapNonOpaqueColorsInfo(cmap, NULL, &max_trans, NULL);
+            png_set_tRNS(png_ptr, info_ptr, (png_bytep)alpha,
+                         max_trans + 1, NULL);
+        }
     }
 
         /* 0.4545 is treated as the default by some image
@@ -1178,7 +1247,6 @@ char        *text;
         }
         if (!pix1) {
             png_destroy_write_struct(&png_ptr, &info_ptr);
-            if (cmflag) LEPT_FREE(palette);
             return ERROR_INT("pix1 not made", procName, 1);
         }
 
@@ -1193,8 +1261,6 @@ char        *text;
             /* Transfer the data */
         png_write_image(png_ptr, row_pointers);
         png_write_end(png_ptr, info_ptr);
-
-        if (cmflag) LEPT_FREE(palette);
         LEPT_FREE(row_pointers);
         pixDestroy(&pix1);
         png_destroy_write_struct(&png_ptr, &info_ptr);
@@ -1229,9 +1295,6 @@ char        *text;
     }
 
     png_write_end(png_ptr, info_ptr);
-
-    if (cmflag)
-        LEPT_FREE(palette);
     png_destroy_write_struct(&png_ptr, &info_ptr);
     return 0;
 }
@@ -1533,24 +1596,22 @@ pixReadMemPng(const l_uint8  *filedata,
               size_t          filesize)
 {
 l_uint8      byte;
-l_int32      rval, gval, bval;
-l_int32      i, j, k, index, ncolors, bitval;
-l_int32      wpl, d, spp, cindex, tRNS;
+l_int32      i, j, k, index, ncolors, rval, gval, bval, valid;
+l_int32      wpl, d, spp, cindex, bitval, bival, quadval, tRNS;
 l_uint32     png_transforms;
 l_uint32    *data, *line, *ppixel;
 int          num_palette, num_text, num_trans;
 png_byte     bit_depth, color_type, channels;
-png_uint_32  w, h, rowbytes;
-png_uint_32  xres, yres;
+png_uint_32  w, h, rowbytes, xres, yres;
 png_bytep    rowptr, trans;
 png_bytep   *row_pointers;
 png_structp  png_ptr;
 png_infop    info_ptr, end_info;
 png_colorp   palette;
 png_textp    text_ptr;  /* ptr to text_chunk */
+MEMIODATA    state;
 PIX         *pix, *pix1;
 PIXCMAP     *cmap;
-MEMIODATA    state;
 
     PROCNAME("pixReadMemPng");
 
@@ -1652,8 +1713,11 @@ MEMIODATA    state;
     pixSetInputFormat(pix, IFF_PNG);
     wpl = pixGetWpl(pix);
     data = pixGetData(pix);
-    pixSetColormap(pix, cmap);
     pixSetSpp(pix, spp);
+    if (pixSetColormap(pix, cmap)) {
+        pixDestroy(&pix);
+        return (PIX *)ERROR_PTR("invalid colormap", procName, NULL);
+    }
 
     if (spp == 1 && !tRNS) {  /* copy straight from buffer to pix */
         for (i = 0; i < h; i++) {
@@ -1696,7 +1760,9 @@ MEMIODATA    state;
         /* Special spp == 1 cases with transparency:
          *    (1) 8 bpp without colormap; assume full transparency
          *    (2) 1 bpp with colormap + trans array (for alpha)
-         *    (3) 8 bpp with colormap + trans array (for alpha)
+         *    (3) 2 bpp with colormap + trans array (for alpha)
+         *    (4) 4 bpp with colormap + trans array (for alpha)
+         *    (5) 8 bpp with colormap + trans array (for alpha)
          * These all require converting to RGBA */
     if (spp == 1 && tRNS) {
         if (!cmap) {
@@ -1767,8 +1833,48 @@ MEMIODATA    state;
                         }
                     }
                 }
+            } else if (d == 2) {
+                    /* Case 3: 2 bpp with cmap and associated transparency */
+                L_INFO("converting 2 bpp cmap with alpha ==> RGBA\n", procName);
+                for (i = 0; i < h; i++) {
+                    ppixel = data + i * wpl;
+                    rowptr = row_pointers[i];
+                    for (j = 0, index = 0; j < rowbytes; j++) {
+                        byte = rowptr[j];
+                        for (k = 0; k < 4 && index < w; k++, index++) {
+                            bival = (byte >> 2 * (3 - k)) & 3;
+                            pixcmapGetColor(cmap, bival, &rval, &gval, &bval);
+                            composeRGBPixel(rval, gval, bval, ppixel);
+                                /* Assume missing entries to be 255 (opaque)
+                                 * according to the spec:
+                                 * http://www.w3.org/TR/PNG/#11tRNS */
+                            SET_DATA_BYTE(ppixel, L_ALPHA_CHANNEL,
+                                bival < num_trans ? trans[bival] : 255);
+                            ppixel++;
+                        }
+                    }
+                }
+            } else if (d == 4) {
+                    /* Case 4: 4 bpp with cmap and associated transparency */
+                L_INFO("converting 4 bpp cmap with alpha ==> RGBA\n", procName);
+                for (i = 0; i < h; i++) {
+                    ppixel = data + i * wpl;
+                    rowptr = row_pointers[i];
+                    for (j = 0, index = 0; j < rowbytes; j++) {
+                        byte = rowptr[j];
+                        for (k = 0; k < 2 && index < w; k++, index++) {
+                            quadval = (byte >> 4 * (1 - k)) & 0xf;
+                            pixcmapGetColor(cmap, quadval, &rval, &gval, &bval);
+                            composeRGBPixel(rval, gval, bval, ppixel);
+                                /* Assume missing entries to be 255 (opaque) */
+                            SET_DATA_BYTE(ppixel, L_ALPHA_CHANNEL,
+                                quadval < num_trans ? trans[quadval] : 255);
+                            ppixel++;
+                        }
+                    }
+                }
             } else if (d == 8) {
-                    /* Case 3: 8 bpp with cmap and associated transparency */
+                    /* Case 5: 8 bpp with cmap and associated transparency */
                 L_INFO("converting 8 bpp cmap with alpha ==> RGBA\n", procName);
                 for (i = 0; i < h; i++) {
                     ppixel = data + i * wpl;
@@ -1844,6 +1950,17 @@ MEMIODATA    state;
         pixSetText(pix, text_ptr->text);
 
     png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
+
+       /* Final validity check on the colormap */
+    if ((cmap = pixGetColormap(pix)) != NULL) {
+        pixcmapIsValid(cmap, pix, &valid);
+        if (!valid) {
+            pixDestroy(&pix);
+            return (PIX *)ERROR_PTR("colormap is not valid", procName, NULL);
+        }
+    }
+
+    pixSetPadBits(pix, 0);
     return pix;
 }
 
@@ -1872,16 +1989,12 @@ pixWriteMemPng(l_uint8  **pfiledata,
                l_float32  gamma)
 {
 char         commentstring[] = "Comment";
-l_int32      i, j, k;
-l_int32      wpl, d, spp, cmflag, opaque;
-l_int32      ncolors, compval;
+l_int32      i, j, k, wpl, d, spp, cmflag, opaque, ncolors, compval, valid;
 l_int32     *rmap, *gmap, *bmap, *amap;
 l_uint32    *data, *ppixel;
 png_byte     bit_depth, color_type;
 png_byte     alpha[256];
-png_uint_32  w, h;
-png_uint_32  xres, yres;
-png_bytep   *row_pointers;
+png_uint_32  w, h, xres, yres;
 png_bytep    rowbuffer;
 png_structp  png_ptr;
 png_infop    info_ptr;
@@ -1908,43 +2021,22 @@ MEMIODATA    state;
     state.m_Count = 0;
     state.m_Last = &state;
 
-        /* Allocate the 2 data structures */
-    if ((png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
-                   (png_voidp)NULL, NULL, NULL)) == NULL)
-        return ERROR_INT("png_ptr not made", procName, 1);
-
-    if ((info_ptr = png_create_info_struct(png_ptr)) == NULL) {
-        png_destroy_write_struct(&png_ptr, (png_infopp)NULL);
-        return ERROR_INT("info_ptr not made", procName, 1);
-    }
-
-        /* Set up png setjmp error handling */
-    if (setjmp(png_jmpbuf(png_ptr))) {
-        png_destroy_write_struct(&png_ptr, &info_ptr);
-        return ERROR_INT("internal png error", procName, 1);
-    }
-
-    png_set_write_fn(png_ptr, &state, memio_png_write_data,
-                     (png_flush_ptr)NULL);
-
-        /* With best zlib compression (9), get between 1 and 10% improvement
-         * over default (6), but the compression is 3 to 10 times slower.
-         * Use the zlib default (6) as our default compression unless
-         * pix->special falls in the range [10 ... 19]; then subtract 10
-         * to get the compression value.  */
-    compval = Z_DEFAULT_COMPRESSION;
-    if (pix->special >= 10 && pix->special < 20)
-        compval = pix->special - 10;
-    png_set_compression_level(png_ptr, compval);
-
     w = pixGetWidth(pix);
     h = pixGetHeight(pix);
     d = pixGetDepth(pix);
     spp = pixGetSpp(pix);
-    if ((cmap = pixGetColormap(pix)))
+
+        /* A cmap validity check should prevent low-level colormap errors. */
+    if ((cmap = pixGetColormap(pix))) {
         cmflag = 1;
-    else
+        pixcmapIsValid(cmap, pix, &valid);
+        if (!valid)
+            return ERROR_INT("colormap is not valid", procName, 1);
+    } else {
         cmflag = 0;
+    }
+
+    pixSetPadBits(pix, 0);
 
         /* Set the color type and bit depth. */
     if (d == 32 && spp == 4) {
@@ -1967,6 +2059,37 @@ MEMIODATA    state;
                 cmflag, bit_depth, color_type);
 #endif  /* DEBUG_WRITE */
 
+        /* Allocate the 2 data structures */
+    if ((png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
+                   (png_voidp)NULL, NULL, NULL)) == NULL)
+        return ERROR_INT("png_ptr not made", procName, 1);
+
+    if ((info_ptr = png_create_info_struct(png_ptr)) == NULL) {
+        png_destroy_write_struct(&png_ptr, (png_infopp)NULL);
+        return ERROR_INT("info_ptr not made", procName, 1);
+    }
+
+        /* Set up png setjmp error handling */
+    pix1 = NULL;
+    if (setjmp(png_jmpbuf(png_ptr))) {
+        png_destroy_write_struct(&png_ptr, &info_ptr);
+        pixDestroy(&pix1);
+        return ERROR_INT("internal png error", procName, 1);
+    }
+
+    png_set_write_fn(png_ptr, &state, memio_png_write_data,
+                     (png_flush_ptr)NULL);
+
+        /* With best zlib compression (9), get between 1 and 10% improvement
+         * over default (6), but the compression is 3 to 10 times slower.
+         * Use the zlib default (6) as our default compression unless
+         * pix->special falls in the range [10 ... 19]; then subtract 10
+         * to get the compression value.  */
+    compval = Z_DEFAULT_COMPRESSION;
+    if (pix->special >= 10 && pix->special < 20)
+        compval = pix->special - 10;
+    png_set_compression_level(png_ptr, compval);
+
     png_set_IHDR(png_ptr, info_ptr, w, h, bit_depth, color_type,
                  PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_BASE,
                  PNG_FILTER_TYPE_BASE);
@@ -1980,27 +2103,27 @@ MEMIODATA    state;
         png_set_pHYs(png_ptr, info_ptr, xres, yres, PNG_RESOLUTION_METER);
 
     if (cmflag) {
-        pixcmapToArrays(cmap, &rmap, &gmap, &bmap, &amap);
-        ncolors = pixcmapGetCount(cmap);
-        pixcmapIsOpaque(cmap, &opaque);
-
             /* Make and save the palette */
+        ncolors = pixcmapGetCount(cmap);
         palette = (png_colorp)LEPT_CALLOC(ncolors, sizeof(png_color));
+        pixcmapToArrays(cmap, &rmap, &gmap, &bmap, &amap);
         for (i = 0; i < ncolors; i++) {
             palette[i].red = (png_byte)rmap[i];
             palette[i].green = (png_byte)gmap[i];
             palette[i].blue = (png_byte)bmap[i];
             alpha[i] = (png_byte)amap[i];
         }
-
-        png_set_PLTE(png_ptr, info_ptr, palette, (int)ncolors);
-        if (!opaque)  /* alpha channel has some transparency; assume valid */
-            png_set_tRNS(png_ptr, info_ptr, (png_bytep)alpha,
-                         (int)ncolors, NULL);
         LEPT_FREE(rmap);
         LEPT_FREE(gmap);
         LEPT_FREE(bmap);
         LEPT_FREE(amap);
+        png_set_PLTE(png_ptr, info_ptr, palette, (int)ncolors);
+        LEPT_FREE(palette);
+
+        pixcmapIsOpaque(cmap, &opaque);
+        if (!opaque)  /* alpha channel has some transparency; assume valid */
+            png_set_tRNS(png_ptr, info_ptr, (png_bytep)alpha,
+                         (int)ncolors, NULL);
     }
 
         /* 0.4545 is treated as the default by some image
@@ -2041,28 +2164,19 @@ MEMIODATA    state;
         }
         if (!pix1) {
             png_destroy_write_struct(&png_ptr, &info_ptr);
-            if (cmflag) LEPT_FREE(palette);
             memio_free(&state);
             return ERROR_INT("pix1 not made", procName, 1);
         }
 
-            /* Make and assign array of image row pointers */
-        row_pointers = (png_bytep *)LEPT_CALLOC(h, sizeof(png_bytep));
+            /* Transfer the data */
         wpl = pixGetWpl(pix1);
         data = pixGetData(pix1);
         for (i = 0; i < h; i++)
-            row_pointers[i] = (png_bytep)(data + i * wpl);
-        png_set_rows(png_ptr, info_ptr, row_pointers);
-
-            /* Transfer the data */
-        png_write_image(png_ptr, row_pointers);
+            png_write_row(png_ptr, (png_bytep)(data + i * wpl));
         png_write_end(png_ptr, info_ptr);
 
-        if (cmflag) LEPT_FREE(palette);
-        LEPT_FREE(row_pointers);
         pixDestroy(&pix1);
         png_destroy_write_struct(&png_ptr, &info_ptr);
-
         memio_png_flush(&state);
         *pfiledata = (l_uint8 *)state.m_Buffer;
         state.m_Buffer = 0;
@@ -2097,13 +2211,9 @@ MEMIODATA    state;
         }
         LEPT_FREE(rowbuffer);
     }
-
     png_write_end(png_ptr, info_ptr);
 
-    if (cmflag)
-        LEPT_FREE(palette);
     png_destroy_write_struct(&png_ptr, &info_ptr);
-
     memio_png_flush(&state);
     *pfiledata = (l_uint8 *)state.m_Buffer;
     state.m_Buffer = 0;
