@@ -33,7 +33,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config_auto.h"
+#include <config_auto.h>
 #endif  /* HAVE_CONFIG_H */
 
 #include "allheaders.h"
@@ -49,25 +49,25 @@ PIX * pixReadStreamPng(FILE *fp)
 
 /* ----------------------------------------------------------------------*/
 
-l_int32 readHeaderPng(const char *filename, l_int32 *pwidth, l_int32 *pheight,
-                      l_int32 *pbps, l_int32 *pspp, l_int32 *piscmap)
+l_ok readHeaderPng(const char *filename, l_int32 *pwidth, l_int32 *pheight,
+                   l_int32 *pbps, l_int32 *pspp, l_int32 *piscmap)
 {
     return ERROR_INT("function not present", "readHeaderPng", 1);
 }
 
 /* ----------------------------------------------------------------------*/
 
-l_int32 freadHeaderPng(FILE *fp, l_int32 *pwidth, l_int32 *pheight,
-                       l_int32 *pbps, l_int32 *pspp, l_int32 *piscmap)
+l_ok freadHeaderPng(FILE *fp, l_int32 *pwidth, l_int32 *pheight,
+                    l_int32 *pbps, l_int32 *pspp, l_int32 *piscmap)
 {
     return ERROR_INT("function not present", "freadHeaderPng", 1);
 }
 
 /* ----------------------------------------------------------------------*/
 
-l_int32 readHeaderMemPng(const l_uint8 *data, size_t size, l_int32 *pwidth,
-                         l_int32 *pheight, l_int32 *pbps, l_int32 *pspp,
-                         l_int32 *piscmap)
+l_ok readHeaderMemPng(const l_uint8 *data, size_t size, l_int32 *pwidth,
+                      l_int32 *pheight, l_int32 *pbps, l_int32 *pspp,
+                      l_int32 *piscmap)
 {
     return ERROR_INT("function not present", "readHeaderMemPng", 1);
 }
@@ -81,35 +81,35 @@ l_int32 fgetPngResolution(FILE *fp, l_int32 *pxres, l_int32 *pyres)
 
 /* ----------------------------------------------------------------------*/
 
-l_int32 isPngInterlaced(const char *filename, l_int32 *pinterlaced)
+l_ok isPngInterlaced(const char *filename, l_int32 *pinterlaced)
 {
     return ERROR_INT("function not present", "isPngInterlaced", 1);
 }
 
 /* ----------------------------------------------------------------------*/
 
-l_int32 fgetPngColormapInfo(FILE *fp, PIXCMAP **pcmap, l_int32 *ptransparency)
+l_ok fgetPngColormapInfo(FILE *fp, PIXCMAP **pcmap, l_int32 *ptransparency)
 {
     return ERROR_INT("function not present", "fgetPngColormapInfo", 1);
 }
 
 /* ----------------------------------------------------------------------*/
 
-l_int32 pixWritePng(const char *filename, PIX *pix, l_float32 gamma)
+l_ok pixWritePng(const char *filename, PIX *pix, l_float32 gamma)
 {
     return ERROR_INT("function not present", "pixWritePng", 1);
 }
 
 /* ----------------------------------------------------------------------*/
 
-l_int32 pixWriteStreamPng(FILE *fp, PIX *pix, l_float32 gamma)
+l_ok pixWriteStreamPng(FILE *fp, PIX *pix, l_float32 gamma)
 {
     return ERROR_INT("function not present", "pixWriteStreamPng", 1);
 }
 
 /* ----------------------------------------------------------------------*/
 
-l_int32 pixSetZlibCompression(PIX *pix, l_int32 compval)
+l_ok pixSetZlibCompression(PIX *pix, l_int32 compval)
 
 {
     return ERROR_INT("function not present", "pixSetZlibCompression", 1);
@@ -132,8 +132,8 @@ PIX * pixReadMemPng(const l_uint8 *filedata, size_t filesize)
 
 /* ----------------------------------------------------------------------*/
 
-l_int32 pixWriteMemPng(l_uint8 **pfiledata, size_t *pfilesize, PIX *pix,
-                       l_float32 gamma)
+l_ok pixWriteMemPng(l_uint8 **pfiledata, size_t *pfilesize, PIX *pix,
+                    l_float32 gamma)
 {
     return ERROR_INT("function not present", "pixWriteMemPng", 1);
 }
