@@ -1,20 +1,32 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
-/*
- *  rank.c
+/*!
+ * \file rank.c
+ * <pre>
  *
  *      Rank filter (gray and rgb)
  *          PIX      *pixRankFilter()
@@ -24,6 +36,9 @@
  *      Median filter
  *          PIX      *pixMedianFilter()
  *
+ *      Rank filter (accelerated with downscaling)
+ *          PIX      *pixRankFilterWithScaling()
+ *
  *  What is a brick rank filter?
  *
  *    A brick rank order filter evaluates, for every pixel in the image,
@@ -80,7 +95,7 @@
  *      * Represented as a linked list.  This would overcome the
  *        summing-over-empty-bin problem, but you lose random access
  *        for insertions and deletions.  No way.
- *  
+ *
  *      * Two histogram solution.  Maintain two histograms with
  *        bin sizes of 1 and 16.  Proceed from coarse to fine.
  *        First locate the coarse bin for the given rank, of which
@@ -90,25 +105,38 @@
  *        coarse and fine histograms, is thus 16.
  *
  *  If someone has a better method, please let me know!
+ *
+ *  The rank filtering operation is relatively expensive, compared to most
+ *  of the other imaging operations.  The speed is only weakly dependent
+ *  on the size of the rank filter.  On standard hardware, it runs at
+ *  about 10 Mpix/sec for a 50 x 50 filter, and 25 Mpix/sec for
+ *  a 5 x 5 filter.   For applications where the rank filter can be
+ *  performed on a downscaled image, significant speedup can be
+ *  achieved because the time goes as the square of the scaling factor.
+ *  We provide an interface that handles the details, and only
+ *  requires the amount of downscaling to be input.
+ * </pre>
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include "allheaders.h"
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
 
+#include "allheaders.h"
 
 /*----------------------------------------------------------------------*
  *                           Rank order filter                          *
  *----------------------------------------------------------------------*/
 /*!
- *  pixRankFilter()
+ * \brief   pixRankFilter()
  *
- *      Input:  pixs (8 or 32 bpp; no colormap)
- *              wf, hf  (width and height of filter; each is >= 1)
- *              rank (in [0.0 ... 1.0])
- *      Return: pixd (of rank values), or null on error
+ * \param[in]    pixs     8 or 32 bpp; no colormap
+ * \param[in]    wf, hf   width and height of filter; each is >= 1
+ * \param[in]    rank     in [0.0 ... 1.0]
+ * \return  pixd of rank values, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This defines, for each pixel in pixs, a neighborhood of
  *          pixels given by a rectangle "centered" on the pixel.
  *          This set of wf*hf pixels has a distribution of values.
@@ -117,6 +145,7 @@
  *          pixels have a lower or equal value and
  *          (1-rank)*(wf*hf-1) pixels have an equal or greater value.
  *      (2) See notes in pixRankFilterGray() for further details.
+ * </pre>
  */
 PIX  *
 pixRankFilter(PIX       *pixs,
@@ -150,14 +179,15 @@ l_int32  d;
 
 
 /*!
- *  pixRankFilterRGB()
+ * \brief   pixRankFilterRGB()
  *
- *      Input:  pixs (32 bpp)
- *              wf, hf  (width and height of filter; each is >= 1)
- *              rank (in [0.0 ... 1.0])
- *      Return: pixd (of rank values), or null on error
+ * \param[in]    pixs     32 bpp
+ * \param[in]    wf, hf   width and height of filter; each is >= 1
+ * \param[in]    rank     in [0.0 ... 1.0]
+ * \return  pixd of rank values, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This defines, for each pixel in pixs, a neighborhood of
  *          pixels given by a rectangle "centered" on the pixel.
  *          This set of wf*hf pixels has a distribution of values.
@@ -167,6 +197,7 @@ l_int32  d;
  *          (1-rank)*(wf*hf-1) pixels have an equal or greater value.
  *      (2) Apply gray rank filtering to each component independently.
  *      (3) See notes in pixRankFilterGray() for further details.
+ * </pre>
  */
 PIX  *
 pixRankFilterRGB(PIX       *pixs,
@@ -209,14 +240,15 @@ PIX  *pixr, *pixg, *pixb, *pixrf, *pixgf, *pixbf, *pixd;
 
 
 /*!
- *  pixRankFilterGray()
+ * \brief   pixRankFilterGray()
  *
- *      Input:  pixs (8 bpp; no colormap)
- *              wf, hf  (width and height of filter; each is >= 1)
- *              rank (in [0.0 ... 1.0])
- *      Return: pixd (of rank values), or null on error
+ * \param[in]    pixs     8 bpp; no colormap
+ * \param[in]    wf, hf   width and height of filter; each is >= 1
+ * \param[in]    rank     in [0.0 ... 1.0]
+ * \return  pixd of rank values, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This defines, for each pixel in pixs, a neighborhood of
  *          pixels given by a rectangle "centered" on the pixel.
  *          This set of wf*hf pixels has a distribution of values,
@@ -233,6 +265,7 @@ PIX  *pixr, *pixg, *pixb, *pixrf, *pixgf, *pixbf, *pixd;
  *      (5) Returns a copy if both wf and hf are 1.
  *      (6) Uses row-major or column-major incremental updates to the
  *          histograms depending on whether hf > wf or hv <= wf, rsp.
+ * </pre>
  */
 PIX  *
 pixRankFilterGray(PIX       *pixs,
@@ -284,8 +317,8 @@ PIX       *pixt, *pixd;
         return (PIX *)ERROR_PTR("pixt not made", procName, NULL);
 
         /* Set up the two histogram arrays. */
-    histo = (l_int32 *)CALLOC(256, sizeof(l_int32));
-    histo16 = (l_int32 *)CALLOC(16, sizeof(l_int32));
+    histo = (l_int32 *)LEPT_CALLOC(256, sizeof(l_int32));
+    histo16 = (l_int32 *)LEPT_CALLOC(16, sizeof(l_int32));
     rankloc = (l_int32)(rank * wf * hf);
 
         /* Place the filter center at (0, 0).  This is just a
@@ -312,7 +345,7 @@ PIX       *pixt, *pixd;
                 lined = datad + i * wpld;
                 if (i == 0) {  /* do full histo */
                     for (k = 0; k < hf; k++) {
-                        linet = datat + (i + k) * wplt; 
+                        linet = datat + (i + k) * wplt;
                         for (m = 0; m < wf; m++) {
                             val = GET_DATA_BYTE(linet, j + m);
                             histo[val]++;
@@ -343,11 +376,16 @@ PIX       *pixt, *pixd;
                         break;
                     }
                 }
+                if (n == 16) {  /* avoid accessing out of bounds */
+                    L_WARNING("n = 16; reducing\n", procName);
+                    n = 15;
+                    sum -= histo16[n];
+                }
                 k = 16 * n;  /* starting value in fine histo */
                 for (m = 0; m < 16; m++) {
                     sum += histo[k];
                     if (sum > rankloc) {
-                        SET_DATA_BYTE(lined, j, k); 
+                        SET_DATA_BYTE(lined, j, k);
                         break;
                     }
                     k++;
@@ -366,7 +404,7 @@ PIX       *pixt, *pixd;
                     /* Update the histos for the new location */
                 if (j == 0) {  /* do full histo */
                     for (k = 0; k < hf; k++) {
-                        linet = datat + (i + k) * wplt; 
+                        linet = datat + (i + k) * wplt;
                         for (m = 0; m < wf; m++) {
                             val = GET_DATA_BYTE(linet, j + m);
                             histo[val]++;
@@ -394,11 +432,16 @@ PIX       *pixt, *pixd;
                         break;
                     }
                 }
+                if (n == 16) {  /* avoid accessing out of bounds */
+                    L_WARNING("n = 16; reducing\n", procName);
+                    n = 15;
+                    sum -= histo16[n];
+                }
                 k = 16 * n;  /* starting value in fine histo */
                 for (m = 0; m < 16; m++) {
                     sum += histo[k];
                     if (sum > rankloc) {
-                        SET_DATA_BYTE(lined, j, k); 
+                        SET_DATA_BYTE(lined, j, k);
                         break;
                     }
                     k++;
@@ -408,8 +451,8 @@ PIX       *pixt, *pixd;
     }
 
     pixDestroy(&pixt);
-    FREE(histo);
-    FREE(histo16);
+    LEPT_FREE(histo);
+    LEPT_FREE(histo16);
     return pixd;
 }
 
@@ -418,11 +461,11 @@ PIX       *pixt, *pixd;
  *                             Median filter                            *
  *----------------------------------------------------------------------*/
 /*!
- *  pixMedianFilter()
+ * \brief   pixMedianFilter()
  *
- *      Input:  pixs (8 or 32 bpp; no colormap)
- *              wf, hf  (width and height of filter; each is >= 1)
- *      Return: pixd (of median values), or null on error
+ * \param[in]    pixs     8 or 32 bpp; no colormap
+ * \param[in]    wf, hf   width and height of filter; each is >= 1
+ * \return  pixd of median values, or NULL on error
  */
 PIX  *
 pixMedianFilter(PIX     *pixs,
@@ -437,3 +480,65 @@ pixMedianFilter(PIX     *pixs,
 }
 
 
+/*----------------------------------------------------------------------*
+ *                Rank filter (accelerated with downscaling)            *
+ *----------------------------------------------------------------------*/
+/*!
+ * \brief   pixRankFilterWithScaling()
+ *
+ * \param[in]    pixs          8 or 32 bpp; no colormap
+ * \param[in]    wf, hf        width and height of filter; each is >= 1
+ * \param[in]    rank          in [0.0 ... 1.0]
+ * \param[in]    scalefactor   scale factor; must be >= 0.2 and <= 0.7
+ * \return  pixd of rank values, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) This is a convenience function that downscales, does
+ *          the rank filtering, and upscales.  Because the down-
+ *          and up-scaling functions are very fast compared to
+ *          rank filtering, the time it takes is reduced from that
+ *          for the simple rank filtering operation by approximately
+ *          the square of the scaling factor.
+ * </pre>
+ */
+PIX  *
+pixRankFilterWithScaling(PIX       *pixs,
+                         l_int32    wf,
+                         l_int32    hf,
+                         l_float32  rank,
+                         l_float32  scalefactor)
+{
+l_int32  w, h, d, wfs, hfs;
+PIX     *pix1, *pix2, *pixd;
+
+    PROCNAME("pixRankFilterWithScaling");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    if (pixGetColormap(pixs) != NULL)
+        return (PIX *)ERROR_PTR("pixs has colormap", procName, NULL);
+    d = pixGetDepth(pixs);
+    if (d != 8 && d != 32)
+        return (PIX *)ERROR_PTR("pixs not 8 or 32 bpp", procName, NULL);
+    if (wf < 1 || hf < 1)
+        return (PIX *)ERROR_PTR("wf < 1 || hf < 1", procName, NULL);
+    if (rank < 0.0 || rank > 1.0)
+        return (PIX *)ERROR_PTR("rank must be in [0.0, 1.0]", procName, NULL);
+    if (wf == 1 && hf == 1)   /* no-op */
+        return pixCopy(NULL, pixs);
+    if (scalefactor < 0.2 || scalefactor > 0.7) {
+        L_ERROR("invalid scale factor; no scaling used\n", procName);
+        return pixRankFilter(pixs, wf, hf, rank);
+    }
+
+    pix1 = pixScaleAreaMap(pixs, scalefactor, scalefactor);
+    wfs = L_MAX(1, (l_int32)(scalefactor * wf + 0.5));
+    hfs = L_MAX(1, (l_int32)(scalefactor * hf + 0.5));
+    pix2 = pixRankFilter(pix1, wfs, hfs, rank);
+    pixGetDimensions(pixs, &w, &h, NULL);
+    pixd = pixScaleToSize(pix2, w, h);
+    pixDestroy(&pix1);
+    pixDestroy(&pix2);
+    return pixd;
+}
