@@ -119,7 +119,7 @@
  *  must work on scaled images.
  *
  *  Here are the steps in doing recog training:
- *  A. Generate a BAR from any exising labeled templates
+ *  A. Generate a BAR from any existing labeled templates
  *    (1) Create a recog and add the templates, using recogAddSample().
  *        This stores the unscaled templates.
  *        [Note: this can be done in one step if the labeled templates are put
@@ -153,11 +153,15 @@
  *  (3) For each candidate, find the average correlation with other
  *      members of its class, and remove those that have a relatively
  *      low average correlation.  This is similar to (1), gives comparable
- *      results and becauses it does not use average templates, it requires
+ *      results and because it does not use average templates, it requires
  *      a bit more computation.
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include "allheaders.h"
 
@@ -174,15 +178,15 @@ static PIX *recogDisplayOutlier(L_RECOG *recog, l_int32 iclass, l_int32 jsamp,
      * in outlier removal functions, and that use template set size
      * to decide if the set of templates (before outliers are removed)
      * is valid.  Values are set to accept most sets of sample templates. */
-static const l_int32    DEFAULT_MIN_SET_SIZE = 1;  /* minimum number of
+static const l_int32    DefaultMinSetSize = 1;  /* minimum number of
                                        samples for a valid class */
-static const l_float32  DEFAULT_MIN_SET_FRACT = 0.4;  /* minimum fraction
+static const l_float32  DefaultMinSetFract = 0.4;  /* minimum fraction
                                of classes required for a valid recog */
 
     /* Defaults in pixaRemoveOutliers1() and pixaRemoveOutliers2() */
-static const l_float32  DEFAULT_MIN_SCORE = 0.75; /* keep everything above */
-static const l_int32    DEFAULT_MIN_TARGET = 3;  /* to be kept if possible */
-static const l_float32  LOWER_SCORE_THRESHOLD = 0.5;  /* templates can be
+static const l_float32  DefaultMinScore = 0.75; /* keep everything above */
+static const l_int32    DefaultMinTarget = 3;  /* to be kept if possible */
+static const l_float32  LowerScoreThreshold = 0.5;  /* templates can be
                  * kept down to this score to if needed to retain the
                  * desired minimum number of templates */
 
@@ -193,11 +197,11 @@ static const l_float32  LOWER_SCORE_THRESHOLD = 0.5;  /* templates can be
 /*!
  * \brief   recogTrainLabeled()
  *
- * \param[in]    recog in training mode
- * \param[in]    pixs if depth > 1, will be thresholded to 1 bpp
- * \param[in]    box [optional] cropping box
- * \param[in]    text [optional] if null, use text field in pix
- * \param[in]    debug 1 to display images of samples not captured
+ * \param[in]    recog     in training mode
+ * \param[in]    pixs      if depth > 1, will be thresholded to 1 bpp
+ * \param[in]    box       [optional] cropping box
+ * \param[in]    text      [optional] if null, use text field in pix
+ * \param[in]    debug     1 to display images of samples not captured
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -208,7 +212,7 @@ static const l_float32  LOWER_SCORE_THRESHOLD = 0.5;  /* templates can be
  *          of the character image.
  * </pre>
  */
-l_int32
+l_ok
 recogTrainLabeled(L_RECOG  *recog,
                   PIX      *pixs,
                   BOX      *box,
@@ -257,7 +261,7 @@ PIX     *pix;
  *          of one character where the charval is inserted into the pix.
  * </pre>
  */
-l_int32
+l_ok
 recogProcessLabeled(L_RECOG  *recog,
                     PIX      *pixs,
                     BOX      *box,
@@ -348,7 +352,7 @@ PIX     *pix1, *pix2, *pix3, *pix4;
  *          the pixaa_u array is extended by adding a pixa to hold them.
  * </pre>
  */
-l_int32
+l_ok
 recogAddSample(L_RECOG  *recog,
                PIX      *pix,
                l_int32   debug)
@@ -376,7 +380,7 @@ PIXAA   *paa;
     }
 
         /* Determine the class array index.  Check if the class
-         * alreadly exists, and if not, add it. */
+         * already exists, and if not, add it. */
     if (recogGetClassIndex(recog, charint, text, &index) == 1) {
             /* New class must be added */
         npa = pixaaGetCount(paa, NULL);
@@ -462,7 +466,7 @@ PIX     *pix1, *pix2;
 /*!
  * \brief   recogAverageSamples()
  *
- * \param[in]   precog      addr of existing recog; may be destroyed
+ * \param[in]   precog    addr of existing recog; may be destroyed
  * \param[in]   debug
  * \return  0 on success, 1 on failure
  *
@@ -644,11 +648,11 @@ L_RECOG   *recog;
 /*!
  * \brief   pixaAccumulateSamples()
  *
- * \param[in]    pixa of samples from the same class, 1 bpp
- * \param[in]    pta [optional] of centroids of the samples
- * \param[out]   ppixd accumulated samples, 8 bpp
- * \param[out]   px [optional] average x coordinate of centroids
- * \param[out]   py [optional] average y coordinate of centroids
+ * \param[in]    pixa     of samples from the same class, 1 bpp
+ * \param[in]    pta      [optional] of centroids of the samples
+ * \param[out]   ppixd    accumulated samples, 8 bpp
+ * \param[out]   px       [optional] average x coordinate of centroids
+ * \param[out]   py       [optional] average y coordinate of centroids
  * \return  0 on success, 1 on failure
  *
  * <pre>
@@ -779,7 +783,7 @@ PTA       *ptac;
  *          unscaled and modified.
  * </pre>
  */
-l_int32
+l_ok
 recogTrainingFinished(L_RECOG  **precog,
                       l_int32    modifyflag,
                       l_int32    minsize,
@@ -922,8 +926,8 @@ NUMA      *na;
     if (!recog)
         return ERROR_INT("recog not defined", procName, 1);
 
-    minsize = (minsize < 0) ? DEFAULT_MIN_SET_SIZE : minsize;
-    minfract = (minfract < 0) ? DEFAULT_MIN_SET_FRACT : minfract;
+    minsize = (minsize < 0) ? DefaultMinSetSize : minsize;
+    minfract = (minfract < 0) ? DefaultMinSetFract : minfract;
     n = pixaaGetCount(recog->pixaa_u, &na);
     validsets = 0;
     for (i = 0, validsets = 0; i < n; i++) {
@@ -945,8 +949,8 @@ NUMA      *na;
  * \param[in]   setsize       size of character set (number of classes)
  * \param[in]   maxkeep       max number of templates to keep in a class
  * \param[in]   max_ht_ratio  max allowed height ratio (see below)
- * \param[out]  pna     [optional] debug output, giving the number in each
- *                      class after filtering; use NULL to skip
+ * \param[out]  pna           [optional] debug output, giving the number
+ *                            in each class after filtering; use NULL to skip
  * \return  pixa   filtered templates, or NULL on error
  *
  * <pre>
@@ -1032,14 +1036,14 @@ PIXAA     *paa;
 /*!
  * \brief   recogSortPixaByClass()
  *
- * \param[in]   pixa          labeled templates
- * \param[in]   setsize       size of character set (number of classes)
+ * \param[in]   pixa       labeled templates
+ * \param[in]   setsize    size of character set (number of classes)
  * \return  paa   pixaa where each pixa has templates for one class,
  *                or null on error
  */
 PIXAA *
-recogSortPixaByClass(PIXA      *pixa,
-                     l_int32    setsize)
+recogSortPixaByClass(PIXA    *pixa,
+                     l_int32  setsize)
 {
 PIXAA    *paa;
 L_RECOG  *recog;
@@ -1077,7 +1081,7 @@ L_RECOG  *recog;
  *          if it fails, the input recog is destroyed.
  * </pre>
  */
-l_int32
+l_ok
 recogRemoveOutliers1(L_RECOG  **precog,
                      l_float32  minscore,
                      l_int32    mintarget,
@@ -1142,9 +1146,9 @@ L_RECOG  *recog;
  *          we supplement a minimum score for retention with a score
  *          necessary to acquire the minimum target number of templates.
  *          To do this we are willing to use a lower threshold,
- *          LOWER_SCORE_THRESHOLD, on the score.  Consequently, with
+ *          LowerScoreThreshold, on the score.  Consequently, with
  *          poor quality templates, we may keep samples with a score
- *          less than %minscore, but never less than LOWER_SCORE_THRESHOLD.
+ *          less than %minscore, but never less than LowerScoreThreshold.
  *          And if the number of samples is less than %minsize, we do
  *          not use any.
  *      (3) This is meant to be used on a BAR, where the templates all
@@ -1179,12 +1183,12 @@ L_RECOG   *recog;
         return (PIXA *)ERROR_PTR("pixas not defined", procName, NULL);
     minscore = L_MIN(minscore, 1.0);
     if (minscore <= 0.0)
-        minscore = DEFAULT_MIN_SCORE;
+        minscore = DefaultMinScore;
     mintarget = L_MIN(mintarget, 3);
     if (mintarget <= 0)
-        mintarget = DEFAULT_MIN_TARGET;
+        mintarget = DefaultMinTarget;
     if (minsize < 0)
-        minsize = DEFAULT_MIN_SET_SIZE;
+        minsize = DefaultMinSetSize;
 
         /* Make a special height-scaled recognizer with average templates */
     debug = (ppixsave || ppixrem) ? 1 : 0;
@@ -1222,19 +1226,19 @@ L_RECOG   *recog;
                                       recog->sumtab, &score);
             numaAddNumber(nascore, score);
             if (debug && score == 0.0)  /* typ. large size difference */
-                fprintf(stderr, "Got 0 score for i = %d, j = %d\n", i, j);
+                lept_stderr("Got 0 score for i = %d, j = %d\n", i, j);
             pixDestroy(&pix2);
         }
         pixDestroy(&pix1);
 
-            /* Find the rankscore, corresonding to the 1.0 - minfract.
+            /* Find the rankscore, corresponding to the 1.0 - minfract.
              * To attempt to maintain the minfract of templates, use as a
              * cutoff the minimum of minscore and the rank score.  However,
              * no template is saved with an actual score less than
              * that at least one template is kept. */
         minfract = (l_float32)mintarget / (l_float32)n;
         numaGetRankValue(nascore, 1.0 - minfract, NULL, 0, &rankscore);
-        threshscore = L_MAX(LOWER_SCORE_THRESHOLD,
+        threshscore = L_MAX(LowerScoreThreshold,
                             L_MIN(minscore, rankscore));
         if (debug) {
             L_INFO("minscore = %4.2f, rankscore = %4.2f, threshscore = %4.2f\n",
@@ -1280,11 +1284,11 @@ L_RECOG   *recog;
 /*!
  * \brief   recogRemoveOutliers2()
  *
- * \param[in]   precog       addr of recog with unscaled labeled templates
- * \param[in]   minscore     keep everything with at least this score
- * \param[in]   minsize      minimum number of samples required for a class
- * \param[out]  ppixsave     [optional debug] saved templates, with scores
- * \param[out]  ppixrem      [optional debug] removed templates, with scores
+ * \param[in]   precog      addr of recog with unscaled labeled templates
+ * \param[in]   minscore    keep everything with at least this score
+ * \param[in]   minsize     minimum number of samples required for a class
+ * \param[out]  ppixsave    [optional debug] saved templates, with scores
+ * \param[out]  ppixrem     [optional debug] removed templates, with scores
  * \return  0 if OK, 1 on error.
  *
  * <pre>
@@ -1295,7 +1299,7 @@ L_RECOG   *recog;
  *          if it fails, the input recog is destroyed.
  * </pre>
  */
-l_int32
+l_ok
 recogRemoveOutliers2(L_RECOG  **precog,
                      l_float32  minscore,
                      l_int32    minsize,
@@ -1381,9 +1385,9 @@ L_RECOG   *recog;
         return (PIXA *)ERROR_PTR("pixas not defined", procName, NULL);
     minscore = L_MIN(minscore, 1.0);
     if (minscore <= 0.0)
-        minscore = DEFAULT_MIN_SCORE;
+        minscore = DefaultMinScore;
     if (minsize < 0)
-        minsize = DEFAULT_MIN_SET_SIZE;
+        minsize = DefaultMinSetSize;
 
         /* Make a special height-scaled recognizer with average templates */
     debug = (ppixsave || ppixrem) ? 1 : 0;
@@ -1458,11 +1462,11 @@ L_RECOG   *recog;
 /*!
  * \brief   recogTrainFromBoot()
  *
- * \param[in]    recogboot  labeled boot recognizer
- * \param[in]    pixas      set of unlabeled input characters
- * \param[in]    minscore   min score for accepting the example; e.g., 0.75
- * \param[in]    threshold  for binarization, if needed
- * \param[in]    debug      1 for debug output saved to recogboot; 0 otherwise
+ * \param[in]    recogboot   labeled boot recognizer
+ * \param[in]    pixas       set of unlabeled input characters
+ * \param[in]    minscore    min score for accepting the example; e.g., 0.75
+ * \param[in]    threshold   for binarization, if needed
+ * \param[in]    debug       1 for debug output saved to recogboot; 0 otherwise
  * \return  pixad   labeled version of input pixas, trained on a BSR,
  *                  or NULL on error
  *
@@ -1573,10 +1577,10 @@ PIXA      *pixa1, *pixa2, *pixa3, *pixad;
 /*!
  * \brief   recogPadDigitTrainingSet()
  *
- * \param[in/out]   precog   trained; if padding is needed, it is replaced
- *                           by a a new padded recog
- * \param[in]       scaleh   must be > 0; suggest ~40.
- * \param[in]       linew    use 0 for original scanned images
+ * \param[in,out]   precog    trained; if padding is needed, it is replaced
+ *                            by a a new padded recog
+ * \param[in]       scaleh    must be > 0; suggest ~40.
+ * \param[in]       linew     use 0 for original scanned images
  * \return       0 if OK, 1 on error
  *
  * <pre>
@@ -1588,7 +1592,7 @@ PIXA      *pixa1, *pixa2, *pixa3, *pixad;
  *          %scaleh and %linew.
  * </pre>
  */
-l_int32
+l_ok
 recogPadDigitTrainingSet(L_RECOG  **precog,
                          l_int32    scaleh,
                          l_int32    linew)
@@ -1783,7 +1787,7 @@ PIXA    *pixa1, *pixa2;
         return (PIXA *)ERROR_PTR("boot charset not available", procName, NULL);
 
         /* Make boot recog templates */
-    pixa1 = recogMakeBootDigitTemplates(0);
+    pixa1 = recogMakeBootDigitTemplates(0, 0);
     n = pixaGetCount(pixa1);
 
         /* Extract the unscaled templates from %recog */
@@ -1812,7 +1816,7 @@ PIXA    *pixa1, *pixa2;
 /*!
  * \brief   recogCharsetAvailable()
  *
- * \param[in]    type of charset for padding
+ * \param[in]    type     of charset for padding
  * \return  1 if available; 0 if not.
  */
 static l_int32
@@ -1850,30 +1854,35 @@ l_int32  ret;
 /*!
  * \brief   recogMakeBootDigitRecog()
  *
- * \param[in]    scaleh   scale all heights to this; typ. use 40
- * \param[in]    linew    normalized line width; typ. use 5; 0 to skip
- * \param[in]    maxyshift from nominal centroid alignment; typically 0 or 1
- * \param[in]    debug  1 for showing templates; 0 otherwise
+ * \param[in]    nsamp       number of samples of each digit; or 0
+ * \param[in]    scaleh      scale all heights to this; typ. use 40
+ * \param[in]    linew       normalized line width; typ. use 5; 0 to skip
+ * \param[in]    maxyshift   from nominal centroid alignment; typically 0 or 1
+ * \param[in]    debug       1 for showing templates; 0 otherwise
  * \return  recog, or NULL on error
  *
  * <pre>
  * Notes:
  *     (1) This takes a set of pre-computed, labeled pixa of single
- *         digits, and generates a recognizer where the character templates
- *         that will be used are derived from the boot-generated pixa:
- *         - extending by replicating the set with different widths,
- *           keeping the height the same
+ *         digits, and generates a recognizer from them.
+ *         The templates used in the recognizer can be modified by:
  *         - scaling (isotropically to fixed height)
- *         - optionally generating a skeleton and thickening so that
- *           all strokes have the same width.
+ *         - generating a skeleton and thickening so that all strokes
+ *           have the same width.
  *     (2) The resulting templates are scaled versions of either the
  *         input bitmaps or images with fixed line widths.  To use the
  *         input bitmaps, set %linew = 0; otherwise, set %linew to the
  *         desired line width.
+ *     (3) If %nsamp == 0, this uses and extends the output from
+ *         three boot generators:
+ *            l_bootnum_gen1, l_bootnum_gen2, l_bootnum_gen3.
+ *         Otherwise, it uses exactly %nsamp templates of each digit,
+ *         extracted by l_bootnum_gen4.
  * </pre>
  */
 L_RECOG  *
-recogMakeBootDigitRecog(l_int32  scaleh,
+recogMakeBootDigitRecog(l_int32  nsamp,
+                        l_int32  scaleh,
                         l_int32  linew,
                         l_int32  maxyshift,
                         l_int32  debug)
@@ -1883,7 +1892,7 @@ PIXA     *pixa;
 L_RECOG  *recog;
 
         /* Get the templates, extended by horizontal scaling */
-    pixa = recogMakeBootDigitTemplates(debug);
+    pixa = recogMakeBootDigitTemplates(nsamp, debug);
 
         /* Make the boot recog; recogModifyTemplate() will scale the
          * templates and optionally turn them into strokes of fixed width. */
@@ -1899,7 +1908,8 @@ L_RECOG  *recog;
 /*!
  * \brief   recogMakeBootDigitTemplates()
  *
- * \param[in]    debug  1 for display of templates
+ * \param[in]    nsamp     number of samples of each digit; or 0
+ * \param[in]    debug     1 for display of templates
  * \return  pixa   of templates; or NULL on error
  *
  * <pre>
@@ -1908,12 +1918,25 @@ L_RECOG  *recog;
  * </pre>
  */
 PIXA  *
-recogMakeBootDigitTemplates(l_int32  debug)
+recogMakeBootDigitTemplates(l_int32  nsamp,
+                            l_int32  debug)
 {
-NUMA  *na;
+NUMA  *na1;
 PIX   *pix1, *pix2, *pix3;
 PIXA  *pixa1, *pixa2, *pixa3;
 
+    if (nsamp > 0) {
+        pixa1 = l_bootnum_gen4(nsamp);
+        if (debug) {
+            pix1 = pixaDisplayTiledWithText(pixa1, 1500, 1.0, 10,
+                                            2, 6, 0xff000000);
+            pixDisplay(pix1, 0, 0);
+            pixDestroy(&pix1);
+        }
+        return pixa1;
+    }
+
+        /* Else, generate from 3 pixa */
     pixa1 = l_bootnum_gen1();
     pixa2 = l_bootnum_gen2();
     pixa3 = l_bootnum_gen3();
@@ -1934,14 +1957,14 @@ PIXA  *pixa1, *pixa2, *pixa3;
     pixaDestroy(&pixa3);
 
         /* Extend by horizontal scaling */
-    na = numaCreate(4);
-    numaAddNumber(na, 0.9);
-    numaAddNumber(na, 1.1);
-    numaAddNumber(na, 1.2);
-    pixa2 = pixaExtendByScaling(pixa1, na, L_HORIZ, 1);
+    na1 = numaCreate(4);
+    numaAddNumber(na1, 0.9);
+    numaAddNumber(na1, 1.1);
+    numaAddNumber(na1, 1.2);
+    pixa2 = pixaExtendByScaling(pixa1, na1, L_HORIZ, 1);
 
     pixaDestroy(&pixa1);
-    numaDestroy(&na);
+    numaDestroy(&na1);
     return pixa2;
 }
 
@@ -1952,13 +1975,13 @@ PIXA  *pixa1, *pixa2, *pixa3;
 /*!
  * \brief   recogShowContent()
  *
- * \param[in]    fp file  stream
+ * \param[in]    fp       file stream
  * \param[in]    recog
  * \param[in]    index    for naming of output files of template images
- * \param[in]    display  1 for showing template images, 0 otherwise
+ * \param[in]    display  1 for showing template images; 0 otherwise
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 recogShowContent(FILE     *fp,
                  L_RECOG  *recog,
                  l_int32   index,
@@ -2006,16 +2029,16 @@ NUMA    *na;
 
     if (display) {
         lept_mkdir("lept/recog");
-        pix = pixaaDisplayByPixa(recog->pixaa_u, 20, 20, 1000);
+        pix = pixaaDisplayByPixa(recog->pixaa_u, 50, 1.0, 20, 20, 0);
         snprintf(buf, sizeof(buf), "/tmp/lept/recog/templates_u.%d.png", index);
-        pixWrite(buf, pix, IFF_PNG);
+        pixWriteDebug(buf, pix, IFF_PNG);
         pixDisplay(pix, 0, 200 * index);
         pixDestroy(&pix);
         if (recog->train_done) {
-            pix = pixaaDisplayByPixa(recog->pixaa, 20, 20, 1000);
+            pix = pixaaDisplayByPixa(recog->pixaa, 50, 1.0, 20, 20, 0);
             snprintf(buf, sizeof(buf),
                      "/tmp/lept/recog/templates.%d.png", index);
-            pixWrite(buf, pix, IFF_PNG);
+            pixWriteDebug(buf, pix, IFF_PNG);
             pixDisplay(pix, 800, 200 * index);
             pixDestroy(&pix);
         }
@@ -2041,7 +2064,7 @@ NUMA    *na;
  *      (3) Destroys the recog if the averaging function finds any bad classes.
  * </pre>
  */
-l_int32
+l_ok
 recogDebugAverages(L_RECOG  **precog,
                    l_int32    debug)
 {
@@ -2085,7 +2108,7 @@ L_RECOG   *recog;
             rchExtract(recog->rch, &index, &score, NULL, NULL, NULL,
                        NULL, NULL);
             if (debug >= 2)
-                fprintf(stderr, "index = %d, score = %7.3f\n", index, score);
+                lept_stderr("index = %d, score = %7.3f\n", index, score);
             pix3 = pixAddBorder(pix2, 2, 1);
             pixaAddPix(pixa, pix3, L_INSERT);
             pixDestroy(&pix1);
@@ -2094,10 +2117,11 @@ L_RECOG   *recog;
         pixaaAddPixa(paa2, pixa, L_INSERT);
         pixaDestroy(&pixat);
     }
-    recog->pixdb_ave = pixaaDisplayByPixa(paa2, 20, 20, 2500);
+    recog->pixdb_ave = pixaaDisplayByPixa(paa2, 50, 1.0, 20, 20, 0);
     if (debug % 2) {
         lept_mkdir("lept/recog");
-        pixWrite("/tmp/lept/recog/templ_match.png", recog->pixdb_ave, IFF_PNG);
+        pixWriteDebug("/tmp/lept/recog/templ_match.png", recog->pixdb_ave,
+                      IFF_PNG);
         pixDisplay(recog->pixdb_ave, 100, 100);
     }
 
@@ -2131,11 +2155,11 @@ PIXA      *pixat, *pixadb;
     if (!recog)
         return ERROR_INT("recog not defined", procName, 1);
 
-    fprintf(stderr, "min/max width_u = (%d,%d); min/max height_u = (%d,%d)\n",
-            recog->minwidth_u, recog->maxwidth_u,
-            recog->minheight_u, recog->maxheight_u);
-    fprintf(stderr, "min splitw = %d, max splith = %d\n",
-            recog->min_splitw, recog->max_splith);
+    lept_stderr("min/max width_u = (%d,%d); min/max height_u = (%d,%d)\n",
+                recog->minwidth_u, recog->maxwidth_u,
+                recog->minheight_u, recog->maxheight_u);
+    lept_stderr("min splitw = %d, max splith = %d\n",
+                recog->min_splitw, recog->max_splith);
 
     pixaDestroy(&recog->pixadb_ave);
 
@@ -2290,9 +2314,10 @@ PIXA  *pixa;
  * \brief   recogShowMatchesInRange()
  *
  * \param[in]    recog
- * \param[in]    pixa of 1 bpp images to match
- * \param[in]    minscore, maxscore range to include output
- * \param[in]    display to display the result
+ * \param[in]    pixa        of 1 bpp images to match
+ * \param[in]    minscore    min score to include output
+ * \param[in]    maxscore    max score to include output
+ * \param[in]    display     1 to display the result
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -2306,12 +2331,12 @@ PIXA  *pixa;
  *          to filter a range of scores.
  * </pre>
  */
-l_int32
-recogShowMatchesInRange(L_RECOG     *recog,
-                        PIXA        *pixa,
-                        l_float32    minscore,
-                        l_float32    maxscore,
-                        l_int32      display)
+l_ok
+recogShowMatchesInRange(L_RECOG   *recog,
+                        PIXA      *pixa,
+                        l_float32  minscore,
+                        l_float32  maxscore,
+                        l_int32    display)
 {
 l_int32    i, n, index, depth;
 l_float32  score;
@@ -2378,11 +2403,11 @@ PIXA      *pixa1, *pixa2;
  * \brief   recogShowMatch()
  *
  * \param[in]    recog
- * \param[in]    pix1  input pix; several possibilities
- * \param[in]    pix2  [optional] matching template
- * \param[in]    box  [optional] region in pix1 for which pix2 matches
- * \param[in]    index  index of matching template; use -1 to disable printing
- * \param[in]    score  score of match
+ * \param[in]    pix1    input pix; several possibilities
+ * \param[in]    pix2    [optional] matching template
+ * \param[in]    box     [optional] region in pix1 for which pix2 matches
+ * \param[in]    index   index of matching template; use -1 to disable printing
+ * \param[in]    score   score of match
  * \return  pixd pair of images, showing input pix and best template,
  *                    optionally with matching information, or NULL on error.
  *
