@@ -1586,8 +1586,6 @@ l_uint32  *lines, *lined;
         }
 
     }
-
-    return;
 }
 
 
@@ -1612,10 +1610,8 @@ l_uint32  *tab;
 
     PROCNAME("makeSumTabSG2");
 
-    if ((tab = (l_uint32 *)LEPT_CALLOC(256, sizeof(l_uint32))) == NULL)
-        return (l_uint32 *)ERROR_PTR("tab not made", procName, NULL);
-
         /* Pack the four sums separately in four bytes */
+    tab = (l_uint32 *)LEPT_CALLOC(256, sizeof(l_uint32));
     for (i = 0; i < 256; i++) {
         tab[i] = (sum[i & 0x3] | sum[(i >> 2) & 0x3] << 8 |
                   sum[(i >> 4) & 0x3] << 16 | sum[(i >> 6) & 0x3] << 24);
@@ -1643,8 +1639,7 @@ l_uint8  *tab;
 
     PROCNAME("makeValTabSG2");
 
-    if ((tab = (l_uint8 *)LEPT_CALLOC(5, sizeof(l_uint8))) == NULL)
-        return (l_uint8 *)ERROR_PTR("tab not made", procName, NULL);
+    tab = (l_uint8 *)LEPT_CALLOC(5, sizeof(l_uint8));
     for (i = 0; i < 5; i++)
         tab[i] = 255 - (i * 255) / 4;
     return tab;
@@ -1748,8 +1743,6 @@ l_uint32  *lines, *lined;
             SET_DATA_BYTE(lined, j + 7, valtab[GET_DATA_BYTE(&sum, 3)]);
         }
     }
-
-    return;
 }
 
 
@@ -1776,10 +1769,8 @@ l_uint32  *tab;
 
     PROCNAME("makeSumTabSG3");
 
-    if ((tab = (l_uint32 *)LEPT_CALLOC(64, sizeof(l_uint32))) == NULL)
-        return (l_uint32 *)ERROR_PTR("tab not made", procName, NULL);
-
         /* Pack the two sums separately in two bytes */
+    tab = (l_uint32 *)LEPT_CALLOC(64, sizeof(l_uint32));
     for (i = 0; i < 64; i++) {
         tab[i] = (sum[i & 0x07]) | (sum[(i >> 3) & 0x07] << 8);
     }
@@ -1795,7 +1786,7 @@ l_uint32  *tab;
  *      (1) Returns an 8 bit value for the sum of ON pixels
  *          in a 3x3 square, according to
  *               val = 255 - (255 * sum)/9
- *          where sum is in set {0, ... ,9}
+ *          where sum is in [0,...,9]
  * </pre>
  */
 static l_uint8 *
@@ -1806,8 +1797,7 @@ l_uint8  *tab;
 
     PROCNAME("makeValTabSG3");
 
-    if ((tab = (l_uint8 *)LEPT_CALLOC(10, sizeof(l_uint8))) == NULL)
-        return (l_uint8 *)ERROR_PTR("tab not made", procName, NULL);
+    tab = (l_uint8 *)LEPT_CALLOC(10, sizeof(l_uint8));
     for (i = 0; i < 10; i++)
         tab[i] = 0xff - (i * 255) / 9;
     return tab;
@@ -1876,8 +1866,6 @@ l_uint32  *lines, *lined;
             SET_DATA_BYTE(lined, j + 1, valtab[GET_DATA_BYTE(&sum, 3)]);
         }
     }
-
-    return;
 }
 
 
@@ -1902,10 +1890,8 @@ l_uint32  *tab;
 
     PROCNAME("makeSumTabSG4");
 
-    if ((tab = (l_uint32 *)LEPT_CALLOC(256, sizeof(l_uint32))) == NULL)
-        return (l_uint32 *)ERROR_PTR("tab not made", procName, NULL);
-
         /* Pack the two sums separately in two bytes */
+    tab = (l_uint32 *)LEPT_CALLOC(256, sizeof(l_uint32));
     for (i = 0; i < 256; i++) {
         tab[i] = (sum[i & 0xf]) | (sum[(i >> 4) & 0xf] << 8);
     }
@@ -1921,7 +1907,7 @@ l_uint32  *tab;
  *      (1) Returns an 8 bit value for the sum of ON pixels
  *          in a 4x4 square, according to
  *              val = 255 - (255 * sum)/16
- *          where sum is in set {0, ... ,16}
+ *          where sum is in [0,...,16]
  * </pre>
  */
 static l_uint8 *
@@ -1932,8 +1918,7 @@ l_uint8  *tab;
 
     PROCNAME("makeValTabSG4");
 
-    if ((tab = (l_uint8 *)LEPT_CALLOC(17, sizeof(l_uint8))) == NULL)
-        return (l_uint8 *)ERROR_PTR("tab not made", procName, NULL);
+    tab = (l_uint8 *)LEPT_CALLOC(17, sizeof(l_uint8));
     for (i = 0; i < 17; i++)
         tab[i] = 0xff - (i * 255) / 16;
     return tab;
@@ -2058,7 +2043,6 @@ l_uint32  *lines, *lined;
             SET_DATA_BYTE(lined, j + 3, valtab[GET_DATA_BYTE(&sum, 3)]);
         }
     }
-    return;
 }
 
 
@@ -2070,7 +2054,7 @@ l_uint32  *lines, *lined;
  *      (1) Returns an 8 bit value for the sum of ON pixels
  *          in a 6x6 square, according to
  *              val = 255 - (255 * sum)/36
- *          where sum is in set {0, ... ,36}
+ *          where sum is in [0,...,36]
  * </pre>
  */
 static l_uint8 *
@@ -2081,8 +2065,7 @@ l_uint8  *tab;
 
     PROCNAME("makeValTabSG6");
 
-    if ((tab = (l_uint8 *)LEPT_CALLOC(37, sizeof(l_uint8))) == NULL)
-        return (l_uint8 *)ERROR_PTR("tab not made", procName, NULL);
+    tab = (l_uint8 *)LEPT_CALLOC(37, sizeof(l_uint8));
     for (i = 0; i < 37; i++)
         tab[i] = 0xff - (i * 255) / 36;
     return tab;
@@ -2154,8 +2137,6 @@ l_uint32  *lines, *lined;
             SET_DATA_BYTE(lined, j, valtab[sum]);
         }
     }
-
-    return;
 }
 
 
@@ -2167,7 +2148,7 @@ l_uint32  *lines, *lined;
  *      (1) Returns an 8 bit value for the sum of ON pixels
  *          in an 8x8 square, according to
  *              val = 255 - (255 * sum)/64
- *          where sum is in set {0, ... ,64}
+ *          where sum is in [0,...,64]
  * </pre>
  */
 static l_uint8 *
@@ -2178,8 +2159,7 @@ l_uint8  *tab;
 
     PROCNAME("makeValTabSG8");
 
-    if ((tab = (l_uint8 *)LEPT_CALLOC(65, sizeof(l_uint8))) == NULL)
-        return (l_uint8 *)ERROR_PTR("tab not made", procName, NULL);
+    tab = (l_uint8 *)LEPT_CALLOC(65, sizeof(l_uint8));
     for (i = 0; i < 65; i++)
         tab[i] = 0xff - (i * 255) / 64;
     return tab;
@@ -2271,8 +2251,6 @@ l_uint32  *lines, *lined;
             SET_DATA_BYTE(lined, j, 255 - sum);
         }
     }
-
-    return;
 }
 
 
