@@ -29,6 +29,7 @@
  * <pre>
  *
  *      Top-level deskew interfaces
+ *          PIX       *pixDeskewBoth()
  *          PIX       *pixDeskew()
  *          PIX       *pixFindSkewAndDeskew()
  *          PIX       *pixDeskewGeneral()
@@ -94,39 +95,43 @@
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <math.h>
 #include "allheaders.h"
 
     /* Default sweep angle parameters for pixFindSkew() */
-static const l_float32  DEFAULT_SWEEP_RANGE = 7.;    /* degrees */
-static const l_float32  DEFAULT_SWEEP_DELTA = 1.;    /* degrees */
+static const l_float32  DefaultSweepRange = 7.0;   /* degrees */
+static const l_float32  DefaultSweepDelta = 1.0;   /* degrees */
 
     /* Default final angle difference parameter for binary
      * search in pixFindSkew().  The expected accuracy is
      * not better than the inverse image width in pixels,
      * say, 1/2000 radians, or about 0.03 degrees. */
-static const l_float32  DEFAULT_MINBS_DELTA = 0.01;  /* degrees */
+static const l_float32  DefaultMinbsDelta = 0.01;  /* degrees */
 
     /* Default scale factors for pixFindSkew() */
-static const l_int32  DEFAULT_SWEEP_REDUCTION = 4;  /* sweep part; 4 is good */
-static const l_int32  DEFAULT_BS_REDUCTION = 2;  /* binary search part */
+static const l_int32  DefaultSweepReduction = 4;  /* sweep part; 4 is good */
+static const l_int32  DefaultBsReduction = 2;  /* binary search part */
 
     /* Minimum angle for deskewing in pixDeskew() */
-static const l_float32  MIN_DESKEW_ANGLE = 0.1;  /* degree */
+static const l_float32  MinDeskewAngle = 0.1;  /* degree */
 
     /* Minimum allowed confidence (ratio) for deskewing in pixDeskew() */
-static const l_float32  MIN_ALLOWED_CONFIDENCE = 3.0;
+static const l_float32  MinAllowedConfidence = 3.0;
 
     /* Minimum allowed maxscore to give nonzero confidence */
-static const l_int32  MIN_VALID_MAXSCORE = 10000;
+static const l_int32  MinValidMaxscore = 10000;
 
     /* Constant setting threshold for minimum allowed minscore
      * to give nonzero confidence; multiply this constant by
      *  (height * width^2) */
-static const l_float32  MINSCORE_THRESHOLD_CONSTANT = 0.000002;
+static const l_float32  MinscoreThreshFactor = 0.000002;
 
     /* Default binarization threshold value */
-static const l_int32  DEFAULT_BINARY_THRESHOLD = 130;
+static const l_int32  DefaultBinaryThreshold = 130;
 
 #ifndef  NO_CONSOLE_IO
 #define  DEBUG_PRINT_SCORES     0
@@ -145,9 +150,9 @@ static const l_int32  DEFAULT_BINARY_THRESHOLD = 130;
 /*!
  * \brief   pixDeskewBoth()
  *
- * \param[in]    pixs any depth
- * \param[in]    redsearch for binary search: reduction factor = 1, 2 or 4;
- *                         use 0 for default
+ * \param[in]    pixs         any depth
+ * \param[in]    redsearch    for binary search: reduction factor = 1, 2 or 4;
+ *                            use 0 for default
  * \return  pixd deskewed pix, or NULL on error
  *
  * <pre>
@@ -169,7 +174,7 @@ PIX  *pix1, *pix2, *pix3, *pix4;
     if (!pixs)
         return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
     if (redsearch == 0)
-        redsearch = DEFAULT_BS_REDUCTION;
+        redsearch = DefaultBsReduction;
     else if (redsearch != 1 && redsearch != 2 && redsearch != 4)
         return (PIX *)ERROR_PTR("redsearch not in {1,2,4}", procName, NULL);
 
@@ -187,9 +192,9 @@ PIX  *pix1, *pix2, *pix3, *pix4;
 /*!
  * \brief   pixDeskew()
  *
- * \param[in]    pixs any depth
- * \param[in]    redsearch for binary search: reduction factor = 1, 2 or 4;
- *                         use 0 for default
+ * \param[in]    pixs        any depth
+ * \param[in]    redsearch   for binary search: reduction factor = 1, 2 or 4;
+ *                           use 0 for default
  * \return  pixd deskewed pix, or NULL on error
  *
  * <pre>
@@ -210,7 +215,7 @@ pixDeskew(PIX     *pixs,
     if (!pixs)
         return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
     if (redsearch == 0)
-        redsearch = DEFAULT_BS_REDUCTION;
+        redsearch = DefaultBsReduction;
     else if (redsearch != 1 && redsearch != 2 && redsearch != 4)
         return (PIX *)ERROR_PTR("redsearch not in {1,2,4}", procName, NULL);
 
@@ -221,13 +226,13 @@ pixDeskew(PIX     *pixs,
 /*!
  * \brief   pixFindSkewAndDeskew()
  *
- * \param[in]    pixs any depth
- * \param[in]    redsearch for binary search: reduction factor = 1, 2 or 4;
- *                         use 0 for default
- * \param[out]   pangle   [optional] angle required to deskew,
- *                        in degrees; use NULL to skip
- * \param[out]   pconf    [optional] conf value is ratio
- *                        of max/min scores; use NULL to skip
+ * \param[in]    pixs        any depth
+ * \param[in]    redsearch   for binary search: reduction factor = 1, 2 or 4;
+ *                           use 0 for default
+ * \param[out]   pangle      [optional] angle required to deskew,
+ *                           in degrees; use NULL to skip
+ * \param[out]   pconf       [optional] conf value is ratio
+ *                           of max/min scores; use NULL to skip
  * \return  pixd deskewed pix, or NULL on error
  *
  * <pre>
@@ -248,7 +253,7 @@ pixFindSkewAndDeskew(PIX        *pixs,
     if (!pixs)
         return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
     if (redsearch == 0)
-        redsearch = DEFAULT_BS_REDUCTION;
+        redsearch = DefaultBsReduction;
     else if (redsearch != 1 && redsearch != 2 && redsearch != 4)
         return (PIX *)ERROR_PTR("redsearch not in {1,2,4}", procName, NULL);
 
@@ -259,19 +264,19 @@ pixFindSkewAndDeskew(PIX        *pixs,
 /*!
  * \brief   pixDeskewGeneral()
  *
- * \param[in]    pixs  any depth
- * \param[in]    redsweep  for linear search: reduction factor = 1, 2 or 4;
- *                         use 0 for default
- * \param[in]    sweeprange in degrees in each direction from 0;
- *                          use 0.0 for default
- * \param[in]    sweepdelta in degrees; use 0.0 for default
- * \param[in]    redsearch  for binary search: reduction factor = 1, 2 or 4;
- *                          use 0 for default;
- * \param[in]    thresh for binarizing the image; use 0 for default
- * \param[out]   pangle   [optional] angle required to deskew,
- *                        in degrees; use NULL to skip
- * \param[out]   pconf    [optional] conf value is ratio
- *                        of max/min scores; use NULL to skip
+ * \param[in]    pixs         any depth
+ * \param[in]    redsweep     for linear search: reduction factor = 1, 2 or 4;
+ *                            use 0 for default
+ * \param[in]    sweeprange   in degrees in each direction from 0;
+ *                            use 0.0 for default
+ * \param[in]    sweepdelta   in degrees; use 0.0 for default
+ * \param[in]    redsearch    for binary search: reduction factor = 1, 2 or 4;
+ *                            use 0 for default;
+ * \param[in]    thresh       for binarizing the image; use 0 for default
+ * \param[out]   pangle       [optional] angle required to deskew,
+ *                            in degrees; use NULL to skip
+ * \param[out]   pconf        [optional] conf value is ratio
+ *                            of max/min scores; use NULL to skip
  * \return  pixd deskewed pix, or NULL on error
  *
  * <pre>
@@ -302,19 +307,19 @@ PIX       *pixb, *pixd;
     if (!pixs)
         return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
     if (redsweep == 0)
-        redsweep = DEFAULT_SWEEP_REDUCTION;
+        redsweep = DefaultSweepReduction;
     else if (redsweep != 1 && redsweep != 2 && redsweep != 4)
         return (PIX *)ERROR_PTR("redsweep not in {1,2,4}", procName, NULL);
     if (sweeprange == 0.0)
-        sweeprange = DEFAULT_SWEEP_RANGE;
+        sweeprange = DefaultSweepRange;
     if (sweepdelta == 0.0)
-        sweepdelta = DEFAULT_SWEEP_DELTA;
+        sweepdelta = DefaultSweepDelta;
     if (redsearch == 0)
-        redsearch = DEFAULT_BS_REDUCTION;
+        redsearch = DefaultBsReduction;
     else if (redsearch != 1 && redsearch != 2 && redsearch != 4)
         return (PIX *)ERROR_PTR("redsearch not in {1,2,4}", procName, NULL);
     if (thresh == 0)
-        thresh = DEFAULT_BINARY_THRESHOLD;
+        thresh = DefaultBinaryThreshold;
 
     deg2rad = 3.1415926535 / 180.;
 
@@ -328,14 +333,14 @@ PIX       *pixb, *pixd;
         /* Use the 1 bpp image to find the skew */
     ret = pixFindSkewSweepAndSearch(pixb, &angle, &conf, redsweep, redsearch,
                                     sweeprange, sweepdelta,
-                                    DEFAULT_MINBS_DELTA);
+                                    DefaultMinbsDelta);
     pixDestroy(&pixb);
     if (pangle) *pangle = angle;
     if (pconf) *pconf = conf;
     if (ret)
         return pixClone(pixs);
 
-    if (L_ABS(angle) < MIN_DESKEW_ANGLE || conf < MIN_ALLOWED_CONFIDENCE)
+    if (L_ABS(angle) < MinDeskewAngle || conf < MinAllowedConfidence)
         return pixClone(pixs);
 
     if ((pixd = pixRotate(pixs, deg2rad * angle, L_ROTATE_AREA_MAP,
@@ -352,10 +357,10 @@ PIX       *pixb, *pixd;
 /*!
  * \brief   pixFindSkew()
  *
- * \param[in]    pixs  1 bpp
+ * \param[in]    pixs     1 bpp
  * \param[out]   pangle   angle required to deskew, in degrees
  * \param[out]   pconf    confidence value is ratio max/min scores
- * \return  0 if OK, 1 on error or if angle measurment not valid
+ * \return  0 if OK, 1 on error or if angle measurement not valid
  *
  * <pre>
  * Notes:
@@ -366,7 +371,7 @@ PIX       *pixb, *pixd;
  *          Clockwise rotations are positive angles.
  * </pre>
  */
-l_int32
+l_ok
 pixFindSkew(PIX        *pixs,
             l_float32  *pangle,
             l_float32  *pconf)
@@ -383,11 +388,11 @@ pixFindSkew(PIX        *pixs,
         return ERROR_INT("pixs not 1 bpp", procName, 1);
 
     return pixFindSkewSweepAndSearch(pixs, pangle, pconf,
-                                     DEFAULT_SWEEP_REDUCTION,
-                                     DEFAULT_BS_REDUCTION,
-                                     DEFAULT_SWEEP_RANGE,
-                                     DEFAULT_SWEEP_DELTA,
-                                     DEFAULT_MINBS_DELTA);
+                                     DefaultSweepReduction,
+                                     DefaultBsReduction,
+                                     DefaultSweepRange,
+                                     DefaultSweepDelta,
+                                     DefaultMinbsDelta);
 }
 
 
@@ -397,12 +402,12 @@ pixFindSkew(PIX        *pixs,
 /*!
  * \brief   pixFindSkewSweep()
  *
- * \param[in]    pixs  1 bpp
- * \param[out]   pangle   angle required to deskew, in degrees
- * \param[in]    reduction  factor = 1, 2, 4 or 8
+ * \param[in]    pixs         1 bpp
+ * \param[out]   pangle       angle required to deskew, in degrees
+ * \param[in]    reduction    factor = 1, 2, 4 or 8
  * \param[in]    sweeprange   half the full range; assumed about 0; in degrees
  * \param[in]    sweepdelta   angle increment of sweep; in degrees
- * \return  0 if OK, 1 on error or if angle measurment not valid
+ * \return  0 if OK, 1 on error or if angle measurement not valid
  *
  * <pre>
  * Notes:
@@ -410,7 +415,7 @@ pixFindSkew(PIX        *pixs,
  *      (2) Caller must check the return value for validity of the result.
  * </pre>
  */
-l_int32
+l_ok
 pixFindSkewSweep(PIX        *pixs,
                  l_float32  *pangle,
                  l_int32     reduction,
@@ -529,16 +534,16 @@ cleanup:
 /*!
  * \brief   pixFindSkewSweepAndSearch()
  *
- * \param[in]    pixs  1 bpp
- * \param[out]   pangle   angle required to deskew; in degrees
- * \param[out]   pconf    confidence given by ratio of max/min score
- * \param[in]    redsweep  sweep reduction factor = 1, 2, 4 or 8
- * \param[in]    redsearch  binary search reduction factor = 1, 2, 4 or 8;
- *                          and must not exceed redsweep
+ * \param[in]    pixs         1 bpp
+ * \param[out]   pangle       angle required to deskew; in degrees
+ * \param[out]   pconf        confidence given by ratio of max/min score
+ * \param[in]    redsweep     sweep reduction factor = 1, 2, 4 or 8
+ * \param[in]    redsearch    binary search reduction factor = 1, 2, 4 or 8;
+ *                            and must not exceed redsweep
  * \param[in]    sweeprange   half the full range, assumed about 0; in degrees
  * \param[in]    sweepdelta   angle increment of sweep; in degrees
  * \param[in]    minbsdelta   min binary search increment angle; in degrees
- * \return  0 if OK, 1 on error or if angle measurment not valid
+ * \return  0 if OK, 1 on error or if angle measurement not valid
  *
  * <pre>
  * Notes:
@@ -554,7 +559,7 @@ cleanup:
  *      (4) See also notes in pixFindSkewSweepAndSearchScore()
  * </pre>
  */
-l_int32
+l_ok
 pixFindSkewSweepAndSearch(PIX        *pixs,
                           l_float32  *pangle,
                           l_float32  *pconf,
@@ -573,19 +578,19 @@ pixFindSkewSweepAndSearch(PIX        *pixs,
 /*!
  * \brief   pixFindSkewSweepAndSearchScore()
  *
- * \param[in]    pixs  1 bpp
- * \param[out]   pangle   angle required to deskew; in degrees
- * \param[out]   pconf    confidence given by ratio of max/min score
- * \param[out]   pendscore [optional] max score; use NULL to ignore
- * \param[in]    redsweep  sweep reduction factor = 1, 2, 4 or 8
- * \param[in]    redsearch  binary search reduction factor = 1, 2, 4 or 8;
- *                          and must not exceed redsweep
+ * \param[in]    pixs         1 bpp
+ * \param[out]   pangle       angle required to deskew; in degrees
+ * \param[out]   pconf        confidence given by ratio of max/min score
+ * \param[out]   pendscore    [optional] max score; use NULL to ignore
+ * \param[in]    redsweep     sweep reduction factor = 1, 2, 4 or 8
+ * \param[in]    redsearch    binary search reduction factor = 1, 2, 4 or 8;
+ *                            and must not exceed redsweep
  * \param[in]    sweepcenter  angle about which sweep is performed; in degrees
  * \param[in]    sweeprange   half the full range, taken about sweepcenter;
  *                            in degrees
  * \param[in]    sweepdelta   angle increment of sweep; in degrees
  * \param[in]    minbsdelta   min binary search increment angle; in degrees
- * \return  0 if OK, 1 on error or if angle measurment not valid
+ * \return  0 if OK, 1 on error or if angle measurement not valid
  *
  * <pre>
  * Notes:
@@ -593,8 +598,8 @@ pixFindSkewSweepAndSearch(PIX        *pixs,
  *          of equal angles, and then doing a binary search until convergence.
  *      (2) There are two built-in constants that determine if the
  *          returned confidence is nonzero:
- *            ~ MIN_VALID_MAXSCORE (minimum allowed maxscore)
- *            ~ MINSCORE_THRESHOLD_CONSTANT (determines minimum allowed
+ *            ~ MinValidMaxscore (minimum allowed maxscore)
+ *            ~ MinscoreThreshFactor (determines minimum allowed
  *                 minscore, by multiplying by (height * width^2)
  *          If either of these conditions is not satisfied, the returned
  *          confidence value will be zero.  The maxscore is optionally
@@ -608,7 +613,7 @@ pixFindSkewSweepAndSearch(PIX        *pixs,
  *      (4) By default, the shear is about the UL corner.
  * </pre>
  */
-l_int32
+l_ok
 pixFindSkewSweepAndSearchScore(PIX        *pixs,
                                l_float32  *pangle,
                                l_float32  *pconf,
@@ -631,20 +636,20 @@ pixFindSkewSweepAndSearchScore(PIX        *pixs,
 /*!
  * \brief   pixFindSkewSweepAndSearchScorePivot()
  *
- * \param[in]    pixs  1 bpp
- * \param[out]   pangle   angle required to deskew; in degrees
- * \param[out]   pconf    confidence given by ratio of max/min score
- * \param[out]   pendscore [optional] max score; use NULL to ignore
- * \param[in]    redsweep  sweep reduction factor = 1, 2, 4 or 8
- * \param[in]    redsearch  binary search reduction factor = 1, 2, 4 or 8;
- *                          and must not exceed redsweep
+ * \param[in]    pixs         1 bpp
+ * \param[out]   pangle       angle required to deskew; in degrees
+ * \param[out]   pconf        confidence given by ratio of max/min score
+ * \param[out]   pendscore    [optional] max score; use NULL to ignore
+ * \param[in]    redsweep     sweep reduction factor = 1, 2, 4 or 8
+ * \param[in]    redsearch    binary search reduction factor = 1, 2, 4 or 8;
+ *                            and must not exceed redsweep
  * \param[in]    sweepcenter  angle about which sweep is performed; in degrees
  * \param[in]    sweeprange   half the full range, taken about sweepcenter;
  *                            in degrees
  * \param[in]    sweepdelta   angle increment of sweep; in degrees
  * \param[in]    minbsdelta   min binary search increment angle; in degrees
- * \param[in]    pivot  L_SHEAR_ABOUT_CORNER, L_SHEAR_ABOUT_CENTER
- * \return  0 if OK, 1 on error or if angle measurment not valid
+ * \param[in]    pivot        L_SHEAR_ABOUT_CORNER, L_SHEAR_ABOUT_CENTER
+ * \return  0 if OK, 1 on error or if angle measurement not valid
  *
  * <pre>
  * Notes:
@@ -657,7 +662,7 @@ pixFindSkewSweepAndSearchScore(PIX        *pixs,
  *          loses too much of the image.
  * </pre>
  */
-l_int32
+l_ok
 pixFindSkewSweepAndSearchScorePivot(PIX        *pixs,
                                     l_float32  *pangle,
                                     l_float32  *pconf,
@@ -916,7 +921,7 @@ PIX       *pixsw, *pixsch, *pixt1, *pixt2;
     numaGetMin(nascore, &minscore, &minloc);
     width = pixGetWidth(pixsch);
     height = pixGetHeight(pixsch);
-    minthresh = MINSCORE_THRESHOLD_CONSTANT * width * width * height;
+    minthresh = MinscoreThreshFactor * width * width * height;
 
 #if  DEBUG_THRESHOLD
     L_INFO(" minthresh = %10.2f, minscore = %10.2f\n", procName,
@@ -933,13 +938,13 @@ PIX       *pixsw, *pixsch, *pixt1, *pixt2;
          * range or if maxscore is small */
     if ((centerangle > rangeleft + 2 * sweeprange - sweepdelta) ||
         (centerangle < rangeleft + sweepdelta) ||
-        (maxscore < MIN_VALID_MAXSCORE))
+        (maxscore < MinValidMaxscore))
         *pconf = 0.0;
 
 #if  DEBUG_PRINT_BINARY
-    fprintf(stderr, "Binary search: angle = %7.3f, score ratio = %6.2f\n",
+    lept_stderr("Binary search: angle = %7.3f, score ratio = %6.2f\n",
             *pangle, *pconf);
-    fprintf(stderr, "               max score = %8.0f\n", maxscore);
+    lept_stderr("               max score = %8.0f\n", maxscore);
 #endif  /* DEBUG_PRINT_BINARY */
 
 #if  DEBUG_PLOT_SCORES
@@ -972,24 +977,25 @@ cleanup:
  *    Search over arbitrary range of angles in orthogonal directions   *
  *---------------------------------------------------------------------*/
 /*
- *   pixFindSkewOrthogonalRange()
+ * \brief   pixFindSkewOrthogonalRange()
  *
- *      Input:  pixs  (1 bpp)
- *              &angle  (<return> angle required to deskew; in degrees cw)
- *              &conf   (<return> confidence given by ratio of max/min score)
- *              redsweep  (sweep reduction factor = 1, 2, 4 or 8)
- *              redsearch  (binary search reduction factor = 1, 2, 4 or 8;
- *                          and must not exceed redsweep)
- *              sweeprange  (half the full range in each orthogonal
- *                           direction, taken about 0, in degrees)
- *              sweepdelta   (angle increment of sweep; in degrees)
- *              minbsdelta   (min binary search increment angle; in degrees)
- *              confprior  (amount by which confidence of 90 degree rotated
- *                          result is reduced when comparing with unrotated
- *                          confidence value)
- *      Return: 0 if OK, 1 on error or if angle measurment not valid
+ * \param[in]    pixs         1 bpp
+ * \param[out]   pangle       angle required to deskew; in degrees cw
+ * \param[out]   pconf        confidence given by ratio of max/min score
+ * \param[in]    redsweep     sweep reduction factor = 1, 2, 4 or 8
+ * \param[in]    redsearch    binary search reduction factor = 1, 2, 4 or 8;
+ *                            and must not exceed redsweep
+ * \param[in]    sweeprange   half the full range in each orthogonal
+ *                            direction, taken about 0, in degrees
+ * \param[in]    sweepdelta   angle increment of sweep; in degrees
+ * \param[in]    minbsdelta   min binary search increment angle; in degrees
+ * \param[in]    confprior    amount by which confidence of 90 degree rotated
+ *                            result is reduced when comparing with unrotated
+ *                            confidence value
+ * \return   0 if OK, 1 on error or if angle measurement not valid
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This searches for the skew angle, first in the range
  *          [-sweeprange, sweeprange], and then in
  *          [90 - sweeprange, 90 + sweeprange], with angles measured
@@ -1027,6 +1033,7 @@ cleanup:
  *          the landscape signal is accidentally larger than the
  *          portrait signal.  To do this use a positive value of
  *          %confprior; say 1.5.
+ * </pre>
  */
 l_int32
 pixFindSkewOrthogonalRange(PIX        *pixs,
@@ -1071,11 +1078,11 @@ PIX       *pixr;
     }
 
 #if  DEBUG_PRINT_ORTH
-    fprintf(stderr, " About 0:  angle1 = %7.3f, conf1 = %7.3f, score1 = %f\n",
+    lept_stderr(" About 0:  angle1 = %7.3f, conf1 = %7.3f, score1 = %f\n",
             angle1, conf1, score1);
-    fprintf(stderr, " About 90: angle2 = %7.3f, conf2 = %7.3f, score2 = %f\n",
+    lept_stderr(" About 90: angle2 = %7.3f, conf2 = %7.3f, score2 = %f\n",
             angle2, conf2, score2);
-    fprintf(stderr, " Final:    angle = %7.3f, conf = %7.3f\n", *pangle, *pconf);
+    lept_stderr(" Final:    angle = %7.3f, conf = %7.3f\n", *pangle, *pconf);
 #endif  /* DEBUG_PRINT_ORTH */
 
     return 0;
@@ -1090,7 +1097,7 @@ PIX       *pixr;
  * \brief   pixFindDifferentialSquareSum()
  *
  * \param[in]    pixs
- * \param[out]   psum  result
+ * \param[out]   psum    result
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1101,7 +1108,7 @@ PIX       *pixr;
  *           ~ not more than 5% of the image width
  * </pre>
  */
-l_int32
+l_ok
 pixFindDifferentialSquareSum(PIX        *pixs,
                              l_float32  *psum)
 {
@@ -1155,11 +1162,11 @@ NUMA      *na;
  * \brief   pixFindNormalizedSquareSum()
  *
  * \param[in]    pixs
- * \param[out]   phratio [optional] ratio of normalized horiz square sum
- *                       to result if the pixel distribution were uniform
- * \param[out]   pvratio [optional] ratio of normalized vert square sum
- *                       to result if the pixel distribution were uniform
- * \param[out]   pfract  [optional] ratio of fg pixels to total pixels
+ * \param[out]   phratio   [optional] ratio of normalized horiz square sum
+ *                         to result if the pixel distribution were uniform
+ * \param[out]   pvratio   [optional] ratio of normalized vert square sum
+ *                         to result if the pixel distribution were uniform
+ * \param[out]   pfract    [optional] ratio of fg pixels to total pixels
  * \return  0 if OK, 1 on error or if there are no fg pixels
  *
  * <pre>
@@ -1174,7 +1181,7 @@ NUMA      *na;
  *      (2) If there are no fg pixels, hratio and vratio are returned as 0.0.
  * </pre>
  */
-l_int32
+l_ok
 pixFindNormalizedSquareSum(PIX        *pixs,
                            l_float32  *phratio,
                            l_float32  *pvratio,
