@@ -60,11 +60,8 @@
  *     Change format for missing library
  *        void        changeFormatForMissingLib()
  *
- *     Deprecated pix output for debugging
- *        l_int32     pixDisplayWrite()  -- still used in tesseract 3.05
- *        l_int32     pixSaveTiled()
- *        l_int32     pixSaveTiledOutline()
- *        l_int32     pixSaveTiledWithText()
+ *     Nonfunctional stub of pix output for debugging
+ *        l_int32     pixDisplayWrite()
  *
  *  Supported file formats:
  *  (1) Writing is supported without any external libraries:
@@ -104,7 +101,7 @@ static l_int32  var_DISPLAY_PROG = L_DISPLAY_WITH_OPEN;  /* default */
 static l_int32  var_DISPLAY_PROG = L_DISPLAY_WITH_XZGV;  /* default */
 #endif  /* _WIN32 */
 
-static const l_int32  Bufsize = 512;
+#define Bufsize 512
 static const l_int32  MaxDisplayWidth = 1000;
 static const l_int32  MaxDisplayHeight = 800;
 static const l_int32  MaxSizeForPng = 200;
@@ -207,7 +204,7 @@ l_int32  prevq, newq;
 /*----------------------------------------------------------------------*
  *    Set global variable LeptDebugOK for writing to named temp files   *
  *----------------------------------------------------------------------*/
-l_int32 LeptDebugOK = 0;  /* default value */
+LEPT_DLL l_int32 LeptDebugOK = 0;  /* default value */
 /*!
  * \brief   setLeptDebugOK()
  *
@@ -448,7 +445,7 @@ pixWriteStream(FILE    *fp,
         return pixWriteStreamGif(fp, pix);
 
     case IFF_JP2:
-        return pixWriteStreamJp2k(fp, pix, 34, 4, 0, 0);
+        return pixWriteStreamJp2k(fp, pix, 34, 4, L_JP2_CODEC, 0, 0);
 
     case IFF_WEBP:
         return pixWriteStreamWebP(fp, pix, 80, 0);
@@ -745,6 +742,7 @@ l_int32  ret;
     case IFF_TIFF_G4:        /* compressed, binary only */
     case IFF_TIFF_LZW:       /* compressed, all depths */
     case IFF_TIFF_ZIP:       /* compressed, all depths */
+    case IFF_TIFF_JPEG:      /* compressed, 8 bpp gray or 32 bpp rgb */
         ret = pixWriteMemTiff(pdata, psize, pix, format);
         break;
 
@@ -1081,6 +1079,7 @@ char            fullpath[_MAX_PATH];
  * <pre>
  * Notes:
  *      (1) If %addlabel == 0, %location and %textcolor are ignored.
+ *      (2) To make an array of color squares, use pixDisplayColorArray().
  * </pre>
  */
 PIX *
@@ -1188,346 +1187,23 @@ changeFormatForMissingLib(l_int32  *pformat)
 /*!
  * \brief   pixDisplayWrite()
  *
- * \param[in]    pix 1, 2, 4, 8, 16, 32 bpp
- * \param[in]    reduction -1 to reset/erase; 0 to disable;
- *                         otherwise this is a reduction factor
- * \return  0 if OK; 1 on error
+ * \param[in]    pix
+ * \param[in]    reduction
+ * \return  1 (error)
  *
  * <pre>
  * Notes:
- *      (0) Deprecated.
- *      (1) This is a simple interface for writing a set of files.
- *      (2) This uses jpeg output for pix that are 32 bpp or 8 bpp
- *          without a colormap; otherwise, it uses png.
- *      (3) To erase any previously written files in the output directory:
- *             pixDisplayWrite(NULL, -1);
- *      (4) If reduction > 1 and depth == 1, this does a scale-to-gray
- *          reduction.
- *      (5) This function uses a static internal variable to number
- *          output files written by a single process.  Behavior
- *          with a shared library may be unpredictable.
- *      (6) For 16 bpp, this displays the full dynamic range with log scale.
- *          Alternative image transforms to generate 8 bpp pix are:
- *             pix8 = pixMaxDynamicRange(pixt, L_LINEAR_SCALE);
- *             pix8 = pixConvert16To8(pixt, L_LS_BYTE);  // low order byte
- *             pix8 = pixConvert16To8(pixt, L_MS_BYTE);  // high order byte
+ *      As of 1.80, this is a non-functional stub.
  * </pre>
  */
 l_ok
 pixDisplayWrite(PIX     *pixs,
                 l_int32  reduction)
 {
-char            buf[Bufsize];
-char           *fname;
-l_float32       scale;
-PIX            *pix1, *pix2;
-static l_int32  index = 0;  /* caution: not .so or thread safe */
-
-    PROCNAME("pixDisplayWrite");
-
-    lept_stderr("\n######################################################"
-                "\n                     Notice:\n"
-                "  pixDisplayWrite() has been deprecated in leptonica \n"
-                "  since version 1.74. It will become a non-functioning\n"
-                "  stub in 1.80.\n"
-                "######################################################"
+    lept_stderr("\n########################################################\n"
+                "  pixDisplayWrite() was last used in tesseract 3.04,"
+                "  in Feb 2016.  As of 1.80, it is a non-functional stub\n"
+                "########################################################"
                 "\n\n\n");
-
-    if (reduction == 0) return 0;
-    if (reduction < 0) {  /* initialize */
-        lept_rmdir("lept/display");
-        index = 0;
-        return 0;
-    }
-    if (!pixs)
-        return ERROR_INT("pixs not defined", procName, 1);
-    if (index == 0)
-        lept_mkdir("lept/display");
-    index++;
-
-    if (reduction == 1) {
-        pix1 = pixClone(pixs);
-    } else {
-        scale = 1. / (l_float32)reduction;
-        if (pixGetDepth(pixs) == 1)
-            pix1 = pixScaleToGray(pixs, scale);
-        else
-            pix1 = pixScale(pixs, scale, scale);
-    }
-
-    if (pixGetDepth(pix1) == 16) {
-        pix2 = pixMaxDynamicRange(pix1, L_LOG_SCALE);
-        snprintf(buf, Bufsize, "file.%03d.png", index);
-        fname = pathJoin("/tmp/lept/display", buf);
-        pixWrite(fname, pix2, IFF_PNG);
-        pixDestroy(&pix2);
-    } else if (pixGetDepth(pix1) < 8 || pixGetColormap(pix1)) {
-        snprintf(buf, Bufsize, "file.%03d.png", index);
-        fname = pathJoin("/tmp/lept/display", buf);
-        pixWrite(fname, pix1, IFF_PNG);
-    } else {
-        snprintf(buf, Bufsize, "file.%03d.jpg", index);
-        fname = pathJoin("/tmp/lept/display", buf);
-        pixWrite(fname, pix1, IFF_JFIF_JPEG);
-    }
-    LEPT_FREE(fname);
-    pixDestroy(&pix1);
-    return 0;
+    return 1;
 }
-
-
-/*!
- * \brief   pixSaveTiled()
- *
- * \param[in]    pixs 1, 2, 4, 8, 32 bpp
- * \param[in]    pixa the pix are accumulated here
- * \param[in]    scalefactor 0.0 to disable; otherwise this is a scale factor
- * \param[in]    newrow 0 if placed on the same row as previous; 1 otherwise
- * \param[in]    space horizontal and vertical spacing, in pixels
- * \param[in]    dp depth of pixa; 8 or 32 bpp; only used on first call
- * \return  0 if OK, 1 on error.
- */
-l_ok
-pixSaveTiled(PIX       *pixs,
-             PIXA      *pixa,
-             l_float32  scalefactor,
-             l_int32    newrow,
-             l_int32    space,
-             l_int32    dp)
-{
-    lept_stderr("\n######################################################"
-                "\n                     Notice:\n"
-                "  pixSaveTiled() has been deprecated in leptonica \n"
-                "  since version 1.78. It will be removed in 1.80.\n"
-                "######################################################"
-                "\n\n\n");
-
-        /* Save without an outline */
-    return pixSaveTiledOutline(pixs, pixa, scalefactor, newrow, space, 0, dp);
-}
-
-
-/*!
- * \brief   pixSaveTiledOutline()
- *
- * \param[in]    pixs 1, 2, 4, 8, 32 bpp
- * \param[in]    pixa the pix are accumulated here
- * \param[in]    scalefactor 0.0 to disable; otherwise this is a scale factor
- * \param[in]    newrow 0 if placed on the same row as previous; 1 otherwise
- * \param[in]    space horizontal and vertical spacing, in pixels
- * \param[in]    linewidth width of added outline for image; 0 for no outline
- * \param[in]    dp depth of pixa; 8 or 32 bpp; only used on first call
- * \return  0 if OK, 1 on error.
- *
- * <pre>
- * Notes:
- *      (1) Before calling this function for the first time, use
- *          pixaCreate() to make the %pixa that will accumulate the pix.
- *          This is passed in each time pixSaveTiled() is called.
- *      (2) %scalefactor scales the input image.  After scaling and
- *          possible depth conversion, the image is saved in the input
- *          pixa, along with a box that specifies the location to
- *          place it when tiled later.  Disable saving the pix by
- *          setting %scalefactor == 0.0.
- *      (3) %newrow and %space specify the location of the new pix
- *          with respect to the last one(s) that were entered.
- *      (4) %dp specifies the depth at which all pix are saved.  It can
- *          be only 8 or 32 bpp.  Any colormap is removed.  This is only
- *          used at the first invocation.
- *      (5) This function uses two variables from call to call.
- *          If they were static, the function would not be .so or thread
- *          safe, and furthermore, there would be interference with two or
- *          more pixa accumulating images at a time.  Consequently,
- *          we use the first pix in the pixa to store and obtain both
- *          the depth and the current position of the bottom (one pixel
- *          below the lowest image raster line when laid out using
- *          the boxa).  The bottom variable is stored in the input format
- *          field, which is the only field available for storing an int.
- * </pre>
- */
-l_ok
-pixSaveTiledOutline(PIX       *pixs,
-                    PIXA      *pixa,
-                    l_float32  scalefactor,
-                    l_int32    newrow,
-                    l_int32    space,
-                    l_int32    linewidth,
-                    l_int32    dp)
-{
-l_int32  n, top, left, bx, by, bw, w, h, depth, bottom;
-BOX     *box;
-PIX     *pix1, *pix2, *pix3, *pix4;
-
-    PROCNAME("pixSaveTiledOutline");
-
-    lept_stderr("\n######################################################"
-                "\n                     Notice:\n"
-                "  pixSaveTiledOutline() has been deprecated in leptonica \n"
-                "  since version 1.78. It will be removed in 1.80.\n"
-                "######################################################"
-                "\n\n\n");
-
-    if (scalefactor == 0.0) return 0;
-
-    if (!pixs)
-        return ERROR_INT("pixs not defined", procName, 1);
-    if (!pixa)
-        return ERROR_INT("pixa not defined", procName, 1);
-
-    n = pixaGetCount(pixa);
-    if (n == 0) {
-        bottom = 0;
-        if (dp != 8 && dp != 32) {
-            L_WARNING("dp not 8 or 32 bpp; using 32\n", procName);
-            depth = 32;
-        } else {
-            depth = dp;
-        }
-    } else {  /* extract the depth and bottom params from the first pix */
-        pix1 = pixaGetPix(pixa, 0, L_CLONE);
-        depth = pixGetDepth(pix1);
-        bottom = pixGetInputFormat(pix1);  /* not typical usage! */
-        pixDestroy(&pix1);
-    }
-
-        /* Remove colormap if it exists; otherwise a copy.  This
-         * guarantees that pix4 is not a clone of pixs. */
-    pix1 = pixRemoveColormapGeneral(pixs, REMOVE_CMAP_BASED_ON_SRC, L_COPY);
-
-        /* Scale and convert to output depth */
-    if (scalefactor == 1.0) {
-        pix2 = pixClone(pix1);
-    } else if (scalefactor > 1.0) {
-        pix2 = pixScale(pix1, scalefactor, scalefactor);
-    } else {  /* scalefactor < 1.0) */
-        if (pixGetDepth(pix1) == 1)
-            pix2 = pixScaleToGray(pix1, scalefactor);
-        else
-            pix2 = pixScale(pix1, scalefactor, scalefactor);
-    }
-    pixDestroy(&pix1);
-    if (depth == 8)
-        pix3 = pixConvertTo8(pix2, 0);
-    else
-        pix3 = pixConvertTo32(pix2);
-    pixDestroy(&pix2);
-
-        /* Add black outline */
-    if (linewidth > 0)
-        pix4 = pixAddBorder(pix3, linewidth, 0);
-    else
-        pix4 = pixClone(pix3);
-    pixDestroy(&pix3);
-
-        /* Find position of current pix (UL corner plus size) */
-    if (n == 0) {
-        top = 0;
-        left = 0;
-    } else if (newrow == 1) {
-        top = bottom + space;
-        left = 0;
-    } else {  /* n > 0 */
-        pixaGetBoxGeometry(pixa, n - 1, &bx, &by, &bw, NULL);
-        top = by;
-        left = bx + bw + space;
-    }
-
-    pixGetDimensions(pix4, &w, &h, NULL);
-    bottom = L_MAX(bottom, top + h);
-    box = boxCreate(left, top, w, h);
-    pixaAddPix(pixa, pix4, L_INSERT);
-    pixaAddBox(pixa, box, L_INSERT);
-
-        /* Save the new bottom value */
-    pix1 = pixaGetPix(pixa, 0, L_CLONE);
-    pixSetInputFormat(pix1, bottom);  /* not typical usage! */
-    pixDestroy(&pix1);
-    return 0;
-}
-
-
-/*!
- * \brief   pixSaveTiledWithText()
- *
- * \param[in]    pixs 1, 2, 4, 8, 32 bpp
- * \param[in]    pixa the pix are accumulated here; as 32 bpp
- * \param[in]    outwidth in pixels; use 0 to disable entirely
- * \param[in]    newrow 1 to start a new row; 0 to go on same row as previous
- * \param[in]    space horizontal and vertical spacing, in pixels
- * \param[in]    linewidth width of added outline for image; 0 for no outline
- * \param[in]    bmf [optional] font struct
- * \param[in]    textstr [optional] text string to be added
- * \param[in]    val color to set the text
- * \param[in]    location L_ADD_ABOVE, L_ADD_AT_TOP, L_ADD_AT_BOT, L_ADD_BELOW
- * \return  0 if OK, 1 on error.
- *
- * <pre>
- * Notes:
- *      (1) Before calling this function for the first time, use
- *          pixaCreate() to make the %pixa that will accumulate the pix.
- *          This is passed in each time pixSaveTiled() is called.
- *      (2) %outwidth is the scaled width.  After scaling, the image is
- *          saved in the input pixa, along with a box that specifies
- *          the location to place it when tiled later.  Disable saving
- *          the pix by setting %outwidth == 0.
- *      (3) %newrow and %space specify the location of the new pix
- *          with respect to the last one(s) that were entered.
- *      (4) All pix are saved as 32 bpp RGB.
- *      (5) If both %bmf and %textstr are defined, this generates a pix
- *          with the additional text; otherwise, no text is written.
- *      (6) The text is written before scaling, so it is properly
- *          antialiased in the scaled pix.  However, if the pix on
- *          different calls have different widths, the size of the
- *          text will vary.
- *      (7) See pixSaveTiledOutline() for other implementation details.
- * </pre>
- */
-l_ok
-pixSaveTiledWithText(PIX         *pixs,
-                     PIXA        *pixa,
-                     l_int32      outwidth,
-                     l_int32      newrow,
-                     l_int32      space,
-                     l_int32      linewidth,
-                     L_BMF       *bmf,
-                     const char  *textstr,
-                     l_uint32     val,
-                     l_int32      location)
-{
-PIX  *pix1, *pix2, *pix3, *pix4;
-
-    PROCNAME("pixSaveTiledWithText");
-
-    lept_stderr("\n######################################################"
-                "\n                     Notice:\n"
-                "  pixSaveTiledWithText() has been deprecated in leptonica \n"
-                "  since version 1.78. It will be removed in 1.80.\n"
-                "######################################################"
-                "\n\n\n");
-
-    if (outwidth == 0) return 0;
-
-    if (!pixs)
-        return ERROR_INT("pixs not defined", procName, 1);
-    if (!pixa)
-        return ERROR_INT("pixa not defined", procName, 1);
-
-    pix1 = pixConvertTo32(pixs);
-    if (linewidth > 0)
-        pix2 = pixAddBorder(pix1, linewidth, 0);
-    else
-        pix2 = pixClone(pix1);
-    if (bmf && textstr)
-        pix3 = pixAddSingleTextblock(pix2, bmf, textstr, val, location, NULL);
-    else
-        pix3 = pixClone(pix2);
-    pix4 = pixScaleToSize(pix3, outwidth, 0);
-    pixSaveTiled(pix4, pixa, 1.0, newrow, space, 32);
-    pixDestroy(&pix1);
-    pixDestroy(&pix2);
-    pixDestroy(&pix3);
-    pixDestroy(&pix4);
-    return 0;
-}
-
-
